@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -27,13 +30,16 @@ namespace apex {
       private:
         static address_resolution * _instance;
         address_resolution(void) {
+#ifdef APEX_HAVE_BFD
           my_bfd_unit_handle = Apex_bfd_registerUnit();
+#endif
         };
         // copy constructor is private
         address_resolution(address_resolution const&);
         // assignment operator is private
         address_resolution& operator=(address_resolution const& a);
       public:
+        uintptr_t getPieOffset();
         static shared_mutex_type _bfd_mutex;
 
 
@@ -55,6 +61,7 @@ namespace apex {
           return _instance;
       }
       static void delete_instance() {
+          disable_memory_wrapper();
           delete(_instance);
       }
       ~address_resolution(void) {
@@ -70,13 +77,15 @@ namespace apex {
           delete node;
         }
         my_hash_table.clear();
+#ifdef APEX_HAVE_BFD
         Apex_delete_bfd_units();
+#endif
       }
       std::unordered_map<uintptr_t, my_hash_node*> my_hash_table;
       apex_bfd_handle_t my_bfd_unit_handle;
   };
 
-  std::string * lookup_address(uintptr_t ip, bool withFileInfo);
+  std::string * lookup_address(uintptr_t ip, bool withFileInfo, bool forceSourceInfo = false);
 
 }
 
