@@ -1,8 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
-//
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -25,10 +27,22 @@
 
 // include the C API
 #include "apex_policies.h"
+// include the simulated annealing class
+#include "simulated_annealing.hpp"
+// include the exhaustive class
+#include "exhaustive.hpp"
+// include the random class
+#include "random.hpp"
+// include the genetic_search class
+#include "genetic_search.hpp"
+// include the nelder_mead class
+#include "nelder_mead.hpp"
 
 enum class apex_param_type : int {NONE, LONG, DOUBLE, ENUM};
-enum class apex_ah_tuning_strategy : int {EXHAUSTIVE, RANDOM, NELDER_MEAD,
-PARALLEL_RANK_ORDER};
+enum class apex_ah_tuning_strategy : int {
+    EXHAUSTIVE, RANDOM, NELDER_MEAD, NELDER_MEAD_INTERNAL,
+    PARALLEL_RANK_ORDER, SIMULATED_ANNEALING,
+    APEX_EXHAUSTIVE, APEX_RANDOM, GENETIC_SEARCH, AUTOMATIC};
 
 struct apex_tuning_session;
 class apex_tuning_request;
@@ -49,6 +63,10 @@ class apex_param {
             return apex_param_type::NONE;
         };
 
+        virtual /*const*/ std::string get_init() const {
+            return "";
+        };
+
         friend apex_tuning_session_handle
         __setup_custom_tuning(apex_tuning_request & request);
         friend int
@@ -57,11 +75,22 @@ class apex_param {
         friend int
         __active_harmony_custom_setup(std::shared_ptr<apex_tuning_session>
         tuning_session, apex_tuning_request & request);
+        friend int __sa_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __exhaustive_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __random_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __genetic_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __nelder_mead_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
 };
 
 class apex_param_long : public apex_param {
     protected:
         std::shared_ptr<long> value;
+        const long init;
         const long min;
         const long max;
         const long step;
@@ -70,24 +99,39 @@ class apex_param_long : public apex_param {
         apex_param_long(const std::string & name, const long init_value,
                         const long min, const long max, const long step)
             : apex_param(name), value{std::make_shared<long>(init_value)},
-              min{min}, max{max}, step{step} {};
+              init{init_value}, min{min}, max{max}, step{step} {};
         virtual ~apex_param_long() {};
 
         /*const*/ long get_value() const {
             return *value;
         };
 
+        virtual /*const*/ std::string get_init() const {
+            return std::to_string(init);
+        };
+
         virtual /*const*/ apex_param_type get_type() const {
             return apex_param_type::LONG;
         };
         friend int
         __active_harmony_custom_setup(std::shared_ptr<apex_tuning_session>
         tuning_session, apex_tuning_request & request);
+        friend int __sa_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __exhaustive_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __random_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __genetic_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __nelder_mead_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
 };
 
 class apex_param_double : public apex_param {
     protected:
         std::shared_ptr<double> value;
+        const double init;
         const double min;
         const double max;
         const double step;
@@ -96,19 +140,33 @@ class apex_param_double : public apex_param {
         apex_param_double(const std::string & name, const double init_value,
                         const double min, const double max, const double step)
             : apex_param(name), value{std::make_shared<double>(init_value)},
-              min{min}, max{max}, step{step} {};
+              init{init_value}, min{min}, max{max}, step{step} {};
         virtual ~apex_param_double() {};
 
         /*const*/ double get_value() const {
             return *value;
         };
 
+        virtual /*const*/ std::string get_init() const {
+            return std::to_string(init);
+        };
+
         virtual /*const*/ apex_param_type get_type() const {
             return apex_param_type::DOUBLE;
         };
         friend int
         __active_harmony_custom_setup(std::shared_ptr<apex_tuning_session>
         tuning_session, apex_tuning_request & request);
+        friend int __sa_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __exhaustive_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __random_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __genetic_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __nelder_mead_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
 };
 
 class apex_param_enum : public apex_param {
@@ -129,12 +187,26 @@ class apex_param_enum : public apex_param {
             return std::string{*value};
         };
 
+        virtual /*const*/ std::string get_init() const {
+            return init_value;
+        };
+
         virtual /*const*/ apex_param_type get_type() const {
             return apex_param_type::ENUM;
         };
         friend int
         __active_harmony_custom_setup(std::shared_ptr<apex_tuning_session>
         tuning_session, apex_tuning_request & request);
+        friend int __sa_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __exhaustive_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __random_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __genetic_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __nelder_mead_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
 };
 
 
@@ -162,7 +234,7 @@ class apex_tuning_request {
         trigger{APEX_INVALID_EVENT},
             tuning_session_handle{0}, running{false},
             strategy{apex_ah_tuning_strategy::PARALLEL_RANK_ORDER},
-            radius(0.5), aggregation_times(1), aggregation_function("median")
+            radius(0.5), aggregation_times(3), aggregation_function("min")
             {};
         virtual ~apex_tuning_request()  {};
 
@@ -249,6 +321,18 @@ class apex_tuning_request {
             strategy = s;
         };
 
+        void set_radius(double r) {
+            radius = r;
+        };
+
+        void set_aggregation_times(size_t t) {
+            aggregation_times = t;
+        };
+
+        void set_aggregation_function(std::string f) {
+            aggregation_function = f;
+        };
+
         friend apex_tuning_session_handle
         __setup_custom_tuning(apex_tuning_request & request);
         friend int
@@ -257,7 +341,16 @@ class apex_tuning_request {
         friend int
         __active_harmony_custom_setup(std::shared_ptr<apex_tuning_session>
         tuning_session, apex_tuning_request & request);
-
+        friend int __sa_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __exhaustive_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __random_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __genetic_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
+        friend int __nelder_mead_setup(std::shared_ptr<apex_tuning_session>
+            tuning_session, apex_tuning_request & request);
 };
 
 
@@ -278,6 +371,16 @@ struct apex_tuning_session {
     std::atomic<bool> apex_energy_init{false};
     std::atomic<bool> apex_timer_init{false};
 
+    // if using simulated annealing, this is the request.
+    apex::simulated_annealing::SimulatedAnnealing sa_session;
+    // if using exhaustive, this is the request.
+    apex::exhaustive::Exhaustive exhaustive_session;
+    // if using exhaustive, this is the request.
+    apex::random::Random random_session;
+    // if using genetic, this is the request.
+    apex::genetic::GeneticSearch genetic_session;
+    // if using nelder mead, this is the request.
+    apex::nelder_mead::NelderMead nelder_mead_session;
     bool converged_message = false;
 
     // variables related to power throttling
