@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -58,6 +61,7 @@ public:
   void on_startup(startup_event_data &data) { APEX_UNUSED(data); };
   void on_dump(dump_event_data &data);
   void on_reset(task_identifier * id);
+  void on_pre_shutdown(void) { } ;
   void on_shutdown(shutdown_event_data &data);
   void on_new_node(node_event_data &data) { APEX_UNUSED(data); };
   void on_new_thread(new_thread_event_data &data);
