@@ -1,7 +1,10 @@
-//  Copyright (c) 2014 University of Oregon
-//
-//  Distributed under the Boost Software License, Version 1.0. (See accompanying
-//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+/*
+ * Copyright (c) 2014-2021 Kevin Huck
+ * Copyright (c) 2014-2021 University of Oregon
+ *
+ * Distributed under the Boost Software License, Version 1.0. (See accompanying
+ * file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+ */
 
 #pragma once
 
@@ -17,7 +20,7 @@
 #include <chrono>
 #include "apex_cxx_shared_lock.hpp"
 #include "profiler.hpp"
-#include "cuda_thread_node.hpp"
+#include "async_thread_node.hpp"
 
 namespace apex {
 
@@ -40,10 +43,13 @@ namespace apex {
         /* The global offset is referenced from the get_time static function,
          * so it needs to be static itself. */
         static uint64_t globalOffset;
+        /* We take an end timestamp before shutdown, because we don't
+         * want to confuse the user with a long trace time */
+        static uint64_t saved_end_timestamp;
         /* All OTF2 callback functions have to be declared static, so that they
          * can be registered with the OTF2 library */
         static OTF2_TimeStamp get_time( void ) {
-            uint64_t stamp = profiler::get_time_ns();
+            uint64_t stamp = profiler::now_ns();
             stamp = stamp - globalOffset;
             return stamp;
         }
@@ -187,11 +193,14 @@ namespace apex {
             uint64_t stamp, bool is_enter);
 #endif
         std::mutex _vthread_mutex;
-        std::map<cuda_thread_node, size_t> vthread_map;
+        std::map<base_thread_node, size_t> vthread_map;
         std::map<uint32_t, OTF2_EvtWriter*> vthread_evt_writer_map;
-        uint32_t make_vtid (uint32_t device, uint32_t context, uint32_t stream);
+        uint32_t make_vtid (base_thread_node &node);
         std::map<uint32_t,uint64_t> last_ts;
         uint64_t dropped;
+        int64_t synchronizeClocks(void);
+        int getCommRank(void);
+        int getCommSize(void);
     public:
         otf2_listener (void);
         //~otf2_listener (void) { shutdown_event_data data(my_saved_node_id,0);
@@ -213,6 +222,7 @@ namespace apex {
         void on_dump(dump_event_data &data);
         void on_reset(task_identifier * id)
             { APEX_UNUSED(id); };
+        void on_pre_shutdown(void);
         void on_shutdown(shutdown_event_data &data);
         void on_new_node(node_event_data &data);
         void on_new_thread(new_thread_event_data &data);
@@ -231,8 +241,10 @@ namespace apex {
             { APEX_UNUSED(data); };
         void on_send(message_event_data &data);
         void on_recv(message_event_data &data);
-        void on_async_event(uint32_t device, uint32_t context,
-            uint32_t stream, std::shared_ptr<profiler> &p);
+        void on_async_event(base_thread_node &node,
+            std::shared_ptr<profiler> &p);
+        void on_async_metric(base_thread_node &node,
+            std::shared_ptr<profiler> &p);
 
     };
 }
