@@ -1,7 +1,7 @@
 #include "lulesh.h"
 
 // If no MPI, then this whole file is stubbed out
-#if USE_MPI
+#if APEX_ENABLE_MPI
 
 #include <mpi.h>
 #include <string.h>
@@ -15,7 +15,7 @@
 
 /*
    There are coherence issues for packing and unpacking message
-   buffers.  Ideally, you would like a lot of threads to 
+   buffers.  Ideally, you would like a lot of threads to
    cooperate in the assembly/dissassembly of each message.
    To do that, each thread should really be operating in a
    different coherence zone.
