@@ -5,7 +5,7 @@
 #include <time.h>
 #include <stdint.h>
 #include "apex_api.hpp"
-#if defined(APEX_HAVE_MPI)
+#if defined(APEX_ENABLE_MPI)
 #include <mpi.h>
 #endif
 
@@ -76,7 +76,7 @@ void* someThread(void* tmp)
     /* Register this thread with APEX */
     int* tid = (int*)tmp;
     char name[32];
-    sprintf(name, "worker-thread %d", *tid);
+    snprintf(name, 32, "worker-thread %d", *tid);
 #ifndef __APPLE__
 #ifndef APEX_HAVE_OTF2
     pthread_barrier_wait(&barrier);
@@ -98,7 +98,7 @@ void* someThread(void* tmp)
 
 int main (int argc, char** argv) {
     /* initialize APEX */
-    #if defined(APEX_HAVE_MPI)
+    #if defined(APEX_ENABLE_MPI)
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
@@ -129,6 +129,9 @@ int main (int argc, char** argv) {
         tids[i] = i;
         pthread_create(&(thread[i]), NULL, someThread, &(tids[i]));
     }
+    for (int i = 0 ; i < num_iterations ; i++) {
+        innerLoop(&(tids[i]));
+    }
     /* wait for the threads to finish */
     for (uint32_t i = 0 ; i < test_numthreads ; i++) {
         pthread_join(thread[i], NULL);
@@ -150,7 +153,7 @@ int main (int argc, char** argv) {
     	}
   	}
   	apex::cleanup();
-    #if defined(APEX_HAVE_MPI)
+    #if defined(APEX_ENABLE_MPI)
     MPI_Finalize();
     #endif
     return 0;
