@@ -7,11 +7,11 @@ void* someThread(void* tmp)
 {
     int* tid = (int*)tmp;
     char name[32];
-    sprintf(name, "worker thread %d", *tid);
+    snprintf(name, 32, "worker thread %d", *tid);
     /* Register this thread with APEX */
     apex::register_thread(name);
     /* Start a timer */
-    apex::profiler* p = apex::start((apex_function_address)&someThread);
+    apex::profiler* p = apex::start("someThread (internal)");
     /* ... */
     /* do some computation */
     /* ... */
