@@ -2,6 +2,10 @@
 #define MPICH_IGNORE_CXX_SEEK
 #endif
 
+#ifndef MPIBULL_IGNORE_CXX_SEEK
+#define MPIBULL_IGNORE_CXX_SEEK
+#endif
+
 #include <mpi.h>
 #include <stdio.h>
 
