@@ -32,6 +32,7 @@
 #define TAU_ACTION_DUMP_PROFILES 1
 #define TAU_ACTION_DUMP_CALLPATHS 2
 #define TAU_ACTION_DUMP_BACKTRACES 3
+#define TAU_ACTION_RUN_CONTROLFILE 4
 
 #define TAU_PLUGIN_ENABLED 1
 
@@ -45,10 +46,12 @@
 
 #define TAU_TRACE_FORMAT_TAU 0
 #define TAU_TRACE_FORMAT_OTF2 1
+#define TAU_TRACE_FORMAT_PERFETTO 2
 
 #define TAU_EBS_RESOLUTION_FILE 0
 #define TAU_EBS_RESOLUTION_FUNCTION 1
 #define TAU_EBS_RESOLUTION_LINE 2
+#define TAU_EBS_RESOLUTION_FUNCTION_LINE 3
 
 #ifdef __cplusplus
 extern "C" {
@@ -77,6 +80,7 @@ extern "C" {
   int  TAUDECL TauEnv_get_current_timer_exit_params();
   int  TAUDECL TauEnv_get_track_message();
   int  TAUDECL TauEnv_get_lite_enabled();
+  int  TAUDECL TauEnv_get_anonymize_enabled();
   int  TAUDECL TauEnv_get_compensate();
   int  TAUDECL TauEnv_get_level_zero_enable_api_tracing();
 
@@ -92,6 +96,7 @@ extern "C" {
   int  TAUDECL TauEnv_set_ompt_resolve_address_eagerly(int value);
   int  TAUDECL TauEnv_get_ompt_support_level();
   int  TAUDECL TauEnv_set_ompt_support_level(int value);
+  int  TAUDECL TauEnv_get_ompt_force_finalize(void);
   int  TAUDECL TauEnv_get_signals_gdb();
   int  TAUDECL TauEnv_get_echo_backtrace();
   int  TAUDECL TauEnv_get_openmp_runtime_enabled();
@@ -118,6 +123,7 @@ extern "C" {
   int TAUDECL TauEnv_get_disable_instrumentation();
 
   const char* TAUDECL TauEnv_get_ebs_source();
+  const char* TAUDECL TauEnv_get_ebs_source_orig();
   void TAUDECL TauEnv_override_ebs_source(const char *newName);
   double      TAUDECL TauEnv_get_throttle_numcalls();
   double      TAUDECL TauEnv_get_throttle_percall();
@@ -132,7 +138,9 @@ extern "C" {
   const char* TAUDECL TauEnv_get_plugins();
   int TAUDECL TauEnv_get_plugins_enabled();
   int TAUDECL TauEnv_get_track_mpi_t_comm_metric_values();
+#ifndef TAU_WINDOWS
   const char  TAUDECL *TauEnv_get_mpi_t_comm_metric_values();
+#endif
   int TAUDECL TauEnv_get_set_node();
   const char* TAUDECL TauEnv_get_cupti_api();
   const char* TAUDECL TauEnv_get_cuda_device_name();
@@ -148,6 +156,9 @@ extern "C" {
   void  TAUDECL TauEnv_set_cudaTotalThreads(int value);
   int TAUDECL TauEnv_get_tauCuptiAvail();
   void TAUDECL TauEnv_set_tauCuptiAvail(int value);
+  int TAUDECL TauEnv_get_tauCuptiPC();
+  int TAUDECL TauEnv_get_tauCuptiPC_hwsize();
+  int TAUDECL TauEnv_get_tauCuptiPC_period();
   int  TAUDECL TauEnv_get_nodeNegOneSeen();
   void  TAUDECL TauEnv_set_nodeNegOneSeen(int value);
   int TAUDECL TauEnv_get_mic_offload();
@@ -156,6 +167,8 @@ extern "C" {
   const char*  TAUDECL TauEnv_get_profile_prefix();
   int  TAUDECL TauEnv_get_profile_format();
   int  TAUDECL TauEnv_get_merge_metadata();
+  int  TAUDECL TauEnv_get_disable_metadata();
+  const char* TAUDECL TauEnv_get_control_file();
   int  TAUDECL TauEnv_get_sigusr1_action();
 
   int TAUDECL TauEnv_get_memdbg();
@@ -177,6 +190,7 @@ extern "C" {
   size_t TAUDECL TauEnv_get_memdbg_alignment();
   int TAUDECL TauEnv_get_memdbg_zero_malloc();
   int TAUDECL TauEnv_get_memdbg_attempt_continue();
+  int TAUDECL TauEnv_get_env_memmgr_max_memblocks();
   int TAUDECL TauEnv_get_pthread_stack_size();
   int TAUDECL TauEnv_get_alfred_port();
   int TAUDECL TauEnv_get_papi_multiplexing();
@@ -188,6 +202,22 @@ extern "C" {
   const char * TAUDECL TauEnv_get_tau_exec_args();
   const char * TAUDECL TauEnv_get_tau_exec_path();
   int TAUDECL TauEnv_get_recycle_threads();
+  int TAUDECL TauEnv_get_rocsdk_enable();
+  int TAUDECL TauEnv_get_rocsdk_pcs_enable();
+  int TAUDECL TauEnv_get_l0_enable();
+  int TAUDECL TauEnv_get_l0_metrics_enable();
+  int TAUDECL TauEnv_get_l0_stall_sampling_enable();
+  int TAUDECL TauEnv_get_perfetto_compress(void);
+  int TAUDECL TauEnv_get_perfetto_merge(void);
+  void TAUDECL TauEnv_set_tracing(int tracing);
+
+  /*TAU enable/disable events
+  const char* TAUDECL TauEnv_get_runtime_disable_trigger(void); 
+  const char* TAUDECL TauEnv_get_runtime_disable_target(void);
+  const char* TAUDECL TauEnv_get_runtime_enable_trigger(void);
+  const char* TAUDECL TauEnv_get_runtime_enable_target(void);
+*/
+
 #ifdef __cplusplus
   void Tau_util_replaceStringInPlace(std::string& subject, const std::string& search,
                           const std::string& replace);
