@@ -43,8 +43,10 @@ extern "C" {
 #define dmesg(level, fmt, ...)
 
 bool LikwidLayer::likwidInitialized = false;
-ThreadValue * LikwidLayer::ThreadList[TAU_MAX_THREADS] = { 0 };
-
+vector<LikwidThreadValue *> & LikwidLayer::TheThreadList() {
+    static vector<LikwidThreadValue *> threadList;
+    return threadList;
+}
 //string LikwidLayer::eventString;//[] = "L2_LINES_IN_ALL:PMC0,L2_TRANS_L2_WB:PMC1";
 int* LikwidLayer::cpus;
 int num_cpus = 0;
@@ -86,7 +88,7 @@ int LikwidLayer::initializeLikwidLayer() //Tau_initialize_likwid_library(void)
             setenv("LIKWID_FORCE", "1", 1); // Overwrite already running counters because currently there are no stopCounters() or finalize() calls
             {
                 char foo[100];
-                int ret = snprintf(foo, 99, "%lu", getpid());
+                int ret = snprintf(foo, 99, "%u", getpid());
                 if (ret > 0)
                 {
                    foo[ret] = '\0';
@@ -98,7 +100,7 @@ int LikwidLayer::initializeLikwidLayer() //Tau_initialize_likwid_library(void)
 	    num_cpus = topo->activeHWThreads; // Store the number of CPUs to use it later in LikwidLayer::getAllCounters
 	    LikwidLayer::likwidInitialized = true; // we initialized it, so set this to true
     }
-
+    return 0;
 }
 
 extern "C" int Tau_is_thread_fake(int tid);
@@ -137,25 +139,25 @@ int LikwidLayer::addEvents(const char *estr) {
 int LikwidLayer::initializeThread(int tid) {
 	int rc;
 
-	if (tid >= TAU_MAX_THREADS) {
-		fprintf(stderr, "TAU: Exceeded max thread count of TAU_MAX_THREADS\n");
-		return -1;
-	}
+	//if (tid >= TAU_MAX_THREADS) {
+	//	fprintf(stderr, "TAU: Exceeded max thread count of TAU_MAX_THREADS\n");
+	//	return -1;
+	//}
 
-	if (!ThreadList[tid]) {
+	if (!GetThreadList(tid)) {
 		RtsLayer::LockDB();
-		if (!ThreadList[tid]) {
+		if (!GetThreadList(tid)) {
 			dmesg(1, "TAU: LIKWID: Initializing Thread Data for TID = %d\n", tid);
 
 			/* Task API does not have a real thread associated with it. It is fake */
 			if (Tau_is_thread_fake(tid) == 1)
 				tid = 0;
 
-			ThreadList[tid] = new ThreadValue;
-			ThreadList[tid]->ThreadID = tid;
+			SetThreadList(tid, new LikwidThreadValue);
+			GetThreadList(tid)->ThreadID = tid;
 
-			ThreadList[tid]->CounterValues = new long long[numCounters];
-			memset(ThreadList[tid]->CounterValues, 0,
+			GetThreadList(tid)->CounterValues = new long long[numCounters];
+			memset(GetThreadList(tid)->CounterValues, 0,
 					numCounters * sizeof(long long));
 
 		} /*if (!ThreadList[tid]) */
@@ -187,7 +189,7 @@ long long *LikwidLayer::getAllCounters(int tid, int *numValues) {
 		return NULL;
 	}
 
-	if (ThreadList[tid] == NULL) {
+	if (GetThreadList(tid) == NULL) {
 		if (initializeThread(tid)) {
 			return NULL;
 		}
@@ -211,13 +213,13 @@ long long *LikwidLayer::getAllCounters(int tid, int *numValues) {
 		//printf("Counter %d longsum %lld dblsum %f\n", comp, static_cast<long long>(dblsum), dblsum);
         //tmpCounters[comp] += static_cast<long long>(s);
 		//for (int j=0; j<numCounters; j++) {
-		ThreadList[tid]->CounterValues[comp] += static_cast<long long>(dblsum);
+		GetThreadList(tid)->CounterValues[comp] += static_cast<long long>(dblsum);
 		//printf("ThreadList[%d]->CounterValues[%d] = %lld\n", tid, comp, ThreadList[tid]->CounterValues[comp]);
 		//}
 
 	}
 
-	return ThreadList[tid]->CounterValues;
+	return GetThreadList(tid)->CounterValues;
 }
 
 
