@@ -22,7 +22,9 @@
 #include <Profile/TauSampling.h>
 #include <Profile/TauMetaDataMerge.h>
 #include <Profile/TauPluginInternals.h>
-
+#ifdef TAU_PERFETTO
+#include <Profile/TauTracePerfetto.h>
+#endif
 // Define DEBUG_PROF if you want to debug timers
 // #define DEBUG_PROF 1
 
@@ -44,6 +46,8 @@
 #include <limits.h>
 
 #include <string>
+#include <vector>
+//#include <mutex>
 
 #ifdef TAU_VAMPIRTRACE
 #include <Profile/TauVampirTrace.h>
@@ -80,14 +84,14 @@ extern "C" void esd_exit (elg_ui4 rid);
 double TauWindowsUsecD(void);
 #endif
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 #ifdef __GNUC__
 #include "cupti_version.h"
 #include "cupti_events.h"
 #include "cupti_metrics.h"
 #include <cuda_runtime_api.h>
 #endif //__GNUC__
-#endif //CUPTI
+#endif //TAU_CUPTI
 
 #ifdef TAU_ENABLE_ROCTRACER
 extern void Tau_roctracer_stop_tracing(void);
@@ -99,6 +103,11 @@ extern "C" void  __real_shmem_finalize() ;
 #endif /* TAU_SHMEM */
 extern "C" int Tau_get_usesSHMEM();
 
+
+#ifdef TAU_TRACK_IDLE_THREADS
+extern "C" bool Tau_check_Stopping_All_Threads();
+#endif /* TAU_TRACK_IDLE_THREADS */
+
 using namespace std;
 using namespace tau;
 
@@ -137,17 +146,93 @@ extern "C" void Tau_profile_exit_most_threads();
 extern "C" int TauCompensateInitialized(void);
 extern "C" void Tau_ompt_resolve_callsite(FunctionInfo &fi, char * resolved_address);
 
-#ifdef TAU_ENABLE_ROCM
+/*#ifdef TAU_ENABLE_ROCM
 extern void TauFlushRocmEventsIfNecessary(int thread_id);
-#endif /* TAU_ENABLE_ROCM */
+#endif*/ /* TAU_ENABLE_ROCM */
 
 x_uint64 Tau_get_firstTimeStamp();
+struct ProfilerData{
+    int profileWriteCount=0;
+    #ifdef TAU_TRACK_IDLE_THREADS
+        double TheLastTimeStamp[TAU_MAX_COUNTERS];
+    #endif /* TAU_TRACK_IDLE_THREADS */
+};
+struct ProfThreadList : vector<ProfilerData*>{
+    ProfThreadList (const ProfThreadList&) = delete;
+    ProfThreadList& operator= (const ProfThreadList&) = delete;
+    ProfThreadList(){
+         //printf("Creating ProfilerThreadList at %p\n", this);
+      }
+     virtual ~ProfThreadList(){
+         //printf("Destroying ProfilerThreadList at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+   };
+
+static ProfThreadList & TheProfilerThreadList(){
+	static ProfThreadList profThreads;
+	return profThreads;
+}
+//static std::lock_guard<std::mutex> guard(ProfilerVectorMutex);
+static std::mutex ProfilerVectorMutex;
+inline void checkProfilerVector(int tid){
+    //if(TheProfilerThreadList().size()<=tid){
+    //  static std::mutex ProfilerVectorMutex;
+    //  std::lock_guard<std::mutex> guard(ProfilerVectorMutex);
+	while(TheProfilerThreadList().size()<=tid){
+        //RtsLayer::LockDB();
+		TheProfilerThreadList().push_back(new ProfilerData());
+        //ProfilerThreadList.back()->profileWriteCount=0;
+        //RtsLayer::UnLockDB();
+	}
+   // }
+    //printf("Write count for tid: %d, post-check: %d\n",tid,ProfilerThreadList[tid]->profileWriteCount);
+}
+
+
+static thread_local int local_tid = RtsLayer::myThread();
+static thread_local ProfilerData* PD_cache=0;
+
+static inline ProfilerData& getProfilerData(int tid){
+
+    if(tid == local_tid){
+        if(PD_cache!=0){
+            return *PD_cache;
+        }
+    }
+
+    //printf("CACHE MISSED seeking %d on %d!!!\n",tid,local_tid);
+    std::lock_guard<std::mutex> guard(ProfilerVectorMutex);
+    checkProfilerVector(tid);
+    ProfilerData* PDOut=TheProfilerThreadList()[tid];
+    if(tid == local_tid){
+        if(PD_cache==0){
+            PD_cache=PDOut;
+        }
+    }
+    return *PDOut;
+}
 
 //////////////////////////////////////////////////////////////////////
 // For OpenMP
 //////////////////////////////////////////////////////////////////////
 #ifdef TAU_TRACK_IDLE_THREADS
-double TheLastTimeStamp[TAU_MAX_THREADS][TAU_MAX_COUNTERS];
+//double TheLastTimeStamp[TAU_MAX_THREADS][TAU_MAX_COUNTERS]; //TODO: DYNATHREAD
+inline void setLastTimeStamp(int tid, int counter, double value){
+    //printf("SLT: TID: %d, CID: %d\n",tid,counter);
+
+    //printf("SLT: Checked\n");
+    //if(ProfilerThreadList()[tid]->TheLastTimeStamp==0||)
+    //{
+    //    printf("SLT: Invalid!\n");
+    //}
+
+    getProfilerData(tid).TheLastTimeStamp[counter]=value;
+}
+inline double getLastTimeStamp(int tid, int counter){
+
+    return getProfilerData(tid).TheLastTimeStamp[counter];
+}
 #endif /* TAU_TRACK_IDLE_THREADS */
 
 //////////////////////////////////////////////////////////////////////
@@ -167,7 +252,7 @@ const char *TauGetCounterString(void)
 #endif /* TAU_PAPI */
   if (tau_env) {
     char *header = new char[1024];
-    sprintf(header, "templated_functions_MULTI_%s", tau_env);
+    snprintf(header, 1024,  "templated_functions_MULTI_%s", tau_env);
     return header;
   } else {
 #ifdef TAU_PAPI_WALLCLOCKTIME
@@ -279,7 +364,9 @@ void Profiler::Start(int tid)
   // get the timers read just after initialization.
 #ifndef TAU_SCOREP
   if (TimeStamp == 0L) {
-    printf("Got a bogus start! %d %s\n", tid, ThisFunction->GetName());
+#if !defined(TAU_USE_OMPT_5_0) && !defined(TAU_GPU) // this can happen with OMPT async threads
+    printf("Got a bogus start! %d %s. Is TIME missing from TAU_METRICS?\n", tid, ThisFunction->GetName());
+#endif
     TauMetrics_getDefaults(tid, StartTime, 1);
     TimeStamp = (x_uint64)StartTime[0];    // USE COUNTER1 for tracing
     if (TimeStamp == 0L) {
@@ -425,13 +512,13 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
   if (GetPhase()) {
     static int sequence=0;
     char annotation[4096];
-    sprintf (annotation, "TAU^seq^%d^phase^%s^nct^%d:%d:%d^timestamp^%lld^start^%lld^", sequence,
+    snprintf (annotation, sizeof(annotation),  "TAU^seq^%d^phase^%s^nct^%d:%d:%d^timestamp^%lld^start^%lld^", sequence,
         ThisFunction->GetName(), RtsLayer::myNode(), RtsLayer::myContext(), RtsLayer::myThread(),
         getTimeStamp(), Tau_get_firstTimeStamp());
     printf ("tau-perfsuite: stopping %s\n", ThisFunction->GetName());
     setenv("PS_HWPC_ANNOTATION", annotation, 1);
     char seqstring[256];
-    sprintf (seqstring, "TAU.%d", sequence);
+    snprintf (seqstring, sizeof(seqstring),  "TAU.%d", sequence);
     int ierr;
 
     ierr = ps_hwpc_suspend();
@@ -466,14 +553,14 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
     /* for openmp parallel regions */
     /* .TAU Application needs to be stopped */
     for (i = 0; i < TAU_MAX_COUNTERS; i++) {
-      CurrentTime[i] = TheLastTimeStamp[tid][i];
+      CurrentTime[i] = getLastTimeStamp(tid,i);
     }
   } else {
     /* use the usual mechanism */
     RtsLayer::getUSecD(tid, CurrentTime);
   }
   for (i = 0; i < TAU_MAX_COUNTERS; i++) {
-    TheLastTimeStamp[tid][i] = CurrentTime[i];
+    setLastTimeStamp(tid,i,CurrentTime[i]);
   }
 #else
   RtsLayer::getUSecD(tid, CurrentTime);
@@ -717,7 +804,9 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
         // Not a destructor of a static object - its a function like main
 
         // Write profile data
+#ifndef TAU_SCOREP
         TauProfiler_StoreData(tid);
+#endif // TAU_SCOREP 
         TAU_VERBOSE("TAU: <Node=%d.Thread=%d>:<pid=%d>: %s initiated TauProfiler_StoreData\n", RtsLayer::myNode(),
             RtsLayer::myThread(), RtsLayer::getPid(), ThisFunction->GetName());
 // Be careful here, we can not disable instrumentation in multithreaded
@@ -736,9 +825,10 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
 #endif /*TAUKTAU */
 
 #ifdef TAU_TRACK_IDLE_THREADS /* Check if we need to shut off .TAU applications on other tids */
-        if (tid == 0) {
+	//Disable if already stopping all threads
+        if ((tid == 0) && (!Tau_check_Stopping_All_Threads())) {
           int i;
-          for (i = 1; i < TAU_MAX_THREADS; i++) {
+          for (i = 1; i < TheProfilerThreadList().size(); i++) {
             /* for all other threads */
             Profiler *cp = TauInternal_CurrentProfiler(i);
             if (cp && strncmp(cp->ThisFunction->GetName(), ".TAU", 4) == 0) {
@@ -767,6 +857,7 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
     plugin_data.timer_group = ThisFunction->GetAllGroups();
     plugin_data.tid = tid;
     plugin_data.timestamp = TimeStamp;
+    plugin_data.metrics = &(TotalTime[0]);
     Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_EXIT, ThisFunction->GetName(), &plugin_data);
   }
 }
@@ -785,9 +876,12 @@ void TauProfiler_theFunctionList(const char ***inPtr, int *numFuncs, bool addNam
     //We do not want to pass back internal pointers.
     *inPtr = (char const **)malloc(sizeof(char *) * numberOfFunctions);
 
+  // don't iterate over the FunctionInfo vector without the lock!
+  RtsLayer::LockDB();
     for (int i = 0; i < numberOfFunctions; i++) {
       (*inPtr)[i] = TheFunctionDB()[i]->GetName();
     }
+  RtsLayer::UnLockDB();
     *numFuncs = numberOfFunctions;
   }
 }
@@ -808,12 +902,12 @@ void TauProfiler_dumpFunctionNames()
 
   //Create temp write to file.
   char filename[1024];
-  sprintf(filename, "%s/temp.%d.%d.%d", dirname, RtsLayer::myNode(), RtsLayer::myContext(), RtsLayer::myThread());
+  snprintf(filename, sizeof(filename),  "%s/temp.%d.%d.%d", dirname, RtsLayer::myNode(), RtsLayer::myContext(), RtsLayer::myThread());
 
   FILE* fp;
   if ((fp = fopen(filename, "w+")) == NULL) {
     char errormsg[1048];
-    sprintf(errormsg, "Error: Could not create %s", filename);
+    snprintf(errormsg, sizeof(errormsg),  "Error: Could not create %s", filename);
     perror(errormsg);
     return;
   }
@@ -827,7 +921,7 @@ void TauProfiler_dumpFunctionNames()
 
   //Rename from the temp filename.
   char dumpfile[1024];
-  sprintf(dumpfile, "%s/dump_functionnames_n,c,t.%d.%d.%d", dirname, RtsLayer::myNode(), RtsLayer::myContext(),
+  snprintf(dumpfile, sizeof(dumpfile),  "%s/dump_functionnames_n,c,t.%d.%d.%d", dirname, RtsLayer::myNode(), RtsLayer::myContext(),
       RtsLayer::myThread());
   rename(filename, dumpfile);
 }
@@ -940,6 +1034,7 @@ void TauProfiler_getFunctionValues(const char **inFuncs, int numFuncs, double **
 
   TauProfiler_updateIntermediateStatistics(tid);
 
+  // don't iterate over the FunctionInfo vector without the lock!
   RtsLayer::LockDB();
 
   for (it = TheFunctionDB().begin(); it != TheFunctionDB().end(); it++) {
@@ -996,6 +1091,7 @@ void TauProfiler_PurgeData(int tid)
   Profiler *curr;
 
   DEBUGPROFMSG("Profiler::PurgeData( tid = "<<tid <<" ) "<<endl;);
+  // don't iterate over the FunctionInfo vector without the lock!
   RtsLayer::LockDB();
 
   // Reset The Function Database
@@ -1139,7 +1235,7 @@ static int writeUserEvents(FILE *fp, int tid)
 static int writeHeader(FILE *fp, int numFunc, char *metricName)
 {
   char header[2096];
-  sprintf(header, "%d %s\n", numFunc, metricName);
+  snprintf(header, sizeof(header),  "%d %s\n", numFunc, metricName);
   strcat(header, "# Name Calls Subrs Excl Incl ");
 
   strcat(header, "ProfileCalls");
@@ -1172,10 +1268,15 @@ int TauProfiler_updateIntermediateStatistics(int tid)
   // an index for iterating over counters
   int c;
 
+  // don't iterate over the FunctionInfo vector without the lock!
+  RtsLayer::LockDB();
   // iterate over all functions in the database.
   for (vector<FunctionInfo*>::iterator it = TheFunctionDB().begin(); it != TheFunctionDB().end(); it++) {
     FunctionInfo *fi = *it;
-
+    if(fi==NULL){
+	    TAU_VERBOSE("WARNING: NULL FunctionInfoPointer!");
+	    continue;
+    }
     // get the current "dump" profile for this timer
     double *incltime = fi->getDumpInclusiveValues(tid);
     double *excltime = fi->getDumpExclusiveValues(tid);
@@ -1261,6 +1362,7 @@ int TauProfiler_updateIntermediateStatistics(int tid)
       //free(ExclTime);
     }
   }
+  RtsLayer::UnLockDB();
   return 0;
 }
 
@@ -1285,6 +1387,8 @@ static int matchFunction(FunctionInfo *fi, const char **inFuncs, int numFuncs)
 // Writes function event data
 static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs, int numFuncs)
 {
+  // don't iterate over the FunctionInfo vector without the lock!
+  RtsLayer::LockDB();
     for (vector<FunctionInfo*>::iterator it = TheFunctionDB().begin(); it != TheFunctionDB().end(); it++) {
         FunctionInfo & fi = **it;
 
@@ -1324,7 +1428,7 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
 
         if (found_one) continue;
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
         // Is a Cupti metric
         if (TauMetrics_getIsCuptiMetric(metric) == TAU_METRIC_CUPTI_METRIC) {
             char const * const metric_name = TauMetrics_getMetricName(metric);
@@ -1430,12 +1534,12 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
                         exclmetric.metricValueDouble, inclmetric.metricValueDouble);
             } // for (dev)
         } else {
-#endif // CUPTI
+#endif // TAU_CUPTI
             double incltime = fi.getDumpInclusiveValues(tid)[metric];
             double excltime = fi.getDumpExclusiveValues(tid)[metric];
 
             /*Do not resolve addresses if they have already been resolved eagerly*/
-            if(strcmp(fi.GetPrimaryGroup(), "TAU_OPENMP") == 0 && !TauEnv_get_ompt_resolve_address_eagerly()) {
+            if(strstr(fi.GetName(), " ADDR <") != NULL && !TauEnv_get_ompt_resolve_address_eagerly()) {
               char resolved_address[10240] = "";
               Tau_ompt_resolve_callsite(fi, resolved_address);
               fprintf(fp, "\"%s", resolved_address);
@@ -1448,14 +1552,15 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
                 fprintf(fp, "[{%#lx}-{%#lx}]", fi.StartAddr, fi.StopAddr);
             }
             fprintf(fp, "\" %ld %ld %.16G %.16G ", fi.GetCalls(tid), fi.GetSubrs(tid), excltime, incltime);
-#ifdef CUPTI
+#ifdef TAU_CUPTI
         }
-#endif //CUPTI
+#endif //TAU_CUPTI
 
         fprintf(fp, "0 ");    // Indicating that profile calls is turned off
         fprintf(fp, "GROUP=\"%s\" \n", fi.GetAllGroups());
 
     } // for (it)
+  RtsLayer::UnLockDB();
 
     return 0;
 }
@@ -1468,6 +1573,8 @@ static int getTrueFunctionCount(int count, int tid, const char **inFuncs, int nu
   AtomicEventDB::iterator it2;
   const char *metricName = TauMetrics_getMetricAtomic(metric);
 
+  // don't iterate over the FunctionInfo vector without the lock!
+  RtsLayer::LockDB();
   for (vector<FunctionInfo*>::iterator it = TheFunctionDB().begin(); it != TheFunctionDB().end(); it++) {
     FunctionInfo *fi = *it;
 
@@ -1506,6 +1613,7 @@ static int getTrueFunctionCount(int count, int tid, const char **inFuncs, int nu
 
     }
   }
+  RtsLayer::UnLockDB();
   return trueCount;
 }
 
@@ -1525,30 +1633,64 @@ static int writeProfile(FILE *fp, char *metricName, int tid, int metric, const c
   return 0;
 }
 
-static int profileWriteCount[TAU_MAX_THREADS];
+//static int profileWriteCount[TAU_MAX_THREADS];
+inline void setProfileWriteCount(int tid, int val){//TODO: DYNATHREAD
+
+    getProfilerData(tid).profileWriteCount=val;
+}
+inline void incProfileWriteCount(int tid){
+
+    //printf("Write count for tid: %d, pre increment: %d\n",tid,ProfilerThreadList[tid]->profileWriteCount);
+    //printf("Vector for tid: %d: %p\n",tid,ProfilerThreadList);
+    //printf("Pointer for tid: %d, pre increment: %p\n",tid,ProfilerThreadList[tid]);
+    //printf("ProfilerThreadList size: %d, checking tid: %d\n",ProfilerThreadList.size(),tid);
+    getProfilerData(tid).profileWriteCount++;
+
+}
+inline int getProfileWriteCount(int tid){//TODO: DYNATHREAD
+
+    return getProfilerData(tid).profileWriteCount;
+}
 static int profileWriteWarningPrinted = 0;
 
 extern "C" int Tau_profiler_initialization()
 {
   int i;
-  for (i = 1; i < TAU_MAX_THREADS; i++) {
-    profileWriteCount[i] = 0;
+  for (i = 1; i < TheProfilerThreadList().size(); i++) {
+    setProfileWriteCount(i,0);
   }
   profileWriteWarningPrinted = 0;
   return 0;
 }
 
 extern "C" int Tau_print_metadata_for_traces(int tid) {
+  MetaDataRepo *localRepo = &(Tau_metadata_getMetaData(tid));
+  bool usePerfetto = false;
 
-  MetaDataRepo *localRepo = NULL;
-    localRepo = &(Tau_metadata_getMetaData(tid));
+#ifdef TAU_PERFETTO
+  if (TauEnv_get_trace_format() == TAU_TRACE_FORMAT_PERFETTO) {
+    usePerfetto = true;
+  }
+#endif
 
-   for (MetaDataRepo::iterator it = (*localRepo).begin(); it != (*localRepo).end(); it++) {
-      string metadata_str(it->first.name + string(" | ") + string(it->second->data.cval));
-      TAU_TRIGGER_EVENT(metadata_str.c_str(), 1.0);
+  for (MetaDataRepo::iterator it = (*localRepo).begin(); it != (*localRepo).end(); it++) {
+#ifdef TAU_PERFETTO
+    if (usePerfetto) {
+      const char* key = it->first.name;
+      const char* val = it->second->data.cval;
+      TauTracePerfettoMetadata(key, val, tid);
+    } else 
+#endif	
+    {
+      std::string metadata_str(std::string(it->first.name) + " | " + std::string(it->second->data.cval));
+      Tau_trigger_userevent(metadata_str.c_str(), 1.0);
+    }
   }
   return 0;
 }
+
+bool& Tau_is_destroyed(void);
+
 // Store profile data at the end of execution (when top level timer stops)
 extern "C" void finalizeCallSites_if_necessary();
 int TauProfiler_StoreData(int tid)
@@ -1557,15 +1699,20 @@ int TauProfiler_StoreData(int tid)
   if(!TheSafeToDumpData()) {
     return -1;
   }
-#ifdef TAU_ENABLE_ROCM
+  /* If TAU has already shut down and written data, return.
+   * This can happen if a thread outlives thread 0. */
+  if (RtsLayer::myThread() > 0 && Tau_is_destroyed()) {
+    return -1;
+  }
+/*#ifdef TAU_ENABLE_ROCM
   TauFlushRocmEventsIfNecessary(tid);
-#endif /* TAU_ENABLE_ROCM */
+#endif *//* TAU_ENABLE_ROCM */
   TauMetrics_finalize();
 
 #ifndef TAU_MPI
   /*Invoke plugins only if both plugin path and plugins are specified
    *Do this first, because the plugin can write TAU_METADATA as recommendations to the user*/
-  if(Tau_plugins_enabled.pre_end_of_execution) {
+  if(RtsLayer::myThread() == 0 && tid == 0 && Tau_plugins_enabled.pre_end_of_execution) {
     Tau_plugin_event_pre_end_of_execution_data_t plugin_data;
     plugin_data.tid = tid;
     Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_PRE_END_OF_EXECUTION, "*", &plugin_data);
@@ -1580,17 +1727,17 @@ int TauProfiler_StoreData(int tid)
 #ifdef TAU_SCOREP
   Tau_write_metadata_records_in_scorep(tid);
 #endif /* TAU_SCOREP */
-  profileWriteCount[tid]++;
+  incProfileWriteCount(tid);
   // if ((tid != 0) && (profileWriteCount[tid] > 1)) return 0;
 #if !defined(PTHREADS)
   // Rob:  Needed to evaluate for kernels to show in profiles (ignore dreaded #2 thread)!
-  if ((tid != 0) && (profileWriteCount[tid] > 1)) {
-    TAU_VERBOSE("[Profiler]: TauProfiler_StoreData: returning, tid: %i, profileWriteCount[%i]: %i\n", tid, tid, profileWriteCount[tid]);
+  if ((tid != 0) && (getProfileWriteCount(tid) > 1)) {
+    TAU_VERBOSE("[Profiler]: TauProfiler_StoreData: returning, tid: %i, profileWriteCount[%i]: %i\n", tid, tid, getProfileWriteCount(tid));
     return 0;
   }
 #endif
   //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 2\n", RtsLayer::myNode(), tid);
-  if (profileWriteCount[tid] == 10) {
+  if (getProfileWriteCount(tid) == 10) {
     RtsLayer::LockDB();
     if (profileWriteWarningPrinted == 0) {
       profileWriteWarningPrinted = 1;
@@ -1644,14 +1791,16 @@ int TauProfiler_StoreData(int tid)
    * support.  For some reason, thread 0 is getting its myThread()
    * value changed from 0, still need to investigate that. */
     if (RtsLayer::myThread() == 0 && tid == 0) {
-    /* clean up other threads? */
-    for (int i = 1; i < RtsLayer::getTotalThreads(); i++) {
-      TAU_VERBOSE("Thread 0 checking other threads... i = %d\n", i);
-      if (TauInternal_CurrentProfiler(i)) {
-        TAU_VERBOSE("Thread 0 writing data for thread %d\n", i);
-        TauProfiler_StoreData(i);
-      }
-    }
+        if (TauEnv_get_recycle_threads()) {
+            /* clean up other threads? */
+            for (int i = 1; i < RtsLayer::getTotalThreads(); i++) {
+                TAU_VERBOSE("Thread 0 checking other threads... i = %d\n", i);
+                if (TauInternal_CurrentProfiler(i)) {
+                    TAU_VERBOSE("Thread 0 writing data for thread %d\n", i);
+                    TauProfiler_StoreData(i);
+                }
+            }
+        }
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
 	/* Only thread 0 should create a merged profile. */
@@ -1709,7 +1858,7 @@ static int getProfileLocation(int metric, char *str)
     int written_bytes = 0;
     unsigned int profile_dir_len = KTAU_NG_PREFIX_LEN + HOSTNAME_LEN;
     profiledir = new char[profile_dir_len];
-    written_bytes = sprintf(profiledir, "%s.", KTAU_NG_PREFIX);
+    written_bytes = snprintf(profiledir, profile_dir_len,  "%s.", KTAU_NG_PREFIX);
     gethostname(profiledir + written_bytes, profile_dir_len - written_bytes);
 #else
     profiledir = TauEnv_get_profiledir();
@@ -1727,7 +1876,7 @@ static int getProfileLocation(int metric, char *str)
 #endif /* DEBUGPROF */
 
         //sanitize metricName before creating a directory name from it.
-        string illegalChars("/\\?%*:|\"<> ");
+        string illegalChars("/\\?%*:|\"<>= ");
         size_t found = metricStr.find_first_of(illegalChars, 0);
         while (found != string::npos) {
             metricStr[found] = '_';
@@ -1776,7 +1925,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
         createdDirectories=true;
       }
    }
-//#ifdef CUPTI
+//#ifdef TAU_CUPTI
 //  CUdevice device;
 //  int retval;
 //  int er, err;
@@ -1802,7 +1951,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
 //      }
 //    }
 //  }
-//#endif //CUPTI
+//#endif //TAU_CUPTI
 
   for (int i = 0; i < Tau_Global_numCounters; i++) {
     if (TauMetrics_getMetricUsed(i)) {
@@ -1816,7 +1965,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
       if (TauMetrics_getIsCuptiMetric(i) == TAU_METRIC_CUPTI_EVENT) continue;
 
       //cout << "metric name: " << metricHeader << endl;
-#ifdef CUPTI
+#ifdef TAU_CUPTI
       // Is a Cupti event, do not record
       //if(TauMetrics_getIsCuptiMetric(i) == 1) continue;
       // Is a Cupti metric
@@ -1839,7 +1988,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
       //                                   NULL, //uint64_t timeDuration,
       //                                   &metricValue);
       //}
-#endif //CUPTI
+#endif //TAU_CUPTI
       getProfileLocation(i, profileLocation);
 //       sprintf(filename, "%s/temp.%d.%d.%d", profileLocation,
 //         RtsLayer::myNode(), RtsLayer::myContext(), tid);
@@ -1863,14 +2012,14 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
         // remove newline
         year[4] = '\0';
         char newStringTime[64];
-        sprintf(newStringTime, "%s-%s-%s-%s-%s", day, month, dayInt, time, year);
+        snprintf(newStringTime, sizeof(newStringTime),  "%s-%s-%s-%s-%s", day, month, dayInt, time, year);
 
-        sprintf(dumpfile, "%s/%s%s__%s__.%d.%d.%d", profileLocation, selectivePrefix, prefix, newStringTime,
+        snprintf(dumpfile, sizeof(dumpfile),  "%s/%s%s__%s__.%d.%d.%d", profileLocation, selectivePrefix, prefix, newStringTime,
             RtsLayer::myNode(), RtsLayer::myContext(), tid);
 
         if ((fp = fopen(dumpfile, "w+")) == NULL) {
           char errormsg[1256];
-          sprintf(errormsg, "Error: Could not create %s", dumpfile);
+          snprintf(errormsg, sizeof(errormsg),  "Error: Could not create %s", dumpfile);
           perror(errormsg);
           return 0;
         }
@@ -1879,7 +2028,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
         char *tst = getcwd(cwd, 1024);
 		if (tst == NULL) {
           char errormsg[1024];
-          sprintf(errormsg, "Error: Could not get current working directory");
+          snprintf(errormsg, sizeof(errormsg),  "Error: Could not get current working directory");
           perror(errormsg);
           return 0;
 		}
@@ -1893,7 +2042,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
 #endif
         int node = RtsLayer::myNode();
 
-        sprintf(dumpfile, "%s/%s%s.%d.%d.%d", profileLocation, selectivePrefix, prefix, node, RtsLayer::myContext(),
+        snprintf(dumpfile, sizeof(dumpfile),  "%s/%s%s.%d.%d.%d", profileLocation, selectivePrefix, prefix, node, RtsLayer::myContext(),
             tid);
 
         int sicortex = 0;
@@ -1905,13 +2054,13 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
           int test = open(dumpfile, flags, mode);
           while (test == -1 && node < 99999) {
             node++;
-            sprintf(dumpfile, "%s/%s%s.%d.%d.%d", profileLocation, selectivePrefix, prefix, node, RtsLayer::myContext(),
+            snprintf(dumpfile, sizeof(dumpfile),  "%s/%s%s.%d.%d.%d", profileLocation, selectivePrefix, prefix, node, RtsLayer::myContext(),
                 tid);
             test = open(dumpfile, flags, mode);
           }
           if ((fp = fdopen(test, "w")) == NULL) {
             char errormsg[1256];
-            sprintf(errormsg, "Error: Could not create %s", dumpfile);
+            snprintf(errormsg, sizeof(errormsg),  "Error: Could not create %s", dumpfile);
             perror(errormsg);
             return 0;
           }
@@ -1925,7 +2074,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
 #endif /* TAU_MPI */
           if ((fp = fopen(dumpfile, "w+")) == NULL) {
             char errormsg[1256];
-            sprintf(errormsg, "Error: Could not create %s", dumpfile);
+            snprintf(errormsg, sizeof(errormsg),  "Error: Could not create %s", dumpfile);
             perror(errormsg);
             return 0;
           }
@@ -1936,7 +2085,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
           char *tst = getcwd(cwd, 1024);
 		  if (tst == NULL) {
             char errormsg[1024];
-            sprintf(errormsg, "Error: Could not get current working directory");
+            snprintf(errormsg, sizeof(errormsg),  "Error: Could not get current working directory");
             perror(errormsg);
             return 0;
 		  }
