@@ -57,6 +57,13 @@ using namespace std;
 //#include <pwd.h>
 #endif /* TAU_BGP */
 
+#ifdef TAU_WINDOWS
+/* We are on Windows which doesn't have strtok_s */
+# define strtok_r strtok_s
+#define TAU_LIB_DIR "tau2/win32/lib"
+#endif
+
+
 #define MAX_LN_LEN 2048
 
 /* We should throttle if number n > a && percall < b .a and b are given below */
@@ -134,6 +141,7 @@ using namespace std;
 #define TAU_EBS_RESOLUTION_STR_LINE "line"
 #define TAU_EBS_RESOLUTION_STR_FILE "file"
 #define TAU_EBS_RESOLUTION_STR_FUNCTION "function"
+#define TAU_EBS_RESOLUTION_STR_FUNCTION_LINE "function_line"
 
 /* Experimental feature - pre-computation of statistics */
 //#if (defined(TAU_UNIFY) && defined(TAU_MPI))
@@ -191,6 +199,9 @@ using namespace std;
 #endif /* TAU_MPI */
 
 #define TAU_CUPTI_API_DEFAULT "runtime"
+#define TAU_CUPTI_PC_DEFAULT 0
+#define TAU_CUPTI_PC_HWS_DEFAULT 0
+#define TAU_CUPTI_PC_PERIOD 0
 #define TAU_CUDA_DEVICE_NAME_DEFAULT NULL
 #define TAU_TRACK_CUDA_INSTRUCTIONS_DEFAULT ""
 #define TAU_TRACK_CUDA_CDP_DEFAULT 0
@@ -213,6 +224,7 @@ using namespace std;
 #define TAU_MEMDBG_ALLOC_MIN_DEFAULT      0 // 0 => undefined, not zero
 #define TAU_MEMDBG_ALLOC_MAX_DEFAULT      0 // 0 => undefined, not zero
 #define TAU_MEMDBG_OVERHEAD_DEFAULT       0 // 0 => undefined, not zero
+#define TAU_MEMMGR_MAX_MEMBLOCKS          64
 #ifdef TAU_BGQ
 #define TAU_MEMDBG_ALIGNMENT_DEFAULT      64
 #else
@@ -225,6 +237,7 @@ using namespace std;
 #define TAU_PTHREAD_STACK_SIZE_DEFAULT    0
 
 #define TAU_MERGE_METADATA_DEFAULT 0
+#define TAU_DISABLE_METADATA_DEFAULT 0
 
 #define TAU_MEM_CALLPATH_DEFAULT 0
 #define TAU_REGION_ADDRESSES_DEFAULT 0
@@ -232,6 +245,13 @@ using namespace std;
 /* Thread recycling */
 #define TAU_RECYCLE_THREADS_DEFAULT 0
 
+/* Rocprofiler-sdk related variables*/
+#define TAU_USE_ROCPROFILERSDK_DEFAULT 0
+#define TAU_USE_ROCSDK_SAMPLING_DEFAULT 0
+
+/* L0 related variables*/
+#define TAU_ENABLE_L0_DEFAULT 0
+
 // forward declartion of cuserid. need for c++ compilers on Cray.
 extern "C" char *cuserid(char *);
 
@@ -266,6 +286,8 @@ static int env_profiling = 0;
 static int env_tracing = 0;
 static int env_thread_per_gpu_stream = 0;
 static int env_trace_format = TAU_TRACE_FORMAT_DEFAULT;
+static int env_perfetto_merge = 1;
+static int env_perfetto_compress = 1;
 static int env_callpath_depth = 0;
 static int env_depth_limit = 0;
 static int env_track_message = 0;
@@ -276,6 +298,7 @@ static int env_track_memory_footprint = 0;
 static int env_show_memory_functions = 0;
 static int env_track_load = 0;
 static int env_tau_lite = 0;
+static int env_tau_anonymize = 0;
 static int env_track_memory_leaks = 0;
 static int env_track_memory_headroom = 0;
 static int env_track_io_params = 0;
@@ -292,6 +315,7 @@ static int env_ebs_period = 0;
 static int env_ebs_inclusive = 0;
 static int env_ompt_resolve_address_eagerly = 1;
 static int env_ompt_support_level = 0;
+static int env_ompt_force_finalize = 1;
 static int env_openmp_runtime_enabled = 1;
 static int env_openmp_runtime_states_enabled = 0;
 static int env_openmp_runtime_events_enabled = 1;
@@ -299,6 +323,7 @@ static int env_openmp_runtime_context = 1;
 static int env_ebs_enabled = 0;
 static int env_ebs_enabled_tau = 0;
 static const char *env_ebs_source = "itimer";
+static const char *env_ebs_source_orig = "itimer";
 static int env_ebs_unwind_enabled = 0;
 static int env_ebs_unwind_depth = TAU_EBS_UNWIND_DEPTH_DEFAULT;
 static int env_ebs_resolution = TAU_EBS_RESOLUTION_LINE;
@@ -324,6 +349,7 @@ static int env_track_mpi_t_comm_metric_values = 0;
 static const char *env_select_file = NULL;
 static const char *env_cupti_api = TAU_CUPTI_API_DEFAULT;
 static const char * env_cuda_device_name = TAU_CUDA_DEVICE_NAME_DEFAULT;
+static const char * env_control_file = NULL;
 static int env_sigusr1_action = TAU_ACTION_DUMP_PROFILES;
 static const char *env_track_cuda_instructions = TAU_TRACK_CUDA_INSTRUCTIONS_DEFAULT;
 static int env_track_cuda_cdp = TAU_TRACK_CUDA_CDP_DEFAULT;
@@ -339,6 +365,10 @@ static int env_node_set = -1;
 
 static int env_cudatotalthreads = 0;
 static int env_taucuptiavail = 0;
+static int env_taucuptipc = 0;
+static int env_taucuptipc_hwsize = 0;
+static int env_taucuptipc_period = 0;
+
 static int env_nodenegoneseen = 0;
 static int env_mic_offload = 0;
 static int env_bfd_lookup = 0;
@@ -364,10 +394,19 @@ static size_t env_memdbg_alignment = TAU_MEMDBG_ALIGNMENT_DEFAULT;
 static int env_memdbg_zero_malloc = TAU_MEMDBG_ZERO_MALLOC_DEFAULT;
 static int env_memdbg_attempt_continue = TAU_MEMDBG_ATTEMPT_CONTINUE_DEFAULT;
 static int env_merge_metadata = 1;
+static int env_disable_metadata = 0;
+static int env_memmgr_max_memblocks = TAU_MEMMGR_MAX_MEMBLOCKS;
 
 static int env_pthread_stack_size = TAU_PTHREAD_STACK_SIZE_DEFAULT;
 static int env_papi_multiplexing = 0;
 
+static int env_rocsdk_enable = 0;
+static int env_rocsdk_pcs_enable = 0;
+
+static int env_l0_enable = 0;
+static int env_l0_metrics_enable = 0;
+static int env_l0_stall_sampling_enable = 0;
+
 #ifdef TAU_ANDROID
 static int env_alfred_port = 6113;
 #endif
@@ -382,6 +421,8 @@ static int env_recycle_threads = TAU_RECYCLE_THREADS_DEFAULT;
 static const char *env_tau_exec_args = NULL;
 static const char *env_tau_exec_path = NULL;
 
+static const char *unitrace_metrics = NULL;
+
 } // extern "C"
 
 /*********************************************************************
@@ -642,9 +683,9 @@ static int TauConf_read()
   if (!cfgFile) {
     char const * exedir = Tau_get_cwd_of_exe();
     if (!exedir) {
-      sprintf(conf_file_name, "./tau.conf");
+      snprintf(conf_file_name, sizeof(conf_file_name),  "./tau.conf");
     } else {
-      sprintf(conf_file_name, "%s/tau.conf", exedir);
+      snprintf(conf_file_name, sizeof(conf_file_name),  "%s/tau.conf", exedir);
       free((void*)exedir);
     }
     TAU_VERBOSE("Trying %s\n", conf_file_name);
@@ -654,7 +695,7 @@ static int TauConf_read()
     TauConf_parse(cfgFile, tmp);
     fclose(cfgFile);
   } else {
-    sprintf(conf_file_name, "%s/tau_system_defaults/tau.conf", TAUROOT);
+    snprintf(conf_file_name, sizeof(conf_file_name),  "%s/tau_system_defaults/tau.conf", TAUROOT);
     cfgFile = fopen(conf_file_name, "r");
     if (cfgFile) {
       TauConf_parse(cfgFile, tmp);
@@ -698,7 +739,7 @@ char * Tau_check_dirname(const char * dir)
     char scratchdir[2048];
 #if (defined (TAU_BGL) || defined(TAU_BGP) || defined(TAU_BGQ) || (defined(__linux__) && !defined(TAU_ANDROID)))
     if (cuserid(user) == NULL) {
-      sprintf(user,"unknown");
+      snprintf(user, sizeof(user), "unknown");
     }
 #else
 
@@ -714,19 +755,19 @@ char * Tau_check_dirname(const char * dir)
     TAU_VERBOSE("TAU: cuserid returns %s\n", temp);
 #endif // TAU_WINDOWS
     if (temp != NULL) {
-      sprintf(user, "%s", temp);
+      snprintf(user, sizeof(user),  "%s", temp);
     } else {
-      sprintf(user, "unknown");
+      snprintf(user, sizeof(user),  "unknown");
     }
     free(temp);
 #endif /* TAU_BGP */
-    ret = sprintf(logfiledir, "%s/%d/%d/%d/%s_id%s_%d-%d-%d", logdir, (thisTime->tm_year + 1900),
+    ret = snprintf(logfiledir, sizeof(logfiledir),  "%s/%d/%d/%d/%s_id%s_%d-%d-%d", logdir, (thisTime->tm_year + 1900),
         (thisTime->tm_mon + 1), thisTime->tm_mday, user, jobid, (thisTime->tm_mon + 1), thisTime->tm_mday,
         (thisTime->tm_hour * 60 * 60 + thisTime->tm_min * 60 + thisTime->tm_sec));
 #ifndef TAU_WINDOWS
-	if (ret < 0) { TAU_VERBOSE("sprintf failed! %s %s %s", __func__, __FILE__, __LINE__); }
+	if (ret < 0) { TAU_VERBOSE("snprintf failed! %s %s %s", sizeof("snprintf failed! %s %s %s"),  __func__, __FILE__, __LINE__); }
 #else
-        if (ret < 0) { TAU_VERBOSE("sprintf failed! %s %s", __FILE__, __LINE__); }
+        if (ret < 0) { TAU_VERBOSE("snprintf failed! %s %s", sizeof("snprintf failed! %s %s"),  __FILE__, __LINE__); }
 #endif  /* TAU_WINDOWS */
     TAU_VERBOSE("Using logdir = %s\n", logfiledir);
     if (RtsLayer::myNode() < 1) {
@@ -737,11 +778,11 @@ char * Tau_check_dirname(const char * dir)
       mode_t oldmode;
       oldmode = umask(0);
       mkdir(logdir, S_IRWXU | S_IRGRP | S_IWGRP | S_IXGRP | S_IRWXO);
-      sprintf(scratchdir, "%s/%d", logdir, (thisTime->tm_year + 1900));
+      snprintf(scratchdir, sizeof(scratchdir),  "%s/%d", logdir, (thisTime->tm_year + 1900));
       mkdir(scratchdir, S_IRWXU | S_IRGRP | S_IWGRP | S_IXGRP | S_IRWXO);
-      sprintf(scratchdir, "%s/%d/%d", logdir, (thisTime->tm_year + 1900), (thisTime->tm_mon + 1));
+      snprintf(scratchdir, sizeof(scratchdir),  "%s/%d/%d", logdir, (thisTime->tm_year + 1900), (thisTime->tm_mon + 1));
       mkdir(scratchdir, S_IRWXU | S_IRGRP | S_IWGRP | S_IXGRP | S_IRWXO);
-      sprintf(scratchdir, "%s/%d/%d/%d", logdir, (thisTime->tm_year + 1900), (thisTime->tm_mon + 1), thisTime->tm_mday);
+      snprintf(scratchdir, sizeof(scratchdir),  "%s/%d/%d/%d", logdir, (thisTime->tm_year + 1900), (thisTime->tm_mon + 1), thisTime->tm_mday);
       mkdir(scratchdir, S_IRWXU | S_IRGRP | S_IWGRP | S_IXGRP | S_IRWXO);
       TAU_VERBOSE("mkdir %s\n", scratchdir);
 
@@ -767,7 +808,9 @@ public:
         if (env_verbose_file == 1 &&
             env_verbose_rank == Tau_get_node()) {
             std::stringstream ss;
-            ss << env_profiledir << "/tau." << Tau_get_node() << ".log";
+            ss << env_profiledir << "/tau." <<
+            (Tau_get_node() < 0 ? 0 : Tau_get_node())
+            << ".log";
             std::string tmp(ss.str());
             pfile = fopen(tmp.c_str(),"w");
         }
@@ -789,6 +832,7 @@ extern "C" { /* C linkage */
 /*********************************************************************
  * Write to stderr if verbose mode is on
  ********************************************************************/
+extern "C" x_uint64 TauMetrics_getTimeOfDay();
 void TAU_VERBOSE(const char *format, ...)
 {
   if (env_verbose == 1) {
@@ -808,6 +852,7 @@ void TAU_VERBOSE(const char *format, ...)
 
 #else
 
+    //fprintf(foo.pfile, "%llu : ", TauMetrics_getTimeOfDay());
     va_start(args, format);
 
 #ifdef TAU_GPI
@@ -950,6 +995,10 @@ int TauEnv_get_ompt_support_level() {
   return env_ompt_support_level;
 }
 
+int TauEnv_get_ompt_force_finalize() {
+  return env_ompt_force_finalize;
+}
+
 int TauEnv_get_track_mpi_t_pvars() {
   return env_track_mpi_t_pvars;
 }
@@ -1049,6 +1098,18 @@ int TauEnv_get_tracing() {
   return env_tracing;
 }
 
+int TauEnv_get_perfetto_compress() {
+  return env_perfetto_compress;
+}
+
+int TauEnv_get_perfetto_merge() {
+  return env_perfetto_merge;
+}
+
+void TauEnv_set_tracing(int tracing) {
+	env_tracing=tracing;
+}
+
 int TauEnv_get_thread_per_gpu_stream() {
   return env_thread_per_gpu_stream;
 }
@@ -1085,6 +1146,10 @@ const char* TauEnv_get_profile_prefix() {
   return env_profile_prefix;
 }
 
+const char *TauEnv_get_control_file(){
+	return env_control_file;
+}
+
 int TauEnv_get_sigusr1_action() {
   return env_sigusr1_action;
 }
@@ -1098,7 +1163,7 @@ int TauEnv_get_ebs_keep_unresolved_addr() {
 void TauEnv_force_set_ebs_period(int period) {
   char tmpstr[512];
   env_ebs_period = period;
-  sprintf(tmpstr, "%d", env_ebs_period);
+  snprintf(tmpstr, sizeof(tmpstr),  "%d", env_ebs_period);
   TAU_METADATA("TAU_EBS_PERIOD (FORCED)", tmpstr);
 }
 
@@ -1159,8 +1224,12 @@ const char *TauEnv_get_ebs_source() {
   return env_ebs_source;
 }
 
+const char *TauEnv_get_ebs_source_orig() {
+  return env_ebs_source_orig;
+}
+
 void TauEnv_override_ebs_source(const char *newName) {
-  env_ebs_source = newName;
+  env_ebs_source = strdup(newName);
   TAU_METADATA("TAU_EBS_SOURCE (Override)", newName);
 }
 
@@ -1226,6 +1295,18 @@ int TauEnv_get_tauCuptiAvail() {
     return env_taucuptiavail;
 }
 
+int TauEnv_get_tauCuptiPC() {
+    return env_taucuptipc;
+}
+
+int TauEnv_get_tauCuptiPC_hwsize() {
+  return env_taucuptipc_hwsize;
+}
+
+int TauEnv_get_tauCuptiPC_period() {
+  return env_taucuptipc_period;
+}
+
 void TauEnv_set_nodeNegOneSeen(int nthreads) {
     env_nodenegoneseen = nthreads;
 }
@@ -1245,6 +1326,10 @@ int TauEnv_get_lite_enabled() {
   return env_tau_lite;
 }
 
+int TauEnv_get_anonymize_enabled() {
+  return env_tau_anonymize;
+}
+
 int TauEnv_get_memdbg() {
   return env_memdbg;
 }
@@ -1327,6 +1412,10 @@ int TauEnv_get_memdbg_attempt_continue() {
   return env_memdbg_attempt_continue;
 }
 
+int TauEnv_get_env_memmgr_max_memblocks() {
+  return env_memmgr_max_memblocks;
+}
+
 int TauEnv_get_pthread_stack_size() {
   return env_pthread_stack_size;
 }
@@ -1335,6 +1424,10 @@ int TauEnv_get_merge_metadata(){
   return env_merge_metadata;
 }
 
+int TauEnv_get_disable_metadata(){
+  return env_disable_metadata;
+}
+
 #ifdef TAU_ANDROID
 int TauEnv_get_alfred_port() {
   return env_alfred_port;
@@ -1359,6 +1452,28 @@ int TauEnv_get_mem_class_present(const char * name) {
     return env_mem_classes_set->count(name);
 }
 
+int TauEnv_get_rocsdk_enable(){
+  return env_rocsdk_enable;
+}
+
+int TauEnv_get_rocsdk_pcs_enable(){
+  return env_rocsdk_pcs_enable;
+}
+
+int TauEnv_get_l0_enable(){
+  return env_l0_enable;
+}
+
+int TauEnv_get_l0_metrics_enable()
+{
+  return env_l0_metrics_enable;
+}
+
+int TauEnv_get_l0_stall_sampling_enable()
+{
+  return env_l0_stall_sampling_enable;
+}
+
 const char * TauEnv_get_tau_exec_args() {
   return env_tau_exec_args;
 }
@@ -1368,6 +1483,48 @@ const char * TauEnv_get_tau_exec_path() {
 }
 
 
+/* Runtime Event Control 
+const char * TauEnv_get_runtime_disable_trigger(void) {
+  static const char *runtime_disable_trigger = NULL;
+  static bool inited = false;
+  if (!inited) {
+    runtime_disable_trigger = getenv("TAU_RUNTIME_DISABLE_TRIGGER");
+    inited = true;
+  }
+  return runtime_disable_trigger;
+}
+
+const char * TauEnv_get_runtime_disable_target(void) {
+  static const char *runtime_disable_target = NULL;
+  static bool inited = false;
+  if (!inited) {
+    runtime_disable_target = getenv("TAU_RUNTIME_DISABLE_TARGET");
+    inited = true;
+  }
+  return runtime_disable_target;
+}
+
+const char * TauEnv_get_runtime_enable_trigger(void) {
+  static const char *runtime_enable_trigger = NULL;
+  static bool inited = false;
+  if (!inited) {
+    runtime_enable_trigger = getenv("TAU_RUNTIME_ENABLE_TRIGGER");
+    inited = true;
+  }
+  return runtime_enable_trigger;
+}
+
+const char * TauEnv_get_runtime_enable_target(void) {
+  static const char *runtime_enable_target = NULL;
+  static bool inited = false;
+  if (!inited) {
+    runtime_enable_target = getenv("TAU_RUNTIME_ENABLE_TARGET");
+    inited = true;
+  }
+  return runtime_enable_target;
+}
+ End Runtime Event Control */
+
 /*********************************************************************
  * Initialize the TauEnv module, get configuration values
  ********************************************************************/
@@ -1406,14 +1563,15 @@ void TauEnv_initialize()
     tmp = getconf("TAU_VERBOSE_RANK");
     if (parse_int(tmp,env_verbose_rank)) {
       TAU_VERBOSE("TAU: VERBOSE RANK enabled\n");
-      sprintf(tmpstr, "%d", env_verbose_rank);
+      snprintf(tmpstr, sizeof(tmpstr),  "%d", env_verbose_rank);
       TAU_METADATA("TAU_VERBOSE_RANK", tmpstr);
       env_verbose_rank = Tau_get_node();
     }
 
-    sprintf(tmpstr, "%d", TAU_MAX_THREADS);
-    TAU_VERBOSE("TAU: Supporting %d threads\n", TAU_MAX_THREADS);
-    TAU_METADATA("TAU_MAX_THREADS", tmpstr);
+    //sprintf(tmpstr, "%d", TAU_MAX_THREADS);//TODO: DYNATHREAD
+    TAU_VERBOSE("TAU: Supporting dynamic allocation of threads\n");//, TAU_MAX_THREADS);
+    //TAU_METADATA("TAU_MAX_THREADS", tmpstr);
+
 
     /*** Options that can be used with Scalasca and VampirTrace ***/
     tmp = getconf("TAU_LITE");
@@ -1423,13 +1581,14 @@ void TauEnv_initialize()
       env_tau_lite = 1;
     }
 
+
     const char *interval = getconf("TAU_INTERRUPT_INTERVAL");
     env_interval = TAU_INTERRUPT_INTERVAL_DEFAULT;;
     if (interval) {
       int interval_value = 0;
       sscanf(interval,"%d",&interval_value);
       env_interval = interval_value;
-      sprintf(tmpstr, "%d", env_interval);
+      snprintf(tmpstr, sizeof(tmpstr),  "%d", env_interval);
       TAU_SET_INTERRUPT_INTERVAL(interval_value);
       TAU_METADATA("TAU_INTERRUPT_INTERVAL", tmpstr);
     }
@@ -1529,6 +1688,15 @@ void TauEnv_initialize()
     }
 #endif /* defined (TAU_USE_OMPT_TR6) || defined (TAU_USE_OMPT_TR7) || defined (TAU_USE_OMPT_5_0) */
 
+    tmp = getconf("TAU_OMPT_FORCE_FINALIZE");
+    if (parse_bool(tmp, env_ompt_force_finalize)) {
+      TAU_VERBOSE("TAU: OMPT Finalize Tool Enabled\n");
+      env_ompt_force_finalize = 1;
+    } else {
+      TAU_VERBOSE("TAU: OMPT Finalize Tool Disabled\n");
+      env_ompt_force_finalize = 0;
+    }
+
     tmp = getconf("TAU_TRACK_HEAP");
     if (parse_bool(tmp, env_track_memory_heap)) {
 #ifdef TAU_DISABLE_MEM_MANAGER
@@ -1666,7 +1834,7 @@ void TauEnv_initialize()
     if(env_memdbg) {
 
       size_t page_size = Tau_page_size();
-      sprintf(tmpstr, "%ld", page_size);
+      snprintf(tmpstr, sizeof(tmpstr),  "%ld", page_size);
       TAU_METADATA("Virtual Memory Page Size", tmpstr);
 
       env_track_signals = 1;
@@ -1721,7 +1889,7 @@ void TauEnv_initialize()
       } else {
         TAU_VERBOSE("TAU: Memory debugging alignment: %ld\n", env_memdbg_alignment);
       }
-      sprintf(tmpstr, "%ld", env_memdbg_alignment);
+      snprintf(tmpstr, sizeof(tmpstr),  "%ld", env_memdbg_alignment);
       TAU_METADATA("TAU_MEMDBG_ALIGNMENT", tmpstr);
 
       tmp = getconf("TAU_MEMDBG_ZERO_MALLOC");
@@ -1891,16 +2059,45 @@ void TauEnv_initialize()
 #endif
       } else if(strcasecmp(tmp, "tau") == 0) {
         env_trace_format = TAU_TRACE_FORMAT_TAU;
+      }
+		else if(strcasecmp(tmp, "perfetto") == 0) {
+#ifdef TAU_PERFETTO
+        env_trace_format = TAU_TRACE_FORMAT_PERFETTO;
+#else
+        fprintf(stderr, "TAU: Warning: requested Perfetto trace but TAU built without Perfetto support, using default instead.\n");
+#endif
       } else {
         fprintf(stderr, "TAU: Warning: unrecognized trace format %s, using default instead.\n", tmp);
       }
     }
+	
+	/* Perfetto compression disable flag */
+    tmp = getconf("TAU_PERFETTO_COMPRESS");
+    if(tmp != NULL) {
+      if(!strcasecmp(tmp,"0") || !strcasecmp(tmp,"false") ||
+         !strcasecmp(tmp,"no") || !strcasecmp(tmp,"off")) {
+        env_perfetto_compress = 0;
+      }
+    }
+	
+	tmp = getconf("TAU_PERFETTO_MERGE");
+    if(tmp != NULL) {
+      if(!strcasecmp(tmp,"0") || !strcasecmp(tmp,"false") ||
+         !strcasecmp(tmp,"no") || !strcasecmp(tmp,"off")) {
+        env_perfetto_merge = 0;
+      }
+    }
+	
+	
     if(env_trace_format == TAU_TRACE_FORMAT_TAU) {
       TAU_VERBOSE("TAU: Trace format is tau\n");
       TAU_METADATA("TAU_TRACE_FORMAT", "tau")
     } else if(env_trace_format == TAU_TRACE_FORMAT_OTF2) {
       TAU_VERBOSE("TAU: Trace format is otf2\n");
       TAU_METADATA("TAU_TRACE_FORMAT", "otf2")
+    } else if(env_trace_format == TAU_TRACE_FORMAT_PERFETTO) {
+      TAU_VERBOSE("TAU: Trace format is perfetto\n");
+      TAU_METADATA("TAU_TRACE_FORMAT", "perfetto")
     }
 
     /* profiling */
@@ -1918,7 +2115,7 @@ void TauEnv_initialize()
     tmp = getconf("TAU_CURRENT_TIMER_EXIT_PARAMS");
     if (parse_bool(tmp, profiling_default)) {
       env_current_timer_exit_params = 1;
-      TAU_VERBOSE("TAU: Profiling Enabled\n");
+      TAU_VERBOSE("TAU: Current Timer Exit Enabled\n");
       TAU_METADATA("TAU_CURRENT_TIMER_EXIT_PARAMS", "on");
     } else {
       env_current_timer_exit_params = 0;
@@ -1992,7 +2189,7 @@ void TauEnv_initialize()
       }
     }
     TAU_VERBOSE("TAU: Callsite Depth Limit = %d\n", env_callsite_depth);
-    sprintf(tmpstr, "%d", env_callsite_depth);
+    snprintf(tmpstr, sizeof(tmpstr),  "%d", env_callsite_depth);
     TAU_METADATA("TAU_CALLSITE_DEPTH", tmpstr);
 
     const char *callsiteOffset = getconf("TAU_CALLSITE_OFFSET");
@@ -2002,7 +2199,7 @@ void TauEnv_initialize()
       if (env_callsite_offset < 0) {
         env_callsite_offset = TAU_CALLSITE_OFFSET_DEFAULT;
       }
-      sprintf(tmpstr, "%d", env_callsite_offset);
+      snprintf(tmpstr, sizeof(tmpstr),  "%d", env_callsite_offset);
       TAU_METADATA("TAU_CALLSITE_OFFSET", tmpstr);
       TAU_VERBOSE("TAU: Callsite Offset = %d\n", env_callsite_offset);
     }
@@ -2099,7 +2296,7 @@ void TauEnv_initialize()
     if (env_callpath) {
       TAU_VERBOSE("TAU: Callpath Depth = %d\n", env_callpath_depth);
     }
-    sprintf(tmpstr, "%d", env_callpath_depth);
+    snprintf(tmpstr, sizeof(tmpstr),  "%d", env_callpath_depth);
     TAU_METADATA("TAU_CALLPATH_DEPTH", tmpstr);
 
 #ifdef TAU_DEPTH_LIMIT
@@ -2110,7 +2307,7 @@ void TauEnv_initialize()
       env_depth_limit = atoi(tmp);
     }
     TAU_VERBOSE("TAU: Depth Limit = %d\n", env_depth_limit);
-    sprintf(tmpstr, "%d", env_depth_limit);
+    snprintf(tmpstr, sizeof(tmpstr),  "%d", env_depth_limit);
     TAU_METADATA("TAU_DEPTH_LIMIT", tmpstr);
 #endif /* TAU_DEPTH_LIMIT */
 
@@ -2168,12 +2365,19 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: Throttle PerCall = %g\n", env_throttle_percall);
       TAU_VERBOSE("TAU: Throttle NumCalls = %g\n", env_throttle_numcalls);
 
-      sprintf(tmpstr, "%g", env_throttle_percall);
+      snprintf(tmpstr, sizeof(tmpstr),  "%g", env_throttle_percall);
       TAU_METADATA("TAU_THROTTLE_PERCALL", tmpstr);
-      sprintf(tmpstr, "%g", env_throttle_numcalls);
+      snprintf(tmpstr, sizeof(tmpstr),  "%g", env_throttle_numcalls);
       TAU_METADATA("TAU_THROTTLE_NUMCALLS", tmpstr);
     }
 
+	const char *controlFile = getconf("TAU_CONTROL_FILE");
+	if(controlFile){
+		env_control_file = controlFile;
+		TAU_VERBOSE("TAU: Control File is %s\n",env_control_file);
+	}
+	
+
     const char *sigusr1Action = getconf("TAU_SIGUSR1_ACTION");
     if (sigusr1Action != NULL && 0 == strcasecmp(sigusr1Action, "backtraces")) {
       env_sigusr1_action = TAU_ACTION_DUMP_BACKTRACES;
@@ -2181,9 +2385,13 @@ void TauEnv_initialize()
     } else if (sigusr1Action != NULL && 0 == strcasecmp(sigusr1Action, "callpaths")) {
       env_sigusr1_action = TAU_ACTION_DUMP_CALLPATHS;
       TAU_VERBOSE("TAU: SIGUSR1 Action: dump callpaths\n");
+    } else if (sigusr1Action != NULL && 0 == strcasecmp(sigusr1Action, "controlfile")) {
+      env_sigusr1_action = TAU_ACTION_RUN_CONTROLFILE;
+      TAU_VERBOSE("TAU: SIGUSR1 Action: run control file\n");
     } else {
       TAU_VERBOSE("TAU: SIGUSR1 Action: dump profiles\n");
 	}
+	
 
     const char *profileFormat = getconf("TAU_PROFILE_FORMAT");
     if (profileFormat != NULL && 0 == strcasecmp(profileFormat, "snapshot")) {
@@ -2210,6 +2418,17 @@ void TauEnv_initialize()
       TAU_METADATA("TAU_PROFILE_FORMAT", "profile");
     }
 
+    tmp = getconf("TAU_ANONYMIZE");
+    if (parse_bool(tmp,env_tau_anonymize)) {
+      TAU_VERBOSE("TAU: Anonymize enabled\n");
+      TAU_METADATA("TAU_ANONYMIZE", "on");
+      env_tau_anonymize = 1;
+      env_profile_format = TAU_FORMAT_MERGED;
+      TAU_VERBOSE("TAU: Output Format: merged\n");
+      TAU_METADATA("TAU_PROFILE_FORMAT", "merged");
+    }
+
+
     tmp = getconf("TAU_SUMMARY");
     if (parse_bool(tmp, env_summary_only)) {
 #ifdef TAU_MPI
@@ -2274,7 +2493,7 @@ void TauEnv_initialize()
           //sprintf(env_plugins,"libTAU-filter-plugin.so(%s)", env_select_file);
           char *plugins = (char *) malloc(1024);
 	  char *filename = strdup(env_select_file);
-          sprintf(plugins, "libTAU-filter-plugin.so(%s)", filename);
+          snprintf(plugins, sizeof(plugins),  "libTAU-filter-plugin.so(%s)", filename);
           env_plugins = plugins;
           TAU_VERBOSE("TAU: TAU plugin is now %s\n", env_plugins);
 	  TAU_METADATA("TAU_SELECT_FILE", filename);
@@ -2378,6 +2597,15 @@ void TauEnv_initialize()
       env_ebs_keep_unresolved_addr = 0;
       TAU_METADATA("TAU_EBS_KEEP_UNRESOLVED_ADDR", "off");
     }
+    
+    tmp = getconf("TAU_MEMMGR_MAX_BLOCKS");
+    if (tmp) {
+      env_memmgr_max_memblocks = atoi(tmp);
+    }
+    TAU_VERBOSE("TAU: MEMMGR_MAX_BLOCKS = %d\n", TauEnv_get_env_memmgr_max_memblocks());
+    snprintf(tmpstr, sizeof(tmpstr),  "%d", TauEnv_get_env_memmgr_max_memblocks());
+    TAU_METADATA("TAU_MEMMGR_MAX_BLOCKS", tmpstr);
+    
 
     if (TauEnv_get_ebs_enabled()) {
 
@@ -2392,8 +2620,9 @@ void TauEnv_initialize()
       if ((env_ebs_source = getconf("TAU_EBS_SOURCE")) == NULL) {
         env_ebs_source = "itimer";
       }
-      sprintf(tmpstr, "%s", env_ebs_source);
-      TAU_METADATA("TAU_EBS_PERIOD", tmpstr);
+      env_ebs_source_orig = strdup(env_ebs_source);
+      snprintf(tmpstr, sizeof(tmpstr),  "%s", env_ebs_source);
+      TAU_METADATA("TAU_EBS_SOURCE", tmpstr);
 
       TAU_VERBOSE("TAU: EBS Source: %s\n", env_ebs_source);
 
@@ -2425,7 +2654,7 @@ void TauEnv_initialize()
         }
       }
       TAU_VERBOSE("TAU: EBS period = %d \n", env_ebs_period);
-      sprintf(tmpstr, "%d", env_ebs_period);
+      snprintf(tmpstr, sizeof(tmpstr),  "%d", env_ebs_period);
       TAU_METADATA("TAU_EBS_PERIOD", tmpstr);
 
       bool ebs_period_forced = false;
@@ -2442,7 +2671,7 @@ void TauEnv_initialize()
       }
 #endif
       if (ebs_period_forced) {
-        sprintf(tmpstr, "%d", env_ebs_period);
+        snprintf(tmpstr, sizeof(tmpstr),  "%d", env_ebs_period);
         TAU_METADATA("TAU_EBS_PERIOD (FORCED)", tmpstr);
       }
 
@@ -2455,7 +2684,7 @@ void TauEnv_initialize()
         }
       }
       TAU_VERBOSE("TAU: EBS inclusive = %d usec\n", env_ebs_inclusive);
-      sprintf(tmpstr, "%d usec", env_ebs_inclusive);
+      snprintf(tmpstr, sizeof(tmpstr),  "%d usec", env_ebs_inclusive);
       TAU_METADATA("TAU_EBS_INCLUSIVE", tmpstr);
 
 #ifdef TAU_UNWIND
@@ -2478,9 +2707,9 @@ void TauEnv_initialize()
           }
         }
 		if (env_ebs_unwind_depth == 0) {
-          sprintf(tmpstr, "auto");
+          snprintf(tmpstr, sizeof(tmpstr),  "auto");
         } else {
-          sprintf(tmpstr, "%d", env_ebs_unwind_depth);
+          snprintf(tmpstr, sizeof(tmpstr),  "%d", env_ebs_unwind_depth);
 		}
         TAU_METADATA("TAU_EBS_UNWIND_DEPTH", tmpstr);
       }
@@ -2494,6 +2723,9 @@ void TauEnv_initialize()
           } else if (strcmp(ebs_resolution, TAU_EBS_RESOLUTION_STR_FUNCTION) == 0) {
               env_ebs_resolution = TAU_EBS_RESOLUTION_FUNCTION;
               TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_FUNCTION);
+          } else if (strcmp(ebs_resolution, TAU_EBS_RESOLUTION_STR_FUNCTION_LINE) == 0) {
+              env_ebs_resolution = TAU_EBS_RESOLUTION_FUNCTION_LINE;
+              TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_FUNCTION_LINE);
           } else if (strcmp(ebs_resolution, TAU_EBS_RESOLUTION_STR_LINE) == 0) { // otherwise, it's the default - line.
               env_ebs_resolution = TAU_EBS_RESOLUTION_LINE;
               TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_LINE);
@@ -2544,6 +2776,61 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: CUPTI API tracking: %s\n", env_cupti_api);
       TAU_METADATA("TAU_CUPTI_API", env_cupti_api);
 		}
+
+    tmp = getconf("TAU_CUPTI_PC");
+    if (parse_bool(tmp, TAU_CUPTI_PC_DEFAULT)) {
+      env_taucuptipc = 1;
+      TAU_VERBOSE("TAU: CUPTI PC sampling Enabled\n");
+      TAU_METADATA("TAU_CUPTI_PC", "on");
+    } else {
+      TAU_VERBOSE("TAU: CUPTI PC sampling Disabled\n");
+      TAU_METADATA("TAU_CUPTI_PC", "off");
+    }
+
+    if(env_taucuptipc)
+    {
+      tmp = getconf("TAU_CUPTI_PC_HWB");
+      if (tmp) {
+        env_taucuptipc_hwsize = atoi(tmp);
+        if(env_taucuptipc_hwsize < 1)
+        {
+          env_taucuptipc_hwsize = 0;
+          TAU_VERBOSE("TAU: CUPTI PC sampling hardware buffer size: default\n");
+          TAU_METADATA("TAU_CUPTI_PC_HWB", "default");
+        }
+        else
+        {
+          TAU_VERBOSE("TAU: CUPTI PC sampling hardware buffer size: %d\n", env_taucuptipc_hwsize);
+          snprintf(tmpstr, sizeof(tmpstr),  "%d MB", env_taucuptipc_hwsize);
+          TAU_METADATA("TAU_CUPTI_PC_HWB", tmpstr);
+        }
+      } else {
+        TAU_VERBOSE("TAU: CUPTI PC sampling hardware buffer size: default\n");
+        TAU_METADATA("TAU_CUPTI_PC_HWB", "default");
+      }
+
+      //default values are defined by CUDA, and will be set when variable equals 0
+      tmp = getconf("TAU_CUPTI_PC_PERIOD");
+      if (tmp) {
+        env_taucuptipc_period = atoi(tmp);
+        if(env_taucuptipc_period < 1)
+        {
+          env_taucuptipc_period = 0;
+          TAU_VERBOSE("TAU: CUPTI PC sampling period: default\n");
+          TAU_METADATA("TAU_CUPTI_PC_PERIOD", "default");
+        }
+        else
+        {
+          TAU_VERBOSE("TAU: CUPTI PC sampling period: %d\n", env_taucuptipc_period);
+          snprintf(tmpstr, sizeof(tmpstr),  "2^%d cycles", env_taucuptipc_period);
+          TAU_METADATA("TAU_CUPTI_PC_PERIOD", tmpstr);
+        }
+      } else {
+        TAU_VERBOSE("TAU: CUPTI PC sampling period: default\n");
+        TAU_METADATA("TAU_CUPTI_PC_PERIOD", "default");
+      }
+    }
+
     env_cuda_device_name = getconf("TAU_CUDA_DEVICE_NAME");
     if (!env_cuda_device_name || 0 == strcasecmp(env_cuda_device_name, "")) {
         env_cuda_device_name = TAU_CUDA_DEVICE_NAME_DEFAULT;
@@ -2590,7 +2877,7 @@ void TauEnv_initialize()
 	env_sass_type = sass_type;
       }
       TAU_VERBOSE("TAU: SASS type = %s \n", env_sass_type);
-      sprintf(tmpstr, "%s", env_sass_type);
+      snprintf(tmpstr, sizeof(tmpstr),  "%s", env_sass_type);
       TAU_METADATA("TAU_SASS_TYPE", tmpstr);
 
     } else {
@@ -2652,6 +2939,15 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: Metadata Merging Disabled\n");
     }
 
+    tmp = getconf("TAU_DISABLE_METADATA");
+    if (parse_bool(tmp, TAU_DISABLE_METADATA_DEFAULT)) {
+      env_disable_metadata = 1;
+      TAU_VERBOSE("TAU: Metadata Disabled\n");
+    } else {
+      env_disable_metadata = 0;
+      TAU_VERBOSE("TAU: Metadata Enabled\n");
+    }
+
 #if defined(TAU_TBB_SUPPORT) && defined(TAU_MPI)
     if (env_profile_format != TAU_FORMAT_MERGED) {
       std::cerr << "TAU: WARNING: TAU_PROFILE_FORMAT=merged is recommended when profiling TBB and MPI." << std::endl;
@@ -2706,6 +3002,51 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: TAU_EXEC_PATH is \"%s\"\n", env_tau_exec_path);
     }
 
+    tmp = getconf("TAU_USE_ROCPROFILERSDK");
+    if (parse_bool(tmp, TAU_USE_ROCPROFILERSDK_DEFAULT)) {
+      env_rocsdk_enable = 1;
+      TAU_VERBOSE("TAU: Rocprofiler-SDK Enabled\n");
+      TAU_METADATA("TAU_ROCPROFILERSDK", "on");
+		}
+
+    if(env_rocsdk_enable)
+    {
+      tmp = getconf("ROCPROFILER_PC_SAMPLING_BETA_ENABLED");
+      if (parse_bool(tmp, TAU_USE_ROCSDK_SAMPLING_DEFAULT)) {
+        env_rocsdk_pcs_enable = 1;
+        TAU_VERBOSE("TAU: Rocprofiler-SDK PC Sampling Enabled\n");
+        TAU_METADATA("TAU_ROCPROFILERSDK_PC_SAMPLING", "on");
+      }
+    }
+
+    tmp = getconf("ZE_ENABLE_TRACING_LAYER");
+    if (parse_bool(tmp, TAU_ENABLE_L0_DEFAULT)) {
+      env_l0_enable = 1;
+      TAU_VERBOSE("TAU: L0 profiling Enabled\n");
+    } else {
+      env_l0_enable = 0;
+      TAU_VERBOSE("TAU: L0 profiling Disabled\n");
+    }
+
+    if ((unitrace_metrics = getconf("L0_METRICGROUP")) == NULL) {
+      unitrace_metrics = "";
+      env_l0_metrics_enable = 0;
+      TAU_VERBOSE("TAU: L0_METRICGROUP is not set\n");
+    } else {
+      TAU_VERBOSE("TAU: L0_METRICGROUP is \"%s\"\n", unitrace_metrics);
+      env_l0_metrics_enable = 1;
+    }
+
+    tmp = getconf("ZE_ENABLE_STALL_SAMPLING");
+    if (parse_bool(tmp, TAU_ENABLE_L0_DEFAULT)) {
+      env_l0_stall_sampling_enable = 1;
+      TAU_VERBOSE("TAU: L0 stall sampling Enabled\n");
+    } else {
+      env_l0_stall_sampling_enable = 0;
+      TAU_VERBOSE("TAU: L0 stall sampling Disabled\n");
+    }
+
+
     initialized = 1;
     TAU_VERBOSE("TAU: Initialized TAU (TAU_VERBOSE=1)\n");
 
@@ -2730,8 +3071,8 @@ void TauEnv_initialize()
     //TauSetupHandler();
   }
 
-  TAU_VERBOSE("Calling TAU_ROCTRACER...\n");
 #ifdef TAU_ENABLE_ROCTRACER
+  TAU_VERBOSE("Calling TAU_ROCTRACER...\n");
   Tau_roctracer_start_tracing();
 #endif /* TAU_ENABLE_ROCTRACER */
 }
