@@ -33,6 +33,10 @@
 
 #ifndef TAU_WINDOWS
 #include <unistd.h>
+#include <strings.h> /* For strcasecmp() on POSIX systems */
+#else
+/* For Windows, define strcasecmp as _stricmp */
+#define strcasecmp _stricmp
 #endif
 
 #include <Profile/TauEnv.h>
@@ -57,9 +61,10 @@
 #include <Profile/TauSCOREP.h>
 #endif
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 #include <Profile/CuptiLayer.h>
-#endif
+#include <Profile/CuptiPCSampling.h>
+#endif //TAU_CUPTI
 
 #ifdef TAU_ANDROID
 
@@ -151,6 +156,148 @@ int dl_initialized = 0;
 int dl_initialized = 1;
 #endif
 
+
+static void trim_trailing_whitespace(char* str) {
+    if (str == NULL) return;
+    int len = strlen(str);
+    while (len > 0 && isspace((unsigned char)str[len - 1])) {
+        str[--len] = '\0';
+    }
+}
+
+/**
+ * Reads and processes a TAU control file to dynamically change profiling settings.
+ *
+ * NOTE ON FILE LIFECYCLE MANAGEMENT:
+ * This function does not rename or delete the control file after
+ * processing. It is the responsibility of the external user or script to manage
+ * the state of this file.
+ *
+ * To ensure all ranks/threads in a job process the exact same command set, the
+ * following "atomic update" procedure is recommended:
+ *
+ * 1.  Write the desired commands to a temporary file (e.g., commands.tmp).
+ *
+ * 2.  Once the file is complete, use the 'mv' command to atomically rename it
+ *     to the target control file path (e.g., 'mv commands.tmp .tau_command.conf').
+ *     The 'rename' operation is atomic on POSIX filesystems, guaranteeing that
+ *     no rank will read a partially written or incomplete file. Do not use 'cp'.
+ *
+ * 3.  Send the signal after the rename operation is complete.
+ *
+ * 4.  The external script is responsible for deleting or clearing the control
+ *     file before the next signal is sent to prevent reprocessing old commands.
+ */
+ void TauRunControlFile() {
+    // Get the path from the environment wrapper function.
+    const char* control_file_path = TauEnv_get_control_file();
+
+    if (control_file_path == NULL || *control_file_path == '\0') {
+        fprintf(stderr, "TAU ERROR: SIGUSR1 action is 'RunControlFile' but no control file path is set.\n");
+        return;
+    }
+
+    // Verify the file exists and open it.
+    FILE* fp = fopen(control_file_path, "r");
+    if (fp == NULL) {
+        // This is not necessarily an error. The user may not have created the file yet.
+        fprintf(stderr, "TAU INFO: Control file '%s' not found, nothing to do.\n", control_file_path);
+        return;
+    }
+
+    fprintf(stderr, "TAU INFO: Processing control file '%s'...\n", control_file_path);
+
+    char line_buffer[2048];
+
+    // Read the file line-by-line.
+    while (fgets(line_buffer, sizeof(line_buffer), fp)) {
+        char* line_ptr = line_buffer;
+
+        // Skip leading whitespace
+        while (isspace(*line_ptr)) line_ptr++;
+
+        // Skip comments and empty lines
+        if (*line_ptr == '#' || *line_ptr == '\0') {
+            continue;
+        }
+        
+        // Trim trailing whitespace/newline from the whole line
+        trim_trailing_whitespace(line_ptr);
+
+        char* command = line_ptr;
+        char* arg = NULL;
+
+        // Check for a arg using the ':' delimiter
+        char* colon = strchr(line_ptr, ':');
+        if (colon != NULL) {
+            *colon = '\0'; // Terminate the command string
+            arg = colon + 1;
+
+            // Trim command and arg
+            trim_trailing_whitespace(command);
+            while (isspace(*arg)) arg++;
+            // Trailing space on arg is already handled by the initial line trim
+        }
+
+        // --- COMMAND DISPATCHER ---
+        if (strcasecmp(command, "exclude_ranks") == 0) {
+            fprintf(stderr, "TAU  > Command: 'exclude_ranks', Argument: '%s'\n", arg ? arg : "N/A");
+            if (arg) Tau_set_rank_exclusion_list(arg);
+            else fprintf(stderr, "TAU WARNING: 'exclude_ranks' command requires an argument.\n");
+        } else if (strcasecmp(command, "exclude_function") == 0) {
+            fprintf(stderr, "TAU  > Command: 'exclude_function', Argument: '%s'\n", arg ? arg : "N/A");
+            if (arg) Tau_exclude_function_by_name(arg);
+            else fprintf(stderr, "TAU WARNING: 'exclude_function' command requires an arg.\n");
+
+        } else if (strcasecmp(command, "include_function") == 0) {
+            fprintf(stderr, "TAU  > Command: 'include_function', Argument: '%s'\n", arg ? arg : "N/A");
+            if (arg) Tau_include_function_by_name(arg);
+            else fprintf(stderr, "TAU WARNING: 'include_function' command requires an arg.\n");
+
+        } else if (strcasecmp(command, "set_rank_mode_exclude") == 0) {
+            fprintf(stderr, "TAU  > Command: 'set_rank_mode_exclude'\n");
+            Tau_exclude_rank_list();
+        
+        } else if (strcasecmp(command, "set_rank_mode_include") == 0) {
+            fprintf(stderr, "TAU  > Command: 'set_rank_mode_include'\n");
+            Tau_include_rank_list();
+        
+        } else if (strcasecmp(command, "set_rank_mode_ignore") == 0) {
+            fprintf(stderr, "TAU  > Command: 'set_rank_mode_ignore'\n");
+            Tau_ignore_rank_list();
+
+        } else if (strcasecmp(command, "tracing_on") == 0) {
+            fprintf(stderr, "TAU  > Command: 'tracing_on'\n");
+            Tau_tracing_on();
+
+        } else if (strcasecmp(command, "tracing_off") == 0) {
+            fprintf(stderr, "TAU  > Command: 'tracing_off'\n");
+            Tau_tracing_off();
+
+        } else if (strcasecmp(command, "enable_function_exclusion") == 0) {
+            fprintf(stderr, "TAU  > Command: 'enable_function_exclusion'\n");
+            Tau_enable_function_exclusion();
+
+        } else if (strcasecmp(command, "disable_function_exclusion") == 0) {
+            fprintf(stderr, "TAU  > Command: 'disable_function_exclusion'\n");
+            Tau_disable_function_exclusion();
+
+        } else if (strcasecmp(command, "exclude_default_group") == 0) {
+            fprintf(stderr, "TAU  > Command: 'exclude_default_group'\n");
+            Tau_exclude_default_group();
+
+        } else if (strcasecmp(command, "include_default_group") == 0) {
+            fprintf(stderr, "TAU  > Command: 'include_default_group'\n");
+            Tau_include_default_group();
+
+        } else {
+            fprintf(stderr, "TAU WARNING: Unknown command in control file: '%s'\n", command);
+        }
+    }
+
+    fclose(fp);
+}
+
 #ifndef TAU_DISABLE_SIGUSR
 
 static void tauSignalHandler(int sig)
@@ -163,7 +310,10 @@ static void tauSignalHandler(int sig)
     Tau_dump_callpaths();
   } else if (TauEnv_get_sigusr1_action() == TAU_ACTION_DUMP_BACKTRACES) {
     fprintf(stderr, "Caught SIGUSR1, dumping backtrace data\n");
-  } else {
+  } else if (TauEnv_get_sigusr1_action() == TAU_ACTION_RUN_CONTROLFILE) {
+    fprintf(stderr, "Caught SIGUSR1, running control file\n");
+	TauRunControlFile();
+  }else {
     fprintf(stderr, "Caught SIGUSR1, dumping TAU profile data\n");
     //TAU_DB_DUMP_PREFIX("profile");
     TauInternalFunctionGuard protects_this_function;
@@ -210,7 +360,7 @@ static void tauBacktraceHandler(int sig, siginfo_t *si, void *context)
 
   // Trigger a context event and record metadata
   char eventname[1024];
-  sprintf(eventname, "TAU_SIGNAL (%s)", strsignal(sig));
+  snprintf(eventname, sizeof(eventname),  "TAU_SIGNAL (%s)", strsignal(sig));
   TAU_REGISTER_CONTEXT_EVENT(evt, eventname);
   TAU_CONTEXT_EVENT(evt, 1);
   TAU_METADATA("SIGNAL", strsignal(sig));
@@ -356,10 +506,12 @@ int Tau_profile_exit_scorep()
 }
 
 /* disable this stupid thing */
+#ifndef TAU_WINDOWS
 extern "C" int __attribute__((weak)) PetscPopSignalHandler(void) {
     return 0;
 }
 typedef int(*PetscPopSignalHandler_p)(void);
+#endif
 
 //////////////////////////////////////////////////////////////////////
 // Initialize signal handling routines
@@ -492,6 +644,19 @@ alfred(void *arg)
 }
 #endif
 
+#ifdef TAU_ENABLE_LEVEL_ZERO
+void TauL0EnableProfiling(void);
+#endif
+
+#ifdef TAU_ENABLE_ROCPROFILERV2
+void Tau_rocm_initialize_v2(void);
+#endif
+
+#ifdef TAU_ENABLE_ROCPROFILERSDK
+void Tau_rocm_initialize_v3(void);
+#endif
+
+
 extern "C" int Tau_init_initializeTAU()
 {
 
@@ -537,6 +702,8 @@ extern "C" int Tau_init_initializeTAU()
   SCOREP_Tau_InitMeasurement();
   SCOREP_Tau_RegisterExitCallback(Tau_profile_exit_scorep);
   initialized = true;
+  tau_inside_initialize() = false;
+  tau_initialized() = 1;
   return 0;
 #endif
 
@@ -567,11 +734,30 @@ extern "C" int Tau_init_initializeTAU()
 
   Tau_call_post_init_callbacks();
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 	//DO NOT MOVE OR FACE ALISTER'S WRATH.
   Tau_cupti_post_init(); //MUST HAPPEN AFTER TAUMETRICS_INIT()
+#ifdef TAU_CUPTIPC
+  if(TauEnv_get_tauCuptiPC())
+  {
+      cupti_pcsampling_init();
+  }
+#endif //TAU_CUPTIPC
+#endif //TAU_CUPTI
+
+#ifdef TAU_ENABLE_LEVEL_ZERO
+  TauL0EnableProfiling();
 #endif
 
+#ifdef TAU_ENABLE_ROCPROFILERV2 
+  Tau_rocm_initialize_v2();
+#endif
+
+#ifdef TAU_ENABLE_ROCPROFILERSDK
+  Tau_rocm_initialize_v3();
+#endif
+
+
   // Mark initialization complete so calls below can start timers
   tau_initialized() = 1;
 
@@ -582,13 +768,25 @@ extern "C" int Tau_init_initializeTAU()
     Tau_compensate_initialization();
   }
 
+  /* Start a top level timer BEFORE we start sampling */
+  Tau_create_top_level_timer_if_necessary();
+
   /* initialize sampling if requested */
 #if !defined(TAU_MPI) && !defined(TAU_WINDOWS)
   if (TauEnv_get_ebs_enabled()) {
-    // Work-around for MVAPHICH 2 to move sampling initialization to after MPI_Init()
+    // As a Work-around for MVAPHICH 2 to move sampling initialization to after MPI_Init(),
+    // don't initialize sampling if this is an MPI build of TAU
     Tau_sampling_init_if_necessary();
   }
-#endif /* TAU_MPI && TAU_WINDOWS */
+#endif /* !TAU_MPI && !TAU_WINDOWS */
+
+#if defined(TAU_MPI) && !defined(TAU_WINDOWS)
+  if (TauEnv_get_ebs_enabled()) {
+    // As a Work-around for MVAPHICH 2 to move sampling initialization to after MPI_Init(),
+    // defer sampling to after MPI_Init is called if this is an MPI build of TAU
+    Tau_sampling_defer_init();
+  }
+#endif 
 
 #ifdef TAU_PGI
   sbrk(102400);
@@ -610,8 +808,6 @@ extern "C" int Tau_init_initializeTAU()
   }
 #endif
 
-  Tau_create_top_level_timer_if_necessary();
-
   Tau_memory_wrapper_enable();
 
 #ifdef TAU_ANDROID
