@@ -67,6 +67,7 @@ double TauWindowsUsecD(); // from RtsLayer.cpp
 #endif /* TAU_CRAYCNL_PMI || TAU_CRAYCNL_PMI_FOUND */
 #endif
 #endif//TAU_CRAYCNL
+#include <mutex>
 
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
@@ -186,14 +187,22 @@ int tau_bgq_init(void) {
  * instability if the application isn't linked with the TAU shared
  * object library (-optShared). */
 
+std::mutex _map_mutex;
 // These come from Tau_metadata_register calls
 MetaDataRepo &Tau_metadata_getMetaData(int tid) {
+
 /* Intel is such an annoying beast.  It won't let us declare this
  * as a static member object in this function.  This may cause
  * instability if the application isn't linked with the TAU shared
  * object library (-optShared). */
-  static MetaDataRepo metadata[TAU_MAX_THREADS];
-  return metadata[tid];
+  static vector<MetaDataRepo*> metadata;
+  
+  while(metadata.size()<=tid){
+        RtsLayer::LockDB();
+		metadata.push_back(new MetaDataRepo);
+        RtsLayer::UnLockDB();
+	}
+  return *metadata[tid];
 }
 
 void MetaDataRepo::freeMetadata (Tau_metadata_value_t * tmv) {
@@ -290,9 +299,12 @@ extern "C" void Tau_metadata_task(const char *name, const char *value, int tid)
   Tau_metadata_value_t* tmv = NULL;
   Tau_metadata_create_value(&tmv, TAU_METADATA_TYPE_STRING);
   tmv->data.cval = strdup(value);
-  //RtsLayer::LockEnv();
-  Tau_metadata_getMetaData(tid)[key] = tmv;
-  //RtsLayer::UnLockEnv();
+  {
+    std::lock_guard<std::mutex> lck(_map_mutex);
+    //Tau_metadata_getMetaData(tid)[key] = tmv;
+    Tau_metadata_getMetaData(tid).insert(std::make_pair(key, tmv));
+  }
+  //printf("%d %s = %s\n", 0, name, value);
   //printf("Metadata for thread %d : %s : %s\n", tid, key.name, tmv->data.cval);
 
   /*Invoke plugins only if both plugin path and plugins are specified*/
@@ -312,6 +324,7 @@ extern "C" void Tau_metadata_task(const char *name, const char *value, int tid)
  * metadata registered on thread 0 up to this point. */
 void Tau_metadata_push_to_plugins(void) {
     int tid = RtsLayer::myThread();
+    std::lock_guard<std::mutex> lck(_map_mutex);
     for (MetaDataRepo::iterator it = Tau_metadata_getMetaData(tid).begin();
          it != Tau_metadata_getMetaData(tid).end(); it++) {
 
@@ -332,7 +345,7 @@ extern "C" void Tau_metadata(const char *name, const char *value) {
 
 void Tau_metadata_register(const char *name, int value) {
   char buf[256];
-  sprintf (buf, "%d", value);
+  snprintf (buf, sizeof(buf),  "%d", value);
   Tau_metadata(name, buf);
 }
 
@@ -344,6 +357,20 @@ void Tau_metadata_register(const char *name, const std::string &value) {
   Tau_metadata(name, value.c_str());
 }
 
+void Tau_metadata_register_task(const char *name, int value, int tid) {
+  char buf[256];
+  snprintf (buf, sizeof(buf),  "%d", value);
+  Tau_metadata_task(name, buf, tid);
+}
+
+void Tau_metadata_register_task(const char *name, const char *value, int tid) {
+  Tau_metadata_task(name, value, tid);
+}
+
+void Tau_metadata_register_task(const char *name, const std::string &value, int tid) {
+  Tau_metadata_task(name, value.c_str(), tid);
+}
+
 #ifdef TAU_WINDOWS
 const char *Tau_metadata_timeFormat = "%I64d";
 #else
@@ -353,6 +380,11 @@ const char *Tau_metadata_timeFormat = "%lld";
 int Tau_metadata_fillMetaData()
 {
 #ifndef TAU_DISABLE_METADATA
+  if(TauEnv_get_disable_metadata()) {
+    TAU_VERBOSE("Metadata is being skipped because TAU_DISABLE_METADATA is set\n");
+    return 0;
+  }
+  int anonymize=TauEnv_get_anonymize_enabled();
 
   static int filled = 0;
   if (filled) {
@@ -362,7 +394,7 @@ int Tau_metadata_fillMetaData()
 
 
   char tmpstr[4096];
-  sprintf (tmpstr, Tau_metadata_timeFormat, TauMetrics_getInitialTimeStamp());
+  snprintf (tmpstr, sizeof(tmpstr),  Tau_metadata_timeFormat, TauMetrics_getInitialTimeStamp());
   Tau_metadata_register("Starting Timestamp", tmpstr);
 
 
@@ -385,12 +417,12 @@ int Tau_metadata_fillMetaData()
     tzone[4] = tzone[3];
     tzone[3] = ':';
   }
-  sprintf (tmpstr, "%s%s", buf, tzone);
+  snprintf (tmpstr, sizeof(tmpstr),  "%s%s", buf, tzone);
 
   Tau_metadata_register("Local Time", tmpstr);
 
   // write out the timestamp (number of microseconds since epoch (unsigned long long)
-  sprintf (tmpstr, Tau_metadata_timeFormat, TauMetrics_getTimeOfDay());
+  snprintf (tmpstr, sizeof(tmpstr),  Tau_metadata_timeFormat, TauMetrics_getTimeOfDay());
   Tau_metadata_register("Timestamp", tmpstr);
 
 
@@ -426,14 +458,14 @@ int Tau_metadata_fillMetaData()
   rts_get_personality(&personality, sizeof(personality));
   BGLPersonality_getLocationString(&personality, location);
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_xCoord(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_xCoord(&personality),
       BGLPersonality_yCoord(&personality),
       BGLPersonality_zCoord(&personality));
   Tau_metadata_register("BGL Coords", bglbuffer);
 
   Tau_metadata_register("BGL Processor ID", rts_get_processor_id());
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_xSize(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_xSize(&personality),
       BGLPersonality_ySize(&personality),
       BGLPersonality_zSize(&personality));
   Tau_metadata_register("BGL Size", bglbuffer);
@@ -445,7 +477,7 @@ int Tau_metadata_fillMetaData()
     Tau_metadata_register("BGL Node Mode", "Coprocessor");
   }
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_isTorusX(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_isTorusX(&personality),
       BGLPersonality_isTorusY(&personality),
       BGLPersonality_isTorusZ(&personality));
   Tau_metadata_register("BGL isTorus", bglbuffer);
@@ -459,17 +491,17 @@ int Tau_metadata_fillMetaData()
   Tau_metadata_register("BGL psetNum", BGLPersonality_psetNum(&personality));
   Tau_metadata_register("BGL numPsets", BGLPersonality_numPsets(&personality));
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_xPsetSize(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_xPsetSize(&personality),
       BGLPersonality_yPsetSize(&personality),
       BGLPersonality_zPsetSize(&personality));
   Tau_metadata_register("BGL PsetSize", bglbuffer);
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_xPsetOrigin(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_xPsetOrigin(&personality),
       BGLPersonality_yPsetOrigin(&personality),
       BGLPersonality_zPsetOrigin(&personality));
   Tau_metadata_register("BGL PsetOrigin", bglbuffer);
 
-  sprintf (bglbuffer, "(%d,%d,%d)", BGLPersonality_xPsetCoord(&personality),
+  snprintf (bglbuffer, sizeof(bglbuffer),  "(%d,%d,%d)", BGLPersonality_xPsetCoord(&personality),
       BGLPersonality_yPsetCoord(&personality),
       BGLPersonality_zPsetCoord(&personality));
   Tau_metadata_register("BGL PsetCoord", bglbuffer);
@@ -483,14 +515,14 @@ int Tau_metadata_fillMetaData()
   Kernel_GetPersonality(&personality, sizeof(_BGP_Personality_t));
   BGP_Personality_getLocationString(&personality, location);
 
-  sprintf (bgpbuffer, "(%d,%d,%d)", BGP_Personality_xCoord(&personality),
+  snprintf (bgpbuffer, sizeof(bgpbuffer),  "(%d,%d,%d)", BGP_Personality_xCoord(&personality),
       BGP_Personality_yCoord(&personality),
       BGP_Personality_zCoord(&personality));
   Tau_metadata_register("BGP Coords", bgpbuffer);
 
   Tau_metadata_register("BGP Processor ID", Kernel_PhysicalProcessorID());
 
-  sprintf (bgpbuffer, "(%d,%d,%d)", BGP_Personality_xSize(&personality),
+  snprintf (bgpbuffer, sizeof(bgpbuffer),  "(%d,%d,%d)", BGP_Personality_xSize(&personality),
       BGP_Personality_ySize(&personality),
       BGP_Personality_zSize(&personality));
   Tau_metadata_register("BGP Size", bgpbuffer);
@@ -499,11 +531,11 @@ int Tau_metadata_fillMetaData()
   if (Kernel_ProcessCount() > 1) {
     Tau_metadata_register("BGP Node Mode", "Virtual");
   } else {
-    sprintf(bgpbuffer, "Coprocessor (%d)", Kernel_ProcessCount());
+    snprintf(bgpbuffer, sizeof(bgpbuffer),  "Coprocessor (%d)", Kernel_ProcessCount());
     Tau_metadata_register("BGP Node Mode", bgpbuffer);
   }
 
-  sprintf (bgpbuffer, "(%d,%d,%d)", BGP_Personality_isTorusX(&personality),
+  snprintf (bgpbuffer, sizeof(bgpbuffer),  "(%d,%d,%d)", BGP_Personality_isTorusX(&personality),
       BGP_Personality_isTorusY(&personality),
       BGP_Personality_isTorusZ(&personality));
   Tau_metadata_register("BGP isTorus", bgpbuffer);
@@ -558,14 +590,14 @@ int Tau_metadata_fillMetaData()
   int z = tau_torus_coord[TAU_BGQ_IDX(4)] * tau_torus_size[TAU_BGQ_IDX(5)]
     + tau_torus_coord[TAU_BGQ_IDX(5)];
 
-  sprintf(bgqbuffer, "(%d,%d,%d)", x,y,z);
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "(%d,%d,%d)", x,y,z);
   Tau_metadata_register("BGQ Coords", bgqbuffer);
 
   int size_x = tau_torus_size[TAU_BGQ_IDX(0)] * tau_torus_size[TAU_BGQ_IDX(1)];
   int size_y = tau_torus_size[TAU_BGQ_IDX(2)] * tau_torus_size[TAU_BGQ_IDX(3)];
   int size_z = tau_torus_size[TAU_BGQ_IDX(4)] * tau_torus_size[TAU_BGQ_IDX(5)];
 
-  sprintf(bgqbuffer, "(%d,%d,%d,%d,%d,%d)", tau_torus_size[0], tau_torus_size[1], tau_torus_size[2],
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "(%d,%d,%d,%d,%d,%d)", tau_torus_size[0], tau_torus_size[1], tau_torus_size[2],
       tau_torus_size[3], tau_torus_size[4], tau_torus_size[5]);
   Tau_metadata_register("BGQ Size", bgqbuffer);
 
@@ -573,62 +605,62 @@ int Tau_metadata_fillMetaData()
   int wrap_y = tau_torus_wraparound[TAU_BGQ_IDX(2)] && tau_torus_wraparound[TAU_BGQ_IDX(3)];
   int wrap_z = tau_torus_wraparound[TAU_BGQ_IDX(4)] && tau_torus_wraparound[TAU_BGQ_IDX(5)];
 
-  sprintf(bgqbuffer, "(%d,%d,%d)", wrap_x,wrap_y,wrap_z);
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "(%d,%d,%d)", wrap_x,wrap_y,wrap_z);
   Tau_metadata_register("BGQ Period", bgqbuffer);
 
   BG_UniversalComponentIdentifier uci = tau_bgq_personality.Kernel_Config.UCI;
   unsigned int row, col, mp, nb, cc;
   bg_decodeComputeCardOnNodeBoardUCI(uci, &row, &col, &mp, &nb, &cc);
-  sprintf(bgqbuffer, "R%x%x-M%d-N%02x-J%02x <%d,%d,%d,%d,%d>", row, col, mp, nb, cc,
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "R%x%x-M%d-N%02x-J%02x <%d,%d,%d,%d,%d>", row, col, mp, nb, cc,
       tau_torus_coord[0], tau_torus_coord[1], tau_torus_coord[2],
       tau_torus_coord[3], tau_torus_coord[4]);
   Tau_metadata_register("BGQ Node Name", bgqbuffer);
 
-  sprintf(bgqbuffer, "%ld", ((uci>>38)&0xFFFFF)); /* encode row,col,mp,nb,cc*/
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%ld", ((uci>>38)&0xFFFFF)); /* encode row,col,mp,nb,cc*/
   Tau_metadata_register("BGQ Node ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%ld", Kernel_PhysicalProcessorID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%ld", Kernel_PhysicalProcessorID());
   Tau_metadata_register("BGQ Physical Processor ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", tau_bgq_personality.Kernel_Config.FreqMHz);
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", tau_bgq_personality.Kernel_Config.FreqMHz);
   Tau_metadata_register("CPU MHz", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_GetJobID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_GetJobID());
   Tau_metadata_register("BGQ Job ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_ProcessorID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_ProcessorID());
   Tau_metadata_register("BGQ Processor ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_PhysicalHWThreadID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_PhysicalHWThreadID());
   Tau_metadata_register("BGQ Physical HW Thread ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_ProcessCount());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_ProcessCount());
   Tau_metadata_register("BGQ Process Count", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_ProcessorCount());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_ProcessorCount());
   Tau_metadata_register("BGQ Processor Count", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_MyTcoord());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_MyTcoord());
   Tau_metadata_register("BGQ tCoord", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_ProcessorCoreID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_ProcessorCoreID());
   Tau_metadata_register("BGQ Processor Core ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_ProcessorThreadID());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_ProcessorThreadID());
   Tau_metadata_register("BGQ Processor Thread ID", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", Kernel_BlockThreadId());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_BlockThreadId());
   Tau_metadata_register("BGQ Block Thread ID", bgqbuffer);
 
   // Returns the Rank associated with the current process
-  sprintf(bgqbuffer, "%d", Kernel_GetRank());
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", Kernel_GetRank());
   Tau_metadata_register("BGQ Rank", bgqbuffer);
 
-  sprintf(bgqbuffer, "%d", tau_bgq_personality.DDR_Config.DDRSizeMB);
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "%d", tau_bgq_personality.DDR_Config.DDRSizeMB);
   Tau_metadata_register("BGQ DDR Size (MB)", bgqbuffer);
 
   // I/O Bridge Network
-  sprintf(bgqbuffer, "(%d, %d, %d, %d, %d)",
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "(%d, %d, %d, %d, %d)",
 	tau_bgq_personality.Network_Config.cnBridge_A,
 	tau_bgq_personality.Network_Config.cnBridge_B,
 	tau_bgq_personality.Network_Config.cnBridge_C,
@@ -637,7 +669,7 @@ int Tau_metadata_fillMetaData()
 
   Tau_metadata_register("BGQ Bridge I/O Coordinates", bgqbuffer);
 
-  sprintf(bgqbuffer, "(%d, %d, %d, %d, %d)",
+  snprintf(bgqbuffer, sizeof(bgqbuffer),  "(%d, %d, %d, %d, %d)",
 	tau_bgq_personality.Network_Config.Acoord,
 	tau_bgq_personality.Network_Config.Bcoord,
 	tau_bgq_personality.Network_Config.Ccoord,
@@ -726,36 +758,79 @@ int Tau_metadata_fillMetaData()
     fclose(f);
   }
 
-  char buffer[4096];
-  bzero(buffer, 4096);
-  int rc = readlink("/proc/self/exe", buffer, 4096);
-  if (rc != -1) {
-    Tau_metadata_register("Executable", buffer);
-  }
-
-  bzero(buffer, 4096);
-  rc = readlink("/proc/self/cwd", buffer, 4096);
-  if (rc != -1) {
-    Tau_metadata_register("CWD", buffer);
-  }
-
-
-  f = fopen("/proc/self/cmdline", "r");
+  f = fopen("/proc/self/status", "r");
   if (f) {
     char line[4096];
-
-    string os;
-    // *CWL* - The following loop performs newline to space conversions
     while (Tau_util_readFullLine(line, f)) {
-      if (os.length() != 0) {
-        os.append(" ");
+      char const * value = strstr(line,":");
+
+      if (!value) {
+        break;
+      } else {
+        value += 2;
       }
-      os.append(line);
+
+      // Allocates a string
+      value = Tau_util_removeRuns(value);
+
+/* capture these:
+Cpus_allowed:	ffffffff,ffffffff,ffffffff,ffffffff
+Cpus_allowed_list:	0-127
+Mems_allowed:	00000000,0000000f
+Mems_allowed_list:	0-3
+*/
+      if (strncmp(line, "Cpus_allowed", 12) == 0) {
+        Tau_metadata_register("CPUs Allowed", value);
+      }
+      if (strncmp(line, "Cpus_allowed_list", 17) == 0) {
+        Tau_metadata_register("CPUs Allowed List", value);
+      }
+      if (strncmp(line, "Mems_allowed", 12) == 0) {
+        Tau_metadata_register("Memories Allowed", value);
+      }
+      if (strncmp(line, "Mems_allowed_list", 17) == 0) {
+        Tau_metadata_register("Memories Allowed List", value);
+      }
+
+      free((void*)value);
     }
-    Tau_metadata_register("Command Line", os.c_str());
     fclose(f);
   }
 
+  /* If TAU_ANONYMIZE is used, don't write out executable and cwd info */
+  if (!anonymize) {
+    char buffer[4096];
+    bzero(buffer, 4096);
+
+    int rc = readlink("/proc/self/exe", buffer, 4096);
+    if (rc != -1) {
+      Tau_metadata_register("Executable", buffer);
+    }
+
+    bzero(buffer, 4096);
+    rc = readlink("/proc/self/cwd", buffer, 4096);
+    if (rc != -1) {
+      Tau_metadata_register("CWD", buffer);
+    }
+
+
+    f = fopen("/proc/self/cmdline", "r");
+    if (f) {
+      char line[4096];
+
+      string os;
+      // *CWL* - The following loop performs newline to space conversions
+      while (Tau_util_readFullLine(line, f)) {
+        if (os.length() != 0) {
+          os.append(" ");
+        }
+        os.append(line);
+      }
+      Tau_metadata_register("Command Line", os.c_str());
+      fclose(f);
+    }
+  }
+
 #elif defined(__APPLE__)
 
   char buffer[4096];
@@ -775,16 +850,31 @@ int Tau_metadata_fillMetaData()
 
 #endif /* __linux__ */
 
-  char *user = getenv("USER");
-  if (user != NULL) {
-    Tau_metadata_register("username", user);
+  if (!anonymize) {
+    char *user = getenv("USER");
+    if (user != NULL) {
+      Tau_metadata_register("username", user);
+    }
   }
 
+
+#endif // TAU_DISABLE_METADATA
+
+  return 0;
+}
+
+// OpenMP metadata can't be collected until the OpenMP runtime is up.
+// This function will be called from a helper thread created in TauOMPT.cpp
+// after OMPT is initialized. 
+// Metadata registered in this function is explicitly created on tid 0
+// so that it will propagate to every thread.
+int Tau_metadata_fillOpenMPMetaData(void) {
 #ifdef _OPENMP
-#if defined(TAU_OPENMP) && !defined(TAU_MPC)
+#if defined(TAU_OPENMP) && !defined(TAU_MPC) && !defined(TAU_CLANG)
+  TAU_VERBOSE("Filling OpenMP-related metadata\n");
   /* Capture OpenMP version */
-  Tau_metadata_register("OpenMP Version String", _OPENMP);
-  Tau_metadata_register("OpenMP Version", TheOpenMPVersionMap().at(_OPENMP).c_str());
+  Tau_metadata_register_task("OpenMP Version String", _OPENMP, 0);
+  Tau_metadata_register_task("OpenMP Version", TheOpenMPVersionMap().at(_OPENMP).c_str(), 0);
 
 // MPC wht OpenMP isn't initialized before TAU is, so these function calls will hang.
   const char* schedule;
@@ -802,47 +892,48 @@ int Tau_metadata_fillMetaData()
   } else {
       schedule = "UNKNOWN";
   }
-  Tau_metadata_register("OMP_SCHEDULE", schedule);
-  Tau_metadata_register("OMP_CHUNK_SIZE", modifier);
-  Tau_metadata_register("OMP_MAX_THREADS", omp_get_max_threads());
-  Tau_metadata_register("OMP_NUM_PROCS", omp_get_num_procs());
-  Tau_metadata_register("OMP_DYNAMIC", omp_get_dynamic() ? "TRUE" : "FALSE");
+  Tau_metadata_register_task("OMP_SCHEDULE", schedule, 0);
+  Tau_metadata_register_task("OMP_CHUNK_SIZE", modifier, 0);
+  Tau_metadata_register_task("OMP_MAX_THREADS", omp_get_max_threads(), 0);
+  Tau_metadata_register_task("OMP_NUM_PROCS", omp_get_num_procs(), 0);
+  Tau_metadata_register_task("OMP_DYNAMIC", omp_get_dynamic() ? "TRUE" : "FALSE", 0);
   // Deprecated
-  //Tau_metadata_register("OMP_NESTED", omp_get_nested() ? "TRUE" : "FALSE");
-  Tau_metadata_register("OMP_MAX_ACTIVE_LEVELS", omp_get_max_active_levels());
-  Tau_metadata_register("OMP_THREAD_LIMIT", omp_get_thread_limit());
+  //Tau_metadata_register_task("OMP_NESTED", omp_get_nested() ? "TRUE" : "FALSE", 0);
+  Tau_metadata_register_task("OMP_MAX_ACTIVE_LEVELS", omp_get_max_active_levels(), 0);
+  Tau_metadata_register_task("OMP_THREAD_LIMIT", omp_get_thread_limit(), 0);
 
 #if _OPENMP >= 201307 // OpenMP 4.0
-  Tau_metadata_register("OMP_PROC_BIND", omp_get_proc_bind() ? "TRUE" : "FALSE");
-#if !defined(__INTEL_COMPILER)
+  Tau_metadata_register_task("OMP_PROC_BIND", omp_get_proc_bind() ? "TRUE" : "FALSE", 0);
+#if !defined(TAU_INTEL_COMPILER)
 /* Don't make this call for the Intel compiler!  It may compile, but without
  * MIC/KNL offload support, the linker will fail with really esoteric error messages
  * about either missing destructor in ~MetaDataRepo() or missing i_ofldbegin_target.o
  */
 #if !defined(TAU_CRAYCC_USE_CLANG)
-  Tau_metadata_register("OMP_DEFAULT_DEVICE", omp_get_default_device());
+  Tau_metadata_register_task("OMP_DEFAULT_DEVICE", omp_get_default_device(), 0);
 #endif
 #endif
 #if _OPENMP == 201307 && !defined(__PGI) // PGI claims 4.0, but is missing these implementations
-  Tau_metadata_register("OMP_CANCELLATION", omp_get_cancellation() ? "TRUE" : "FALSE");
-  Tau_metadata_register("OMP_NUM_DEVICES", omp_get_num_devices());
+  Tau_metadata_register_task("OMP_CANCELLATION", omp_get_cancellation() ? "TRUE" : "FALSE", 0);
+  Tau_metadata_register_task("OMP_NUM_DEVICES", omp_get_num_devices(), 0);
 #endif
 #endif
 
 #if _OPENMP >= 201511 // OpenMP 4.5
 #if (!defined (__PGI))
   // This API call crashes with NVCHPC 21.7, it doesn't tell us anything anyway
-  //Tau_metadata_register("OMP_MAX_TASK_PRIORITY", omp_get_max_task_priority());
+  //Tau_metadata_register_task("OMP_MAX_TASK_PRIORITY", omp_get_max_task_priority());
   char * omp_var = getenv("OMP_PLACES");
   if (omp_var != NULL) {
-    Tau_metadata_register("OMP_PLACES", omp_var);
+    Tau_metadata_register_task("OMP_PLACES", omp_var, 0);
   }
-  Tau_metadata_register("OMP_NUM_PLACES", omp_get_num_places());
+  auto places = omp_get_num_places();
+  Tau_metadata_register_task("OMP_NUM_PLACES", places, 0);
   stringstream ss_num;
   stringstream ss_ids;
   ss_num << "{";
   ss_ids << "{";
-  for (int i = 0 ; i < omp_get_num_places() ; i++) {
+  for (int i = 0 ; i < places ; i++) {
     if (i > 0) {
       ss_num << ",";
       ss_ids << ",";
@@ -861,17 +952,14 @@ int Tau_metadata_fillMetaData()
   }
   ss_num << "}";
   ss_ids << "}";
-  Tau_metadata_register("OMP_PLACE_NUM_PROCS", ss_num.str());
-  Tau_metadata_register("OMP_PLACE_PROC_IDS", ss_ids.str());
+  Tau_metadata_register_task("OMP_PLACE_NUM_PROCS", ss_num.str(), 0);
+  Tau_metadata_register_task("OMP_PLACE_PROC_IDS", ss_ids.str(), 0);
 #endif /* PGI - PrgEnv-nvidia */
 #endif
 
 #endif // TAU_OPENMP && !TAU_MPC
 
 #endif // _OPENMP
-
-#endif // TAU_DISABLE_METADATA
-
   return 0;
 }
 
@@ -892,15 +980,15 @@ extern "C" int writeMetaDataAfterMPI_Init(void) {
   switch (dim) {
   case 1:
     retcode = FJMPI_Topology_rank2x(RtsLayer::myNode(), &xrank);
-    sprintf (fbuffer, "(%d)", xrank);
+    snprintf (fbuffer, sizeof(fbuffer),  "(%d)", xrank);
     break;
   case 2:
     retcode = FJMPI_Topology_rank2xy(RtsLayer::myNode(), &xrank, &yrank);
-    sprintf (fbuffer, "(%d,%d)", xrank, yrank);
+    snprintf (fbuffer, sizeof(fbuffer),  "(%d,%d)", xrank, yrank);
     break;
   case 3:
     retcode = FJMPI_Topology_rank2xyz(RtsLayer::myNode(), &xrank, &yrank, &zrank);
-    sprintf (fbuffer, "(%d,%d,%d)", xrank, yrank, zrank);
+    snprintf (fbuffer, sizeof(fbuffer),  "(%d,%d,%d)", xrank, yrank, zrank);
     break;
   default:
     fprintf(stderr, "FJMPI_Topology_get_dimension ERROR in switch TauMetaData.cpp\n");
@@ -919,10 +1007,10 @@ extern "C" int writeMetaDataAfterMPI_Init(void) {
   }
 
 
-  sprintf (fbuffer, "(%d,%d,%d)", xshape, yshape, zshape);
+  snprintf (fbuffer, sizeof(fbuffer),  "(%d,%d,%d)", xshape, yshape, zshape);
   Tau_metadata_register("FUJITSU Size", fbuffer);
 
-  sprintf (fbuffer, "%d", dim);
+  snprintf (fbuffer, sizeof(fbuffer),  "%d", dim);
   Tau_metadata_register("FUJITSU Dimension", fbuffer);
 
 #endif /* TAU_FUJITSU && TAU_MPI */
@@ -930,7 +1018,7 @@ extern "C" int writeMetaDataAfterMPI_Init(void) {
 /****** CRAY RELATED DATA *********/
 #ifdef TAU_CRAYCNL
   FILE* procfile = fopen("/proc/self/stat", "r");
-  long to_read = 8192;
+  const size_t to_read = 8192;
   char buffer[to_read];
   if (procfile != NULL) {
     int read = fread(buffer, sizeof(char), to_read, procfile);
@@ -994,15 +1082,7 @@ extern "C" int writeMetaDataAfterMPI_Init(void) {
   return 0;
 }
 
-extern "C" void Tau_metadata_writeEndingTimeStamp(void) {
-  char tmpstr[4096];
-  TauMetrics_finalize(); // make sure there's a valid final timestamp
-  sprintf (tmpstr, Tau_metadata_timeFormat, TauMetrics_getFinalTimeStamp());
-  Tau_metadata_register("Ending Timestamp", tmpstr);
-}
-
 static int writeMetaData(Tau_util_outputDevice *out, bool newline, int counter, int tid) {
-  Tau_metadata_writeEndingTimeStamp();
 
   const char *endl = "";
   //newline = true;
@@ -1017,6 +1097,7 @@ static int writeMetaData(Tau_util_outputDevice *out, bool newline, int counter,
     Tau_XML_writeAttribute(out, "Metric Name", RtsLayer::getCounterName(counter), newline);
   }
 
+  //printf("Writing metadata for %d, its map has %lu items\n", tid, Tau_metadata_getMetaData(tid).size());
   /*
    * In order to support thread-specific metadata, we will need to aggregate
    * the metadata which is common to all threads in this process (thread 0
@@ -1032,15 +1113,18 @@ static int writeMetaData(Tau_util_outputDevice *out, bool newline, int counter,
     // create a new aggregator
     localRepo = new MetaDataRepo();
 	// copy all metadata from thread 0
+    std::lock_guard<std::mutex> lck(_map_mutex);
     for (MetaDataRepo::iterator it = Tau_metadata_getMetaData(0).begin(); it != Tau_metadata_getMetaData(0).end(); it++) {
 	  // DON'T copy the context metadata fields
 	  if (it->first.timer_context == NULL) {
-        (*localRepo)[it->first] = it->second;
+        //(*localRepo)[it->first] = it->second;
+        (*localRepo).insert(std::make_pair(it->first, it->second));
 	  }
 	}
  	// overwrite with thread-specific data
     for (MetaDataRepo::iterator it = Tau_metadata_getMetaData(tid).begin(); it != Tau_metadata_getMetaData(tid).end(); it++) {
-      (*localRepo)[it->first] = it->second;
+      //(*localRepo)[it->first] = it->second;
+      (*localRepo).insert(std::make_pair(it->first, it->second));
 	}
   }
   /*
@@ -1069,6 +1153,16 @@ static int writeMetaData(Tau_util_outputDevice *out, bool newline, int counter,
 	//}
 	i++;
   }
+  if (RtsLayer::myThread() == 0) {
+      char tmpstr[4096];
+      snprintf (tmpstr, sizeof(tmpstr),  Tau_metadata_timeFormat, TauMetrics_getFinalTimeStamp());
+      Tau_metadata_register("Ending Timestamp", tmpstr);
+#ifndef TAU_SCOREP
+      Tau_XML_writeAttribute(out, "Ending Timestamp", tmpstr, newline);
+#elif TAU_SCOREP_METADATA /* TAU_SCOREP */
+      SCOREP_Tau_AddLocationProperty("Ending Timestamp", tmpstr);
+#endif
+  }
 
 // can't do full delete, because the aggregation does not do deep copies. :(
 /* Don't delete the repo now! If Tau_dump() is called before exit, this will
@@ -1104,8 +1198,9 @@ extern "C" void Tau_context_metadata(const char *name, const char *value) {
   // it IS possible to request metadata with no active timer.
   if (current != NULL) {
     FunctionInfo *fi = current->ThisFunction;
-    char *fname = (char*)(malloc(sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2)));
-    sprintf(fname, "%s %s", fi->GetName(), fi->GetType());
+    const int len = sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2);
+    char *fname = (char*)(malloc(len));
+    snprintf(fname, len,  "%s %s", fi->GetName(), fi->GetType());
 	key->timer_context = fname;
 	key->call_number = fi->GetCalls(tid);
 	key->timestamp = (x_uint64)current->StartTime[0];
@@ -1114,9 +1209,10 @@ extern "C" void Tau_context_metadata(const char *name, const char *value) {
   Tau_metadata_value_t* tmv = NULL;
   Tau_metadata_create_value(&tmv, TAU_METADATA_TYPE_STRING);
   tmv->data.cval = strdup(value);
-  //RtsLayer::LockEnv();
-  Tau_metadata_getMetaData(tid)[*key] = tmv;
-  //RtsLayer::UnLockEnv();
+  //printf("%d CONTEXT %s = %s\n", tid, name, value);
+  //printf("Metadata for thread %d : %s : %s\n", tid, key->name, tmv->data.cval);
+  std::lock_guard<std::mutex> lck(_map_mutex);
+  Tau_metadata_getMetaData(tid).insert(std::make_pair(*key, tmv));
 #endif
 }
 
@@ -1128,14 +1224,14 @@ extern "C" void Tau_structured_metadata(const Tau_metadata_object_t *object, boo
   int tid = RtsLayer::myThread();
   Tau_metadata_key *key = new Tau_metadata_key();
   if (context) {
-    RtsLayer::LockEnv();
     // get the current calling context
     Profiler *current = TauInternal_CurrentProfiler(tid);
     // it IS possible to request metadata with no active timer.
     if (current != NULL) {
       FunctionInfo *fi = current->ThisFunction;
-      char *fname = (char*)(malloc(sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2)));
-      sprintf(fname, "%s %s", fi->GetName(), fi->GetType());
+      const int len = sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2);
+      char *fname = (char*)(malloc(len));
+      snprintf(fname, len,  "%s %s", fi->GetName(), fi->GetType());
 	  key->timer_context = fname;
 	  key->call_number = fi->GetCalls(tid);
 	  key->timestamp = (x_uint64)current->StartTime[0];
@@ -1146,9 +1242,11 @@ extern "C" void Tau_structured_metadata(const Tau_metadata_object_t *object, boo
     key->name = strdup(object->names[i]);
     Tau_metadata_value_t* tmv = object->values[i];
     //printf("%p  %s:%s:%d:%llu = %s\n", &(Tau_metadata_getMetaData(RtsLayer::myThread())), key->name, key->timer_context, key->call_number, key->timestamp, tmv->data.cval);
-    Tau_metadata_getMetaData(tid)[*key] = tmv;
+  std::lock_guard<std::mutex> lck(_map_mutex);
+    //Tau_metadata_getMetaData(tid)[*key] = tmv;
+  Tau_metadata_getMetaData(tid).insert(std::make_pair(*key, tmv));
+  //printf("Metadata for thread %d : %s : %s\n", tid, key->name, tmv->data.cval);
   }
-  RtsLayer::UnLockEnv();
 #endif
 }
 
@@ -1166,8 +1264,9 @@ extern "C" void Tau_phase_metadata(const char *name, const char *value) {
   while (current != NULL) {
     if (current->GetPhase()) {
       FunctionInfo *fi = current->ThisFunction;
-      char *fname = (char*)(malloc(sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2)));
-      sprintf(fname, "%s %s", fi->GetName(), fi->GetType());
+      const int len = sizeof(char)*(strlen(fi->GetName()) + strlen(fi->GetType()) + 2);
+      char *fname = (char*)(malloc(len));
+      snprintf(fname, len,  "%s %s", fi->GetName(), fi->GetType());
 	  key->timer_context = fname;
 	  key->call_number = fi->GetCalls(tid);
 	  key->timestamp = (x_uint64)current->StartTime[0];
@@ -1178,9 +1277,10 @@ extern "C" void Tau_phase_metadata(const char *name, const char *value) {
   Tau_metadata_value_t* tmv = NULL;
   Tau_metadata_create_value(&tmv, TAU_METADATA_TYPE_STRING);
   tmv->data.cval = strdup(value);
-  //RtsLayer::LockEnv();
-  Tau_metadata_getMetaData(tid)[*key] = tmv;
-  //RtsLayer::UnLockEnv();
+  std::lock_guard<std::mutex> lck(_map_mutex);
+  //Tau_metadata_getMetaData(tid)[*key] = tmv;
+  Tau_metadata_getMetaData(tid).insert(std::make_pair(*key, tmv));
+  //printf("Metadata for thread %d : %s : %s\n", tid, key.name, tmv->data.cval);
 #else
   Tau_context_metadata(name, value);
 #endif
@@ -1277,6 +1377,7 @@ void Tau_metadata_removeDuplicates(char *buffer, int buflen) {
 
     Tau_metadata_key key;
 	key.name = (char*)attribute;
+  std::lock_guard<std::mutex> lck(_map_mutex);
     MetaDataRepo::iterator iter = Tau_metadata_getMetaData(RtsLayer::myThread()).find(key);
     if (iter != Tau_metadata_getMetaData(RtsLayer::myThread()).end()) {
 	  if (iter->second->type == TAU_METADATA_TYPE_STRING) {
@@ -1299,6 +1400,7 @@ char * Tau_metadata_get(const char * name, int tid) {
     char * returnval = NULL;
     Tau_metadata_key key;
     key.name = strdup(name);
+  std::lock_guard<std::mutex> lck(_map_mutex);
     MetaDataRepo::iterator it = Tau_metadata_getMetaData(tid).find(key);
     if (it != Tau_metadata_getMetaData(tid).end()) {
 	  if (it->second->type == TAU_METADATA_TYPE_STRING) {
