@@ -9,6 +9,9 @@
 
 #include <vector>
 #include <map>
+#include <mutex>
+
+#include <Profile/TauEnv.h>
 
 #define TAU_MEMMGR_MAP_CREATION_FAILED -1
 #define TAU_MEMMGR_MAX_MEMBLOCKS_REACHED -2
@@ -22,8 +25,8 @@
 
 struct TauMemMgrSummary
 {
-  int numBlocks;
-  unsigned long totalAllocatedMemory;
+  int numBlocks=0;
+  unsigned long totalAllocatedMemory=0;
 };
 
 struct TauMemMgrInfo
@@ -34,11 +37,13 @@ struct TauMemMgrInfo
   unsigned long high;
 };
 
-TauMemMgrSummary memSummary[TAU_MAX_THREADS];
-TauMemMgrInfo memInfo[TAU_MAX_THREADS][TAU_MEMMGR_MAX_MEMBLOCKS];
+std::mutex& getMapMutex() {
+    static std::mutex _mtx;
+    return _mtx;
+}
 
 /* For "freeing" memory, we need to maintain a map of "queues"
- * for each thread.  The map will map from a "length" to a 
+ * for each thread.  The map will map from a "length" to a
  * queue of free blocks of that length. Each thread will have
  * its own map. The map and the queue require the custom allocator,
  * so this memory manager will eat its own dog food, so to speak.
@@ -50,8 +55,59 @@ TauMemMgrInfo memInfo[TAU_MAX_THREADS][TAU_MEMMGR_MAX_MEMBLOCKS];
 typedef std::vector<void*, TauSignalSafeAllocator<void*> > __custom_vector_t;
 typedef std::pair<const std::size_t, std::vector<void*> > __custom_pair_t;
 typedef std::map<std::size_t, __custom_vector_t*, std::less<std::size_t>, TauSignalSafeAllocator<__custom_pair_t> > __custom_map_t;
-__custom_map_t free_chunks[TAU_MAX_THREADS];
 #endif
+
+struct TMMUnit
+{
+    TauMemMgrSummary memSummary;
+    TauMemMgrInfo* memInfo;
+#ifdef USE_RECYCLER
+    __custom_map_t free_chunks;
+#endif
+};
+
+std::mutex TMMVectorMutex;
+static vector<TMMUnit*> TMMList;
+
+inline void checkTMMVector(int tid){
+	while(TMMList.size()<=tid){
+    //std::lock_guard<std::mutex> guard(TMMVectorMutex);
+    TMMUnit* thread_TTM = new TMMUnit();
+    thread_TTM->memInfo = (TauMemMgrInfo*) malloc(TauEnv_get_env_memmgr_max_memblocks()*sizeof(TauMemMgrInfo));
+		TMMList.push_back(thread_TTM);
+	}
+}
+
+inline void freeTMMVector(){
+  int tid;
+  for( tid=0; tid<TMMList.size(); tid++ )
+	while(TMMList.size()<=tid){
+		free(TMMList[tid]->memInfo);
+	}
+}
+
+inline TauMemMgrSummary& getMemSummary(int tid){
+    std::lock_guard<std::mutex> guard(TMMVectorMutex);
+    checkTMMVector(tid);
+    return TMMList[tid]->memSummary;
+}
+inline TauMemMgrInfo& getMemInfo(int tid,int block){
+    std::lock_guard<std::mutex> guard(TMMVectorMutex);    
+    checkTMMVector(tid);
+    return TMMList[tid]->memInfo[block];
+}
+inline int getMemSumSize(){
+  std::lock_guard<std::mutex> guard(TMMVectorMutex);
+	return TMMList.size();
+}
+#ifdef USE_RECYCLER
+inline __custom_map_t& getFreeChunks(int tid){
+    std::lock_guard<std::mutex> guard(TMMVectorMutex);
+    checkTMMVector(tid);
+    return TMMList[tid]->free_chunks;
+}
+#endif
+
 bool finalized = false;
 
 bool Tau_MemMgr_initIfNecessary(void)
@@ -65,16 +121,16 @@ bool Tau_MemMgr_initIfNecessary(void)
   //   for the first time! Right now, the correct place for this
   //   is sampling init.
   if (!initialized) {
-    RtsLayer::LockEnv();
+    std::lock_guard<std::mutex> lck (getMapMutex());
     // check again, someone else might already have initialized by now.
     if (!initialized) {
-      for (int i = 0; i < TAU_MAX_THREADS; i++) {
-        memSummary[i].numBlocks = 0;
-        memSummary[i].totalAllocatedMemory = 0;
+      int maxThreads=getMemSumSize();
+      for (int i = 0; i < maxThreads; i++) { //TODO: DYNATHREAD. This may be unnecessary since these are allocated on demand now.
+        getMemSummary(i).numBlocks = 0;
+        getMemSummary(i).totalAllocatedMemory = 0;
       }
       initialized = true;
     }
-    RtsLayer::UnLockEnv();
   }
   return true;
 }
@@ -82,12 +138,12 @@ bool Tau_MemMgr_initIfNecessary(void)
 extern "C" void Tau_MemMgr_finalizeIfNecessary(void) {
   if (!finalized) {
     Tau_global_incr_insideTAU();
-    RtsLayer::LockEnv();
+    std::lock_guard<std::mutex> lck (getMapMutex());
     // check again, someone else might already have initialized by now.
     if (!finalized) {
+      freeTMMVector();
       finalized = true;
     }
-    RtsLayer::UnLockEnv();
     Tau_global_decr_insideTAU();
   }
 }
@@ -122,14 +178,14 @@ void *Tau_MemMgr_mmap(int tid, size_t size)
     fprintf(stderr, "Tau_MemMgr_mmap: mmap failed\n");
     addr = NULL;
   } else {
-    int numBlocks = memSummary[tid].numBlocks;
-    memInfo[tid][numBlocks].start = (unsigned long)addr;
-    memInfo[tid][numBlocks].size = size;
-    memInfo[tid][numBlocks].low = (unsigned long)addr;
-    memInfo[tid][numBlocks].high = (unsigned long)addr + size;
-    memSummary[tid].numBlocks++;
+    int numBlocks = getMemSummary(tid).numBlocks;
+    getMemInfo(tid,numBlocks).start = (unsigned long)addr;
+    getMemInfo(tid,numBlocks).size = size;
+    getMemInfo(tid,numBlocks).low = (unsigned long)addr;
+    getMemInfo(tid,numBlocks).high = (unsigned long)addr + size;
+    getMemSummary(tid).numBlocks++;
     //printf("********* %d: Incremented numblocks! %d\n", tid, memSummary[tid].numBlocks); fflush(stdout);
-    memSummary[tid].totalAllocatedMemory += size;
+    getMemSummary(tid).totalAllocatedMemory += size;
   }
 
 //  TAU_VERBOSE("Tau_MemMgr_mmap: tid=%d, size = %ld, fd = %d, addr = %p, blocks = %ld, used = %ld\n", tid, size, fd,
@@ -139,7 +195,7 @@ void *Tau_MemMgr_mmap(int tid, size_t size)
 
 int Tau_MemMgr_findFit(int tid, size_t size)
 {
-  int numBlocks = memSummary[tid].numBlocks;
+  int numBlocks = getMemSummary(tid).numBlocks;
   size_t blockSize = TAU_MEMMGR_DEFAULT_BLOCKSIZE;
   // If the request bigger than the default size.
   if (size > TAU_MEMMGR_DEFAULT_BLOCKSIZE) {
@@ -148,19 +204,20 @@ int Tau_MemMgr_findFit(int tid, size_t size)
 
   // Hunt for an existing block with sufficient memory.
   for (int i = 0; i < numBlocks; i++) {
-    if (memInfo[tid][i].high - memInfo[tid][i].low > size) {
+    if (getMemInfo(tid,i).high - getMemInfo(tid,i).low > size) {
       return i;
     }
   }
 
   // Didn't find any suitable block. Attempt to acquire a new one.
-  if (numBlocks < TAU_MEMMGR_MAX_MEMBLOCKS) {
+  if (numBlocks < TauEnv_get_env_memmgr_max_memblocks()) {
     if (!Tau_MemMgr_mmap(tid, blockSize)) {
       return TAU_MEMMGR_MAP_CREATION_FAILED;
     }
     // return index to new block
-    return memSummary[tid].numBlocks - 1;
+    return getMemSummary(tid).numBlocks - 1;
   } else {
+    printf("********* TAU_MEMMGR_MAX_MEMBLOCKS_REACHED\n Current is %d, increase environmental variable TAU_MEMMGR_MAX_BLOCKS\n", TauEnv_get_env_memmgr_max_memblocks()); fflush(stdout);
     return TAU_MEMMGR_MAX_MEMBLOCKS_REACHED;
   }
 }
@@ -171,32 +228,29 @@ void * Tau_MemMgr_recycle(int tid, size_t size)
     // does the map for this thread exist?
     // get the vector for this size
     __custom_vector_t * queue;
-    RtsLayer::LockEnv();
-    __custom_map_t::const_iterator it = free_chunks[tid].find(size);
+    std::lock_guard<std::mutex> lck (getMapMutex());
+    __custom_map_t::const_iterator it = getFreeChunks(tid).find(size);
 
     // is this a new size that we haven't freed yet?
-    if (it == free_chunks[tid].end()) {
-        RtsLayer::UnLockEnv();
+    if (it == getFreeChunks(tid).end()) {
         return NULL;
     } else {
         queue = (*it).second;
     }
     // Does this vector have a chunk for us to use?
     if (queue->empty()) {
-        RtsLayer::UnLockEnv();
         return NULL;
     }
     // there is a chunk! Recycle it!
     void * tmp = queue->back();
     queue->pop_back();
-    RtsLayer::UnLockEnv();
     return tmp;
 }
 #endif
 
 void * Tau_MemMgr_malloc(int tid, size_t size)
 {
-/* In some cases, when not using sampling or signal handling, we 
+/* In some cases, when not using sampling or signal handling, we
    want to disable the memory management altogether. */
 #ifdef TAU_DISABLE_MEM_MANAGER
     void * ptr = malloc(size);
@@ -212,10 +266,10 @@ void * Tau_MemMgr_malloc(int tid, size_t size)
 #ifdef USE_RECYCLER
   // can we recycle an old block?
   void * recycled = Tau_MemMgr_recycle(tid, size);
-  if (recycled != NULL) { 
+  if (recycled != NULL) {
     //printf("Recycling block of size %d at address %p\n", size, recycled);
     memset(recycled, 0, size);
-    return recycled; 
+    return recycled;
   }
 #endif
 
@@ -230,6 +284,7 @@ void * Tau_MemMgr_malloc(int tid, size_t size)
       break;
     case TAU_MEMMGR_MAX_MEMBLOCKS_REACHED:
       printf("Tau_MemMgr_malloc: MMAP MAX MEMBLOCKS REACHED!\n");
+      printf("Current is %d, increase environmental variable TAU_MEMMGR_MAX_BLOCKS\n", TauEnv_get_env_memmgr_max_memblocks()); fflush(stdout);
       break;
     default:
       printf("Tau_MemMgr_malloc: UNKNOWN ERROR!\n");
@@ -239,8 +294,8 @@ void * Tau_MemMgr_malloc(int tid, size_t size)
     return NULL;
   }
 
-  void * addr = (void *)((memInfo[tid][myBlock].low + (TAU_MEMMGR_ALIGN-1)) & ~(TAU_MEMMGR_ALIGN-1));
-  memInfo[tid][myBlock].low += myRequest;
+  void * addr = (void *)((getMemInfo(tid,myBlock).low + (TAU_MEMMGR_ALIGN-1)) & ~(TAU_MEMMGR_ALIGN-1));
+  getMemInfo(tid,myBlock).low += myRequest;
 
   TAU_ASSERT(addr != NULL, "Tau_MemMgr_malloc unexpectedly returning NULL!");
 
@@ -251,7 +306,7 @@ void * Tau_MemMgr_malloc(int tid, size_t size)
 
 void Tau_MemMgr_free(int tid, void *addr, size_t size)
 {
-/* In some cases, when not using sampling or signal handling, we 
+/* In some cases, when not using sampling or signal handling, we
    want to disable the memory management altogether. */
 #ifdef TAU_DISABLE_MEM_MANAGER
     free(addr);
@@ -264,25 +319,24 @@ void Tau_MemMgr_free(int tid, void *addr, size_t size)
     // freed memory just allocates more memory...
     if (finalized || size == 0) return;
 #ifdef USE_RECYCLER
-    RtsLayer::LockEnv();
+    std::lock_guard<std::mutex> lck (getMapMutex());
     // get the vector for this size
-    __custom_map_t::const_iterator it = free_chunks[tid].find(size);
+    __custom_map_t::const_iterator it = getFreeChunks(tid).find(size);
     __custom_vector_t * queue;
 
     // is this a new size that we haven't freed yet?
-    if (it == free_chunks[tid].end()) {
+    if (it == getFreeChunks(tid).end()) {
         // eat our own dog food.  Have to. Create the new vector...
         queue = (__custom_vector_t*)Tau_MemMgr_malloc(tid, sizeof(__custom_vector_t));
         // ...call its constructor...
         new(queue) __custom_vector_t();
         // ...and add the new vector into the map
-        free_chunks[tid][size] = queue;
+        getFreeChunks(tid)[size] = queue;
     } else {
         queue = (*it).second;
     }
     // Add this address to the end of the vector
     queue->push_back(addr);
-    RtsLayer::UnLockEnv();
 #endif
     return;
 }
@@ -290,6 +344,7 @@ void Tau_MemMgr_free(int tid, void *addr, size_t size)
 #else /* TAU_WINDOWS */
 #include <stdlib.h>
 extern "C" bool Tau_MemMgr_initIfNecessary(void) {
+	return true;
 }
 extern "C" void Tau_MemMgr_finalizeIfNecessary(void) {
 }
