@@ -26,10 +26,10 @@
 #include <openacc.h>
 #include "pgi_acc_prof.h"
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 #include <cupti.h>
 #include <cuda.h>
-#endif
+#endif //TAU_CUPTI
 
 #include <sstream>
 
@@ -37,31 +37,43 @@
 #include <Profile/TauOpenACC.h>
 #include <Profile/TauGpuAdapterOpenACC.h>
 #include <Profile/TauGpu.h>
+#include <Profile/TauBfd.h>
 
 #define TAU_SET_EVENT_NAME(event_name, str) strcpy(event_name, str)
 ////////////////////////////////////////////////////////////////////////////
 
 // strings for OpenACC parent constructs; based on enum acc_construct_t
 const char* acc_constructs[] = {
-    "parallel",
-    "kernels",
-    "loop",
-    "data",
-    "enter data",
-    "exit data",
-    "host data",
-    "atomic",
-    "declare",
-    "init",
-    "shutdown",
-    "set",
-    "update",
-    "routine",
-    "wait",
-    "runtime api",
-    "serial",
+    "parallel",   // 0
+    "kernels",   // 1
+    "loop",   // 2
+    "data",   // 3
+    "enter data",   // 4
+    "exit data",   // 5
+    "host data",   // 6
+    "atomic",   // 7
+    "declare",   // 8
+    "init",   // 9
+    "shutdown",   // 10
+    "set",   // 11
+    "update",   // 12
+    "routine",   // 13
+    "wait",   // 14
+    "runtime api",   // 15
+    "serial",   // 16
 };
 
+/* We need to validate the construct that PGI sets, because they give us
+ * garbage like:
+ * 269822688 or -887713968 */
+const char* get_acc_construct(const acc_construct_t index) {
+    static const char* unknown = "unknown";
+    if (index < 0 || index > 16) {
+        return unknown;
+    }
+    return acc_constructs[index];
+}
+
 extern "C" static void
 Tau_openacc_launch_callback(acc_prof_info* prof_info, acc_event_info* event_info, acc_api_info* api_info)
 {
@@ -110,7 +122,7 @@ Tau_openacc_launch_callback(acc_prof_info* prof_info, acc_event_info* event_info
     }
     if (launch_event->parent_construct < 9999) {
         ss << " "
-           << acc_constructs[launch_event->parent_construct];
+           << get_acc_construct(launch_event->parent_construct);
     }
 
     ss << " [{"
@@ -184,37 +196,37 @@ Tau_openacc_data_callback( acc_prof_info* prof_info, acc_event_info* event_info,
     switch(prof_info->event_type) {
         case acc_ev_enqueue_upload_start:
             start = 1;
-            sprintf(event_name, "OpenACC enqueue data transfer (HtoD)");
+            snprintf(event_name, sizeof(event_name),  "OpenACC enqueue data transfer (HtoD)");
             break;
         case acc_ev_enqueue_upload_end:
             start = 0;
             break;
         case acc_ev_enqueue_download_start:
             start = 1;
-            sprintf(event_name, "OpenACC enqueue data transfer (DtoH)");
+            snprintf(event_name, sizeof(event_name),  "OpenACC enqueue data transfer (DtoH)");
             break;
         case acc_ev_enqueue_download_end:
             start = 0;
             break;
         case acc_ev_create:
             start = -1;
-            sprintf(event_name, "OpenACC device data create");
+            snprintf(event_name, sizeof(event_name),  "OpenACC device data create");
             break;
         case acc_ev_delete:
             start = -1;
-            sprintf(event_name, "OpenACC device data delete");
+            snprintf(event_name, sizeof(event_name),  "OpenACC device data delete");
             break;
         case acc_ev_alloc:
             start = -1;
-            sprintf(event_name, "OpenACC device alloc");
+            snprintf(event_name, sizeof(event_name),  "OpenACC device alloc");
             break;
         case acc_ev_free:
             start = -1;
-            sprintf(event_name, "OpenACC device free");
+            snprintf(event_name, sizeof(event_name),  "OpenACC device free");
             break;
         default:
             start = -1;
-            sprintf(event_name, "UNKNOWN OPENACC DATA EVENT");
+            snprintf(event_name, sizeof(event_name),  "UNKNOWN OPENACC DATA EVENT");
             fprintf(stderr, "ERROR: Unknown data event passed to OpenACC data event callback.");
     }
 
@@ -232,12 +244,12 @@ Tau_openacc_data_callback( acc_prof_info* prof_info, acc_event_info* event_info,
         return;
     }
 
-    sprintf(file_name, "%s:%s-%s",
+    snprintf(file_name, sizeof(file_name),  "%s:%s-%s",
             prof_info->src_file,
             (prof_info->line_no > 0) ? std::to_string(prof_info->line_no).c_str() : "?",
             (prof_info->end_line_no > 0) ? std::to_string(prof_info->end_line_no).c_str() : "?");
 
-    sprintf(event_data, " ; variable name = %s %s; parent construct = %s (%s)",
+    snprintf(event_data, sizeof(event_data),  " ; variable name = %s %s; parent construct = %s (%s)",
             //                               name ^  ^ (implicit move?)         ^ file and line no.
             (data_event->var_name) ? data_event->var_name : "unknown variable",
             (data_event->implicit) ? "(implicit move)" : "",
@@ -326,6 +338,7 @@ Tau_openacc_other_callback( acc_prof_info* prof_info, acc_event_info* event_info
             start = -1;
             ss << "UNKNOWN OPENACC OTHER EVENT";
             fprintf(stderr, "ERROR: Unknown other event passed to OpenACC other event callback.");
+            return;
     }
 
     // if this is an end event, short circuit by grabbing the FunctionInfo pointer out of tool_info
@@ -347,16 +360,18 @@ Tau_openacc_other_callback( acc_prof_info* prof_info, acc_event_info* event_info
     }
     if (other_event->parent_construct < 9999) {
         ss << " "
-            << acc_constructs[other_event->parent_construct];
+           << get_acc_construct(other_event->parent_construct);
     }
 
-    ss << " [{"
-        << prof_info->src_file
-        << "} {"
-        << prof_info->line_no
-        << ","
-        << prof_info->end_line_no
-        << "}]";
+    if (prof_info->src_file != nullptr) {
+        ss << " [{"
+            << prof_info->src_file
+            << "} {"
+            << prof_info->line_no
+            << ","
+            << prof_info->end_line_no
+            << "}]";
+    }
 
 #ifdef DEBUG_OPENACC
     printf("%s\n", ss.str().c_str());
@@ -416,7 +431,7 @@ const char* openacc_event_names[] = {
 };
 
 static size_t openacc_records = 0;
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     void
 Tau_openacc_process_cupti_activity(CUpti_Activity *record)
 {
@@ -553,18 +568,18 @@ void CUPTIAPI bufferCompleted(CUcontext ctx, uint32_t streamId, uint8_t *buffer,
 
     free(buffer);
 }
-#endif
+#endif //TAU_CUPTI
 void finalize()
 {
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     cuptiActivityFlushAll(0);
-#endif
+#endif //TAU_CUPTI
     printf("Found %llu OpenACC records\n", (long long unsigned) openacc_records);
 }
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
 //void Tau_cupti_onload(void);
-#endif
+#endif //TAU_CUPTI
 
 ////////////////////////////////////////////////////////////////////////////
     extern "C" void
@@ -610,7 +625,7 @@ acc_register_library(acc_prof_reg reg, acc_prof_reg unreg, acc_prof_lookup looku
     reg( acc_ev_wait_end,                  Tau_openacc_other_callback, acc_reg );
 
 
-#ifdef CUPTI
+#ifdef TAU_CUPTI
     if (cuptiOpenACCInitialize((void*)reg, (void*)unreg, (void*)lookup) != CUPTI_SUCCESS) {
         printf("ERROR: failed to initialize CUPTI OpenACC support\n");
     }
@@ -638,7 +653,7 @@ acc_register_library(acc_prof_reg reg, acc_prof_reg unreg, acc_prof_lookup looku
        printf("ERROR: unable to register buffers with CUPTI\n");
        }
        */
-#endif
+#endif //TAU_CUPTI
     atexit(finalize);
 
 } // acc_register_library
