@@ -113,6 +113,8 @@ std::string Tau_roctracer_lookup_activity(uint64_t id) {
     if (i != themap.end()) {
         name = i->second;
         themap.erase(id);
+    } else {
+        TAU_VERBOSE("WARNING! Kernel name not found for correlation %lu\n", id);
     }
     mapLock.unlock();
     return name;
@@ -211,6 +213,12 @@ void Tau_roctx_api_callback(
 
 }
 
+inline void capture_correlation_id(uint32_t correlation_id) {
+    if (TauEnv_get_thread_per_gpu_stream()) {
+      TAU_TRIGGER_EVENT("Correlation ID", (double)(correlation_id));
+    }
+}
+
 // Runtime API callback function
 void Tau_roctracer_api_callback(
     uint32_t domain,
@@ -228,37 +236,76 @@ void Tau_roctracer_api_callback(
   }
   const hip_api_data_t* data = reinterpret_cast<const hip_api_data_t*>(callback_data);
   const char *activity_name = roctracer_op_string(ACTIVITY_DOMAIN_HIP_API, cid, 0);
+  /*
   TAU_VERBOSE("<%s id(%u)\tcorrelation_id(%lu) %s> ",
     activity_name,
     cid,
     data->correlation_id,
     (data->phase == ACTIVITY_API_PHASE_ENTER) ? "on-enter" : "on-exit");
+    */
   if (data->phase == ACTIVITY_API_PHASE_ENTER) {
     TAU_START(activity_name);
-    if (TauEnv_get_thread_per_gpu_stream() &&
-        cid != HIP_API_ID_hipSetDevice) { // no corresponding cid on the GPU side
-      TAU_TRIGGER_EVENT("Correlation ID", (double)(data->correlation_id));
-    }
     switch (cid) {
+      /* If it's a kernel launch, map the correlation ID to the kernel name.
+         also, capture the correlation ID */
       case HIP_API_ID_hipLaunchKernel:
         Tau_roctracer_register_activity(data->correlation_id,
-          hipKernelNameRefByPtr(data->args.hipLaunchKernel.function_address, data->args.hipLaunchKernel.stream));
+          hipKernelNameRefByPtr(data->args.hipLaunchKernel.function_address,
+          data->args.hipLaunchKernel.stream));
+        capture_correlation_id(data->correlation_id);
         break;
       case HIP_API_ID_hipModuleLaunchKernel:
         Tau_roctracer_register_activity(data->correlation_id,
           hipKernelNameRef(data->args.hipModuleLaunchKernel.f));
+        capture_correlation_id(data->correlation_id);
         break;
       case HIP_API_ID_hipHccModuleLaunchKernel:
         Tau_roctracer_register_activity(data->correlation_id,
           hipKernelNameRef(data->args.hipHccModuleLaunchKernel.f));
+        capture_correlation_id(data->correlation_id);
         break;
       case HIP_API_ID_hipExtModuleLaunchKernel:
         Tau_roctracer_register_activity(data->correlation_id,
           hipKernelNameRef(data->args.hipExtModuleLaunchKernel.f));
+        capture_correlation_id(data->correlation_id);
         break;
       case HIP_API_ID_hipExtLaunchKernel:
         Tau_roctracer_register_activity(data->correlation_id,
-          hipKernelNameRefByPtr(data->args.hipExtLaunchKernel.function_address, data->args.hipLaunchKernel.stream));
+          hipKernelNameRefByPtr(data->args.hipExtLaunchKernel.function_address,
+          data->args.hipLaunchKernel.stream));
+        capture_correlation_id(data->correlation_id);
+        break;
+      /* If it's a memcpy, capture the correlation ID */
+      case HIP_API_ID_hipMemcpy:
+      case HIP_API_ID_hipMemcpy2D:
+      case HIP_API_ID_hipMemcpy2DAsync:
+      case HIP_API_ID_hipMemcpy2DFromArray:
+      case HIP_API_ID_hipMemcpy2DFromArrayAsync:
+      case HIP_API_ID_hipMemcpy2DToArray:
+      case HIP_API_ID_hipMemcpy2DToArrayAsync:
+      case HIP_API_ID_hipMemcpy3D:
+      case HIP_API_ID_hipMemcpy3DAsync:
+      case HIP_API_ID_hipMemcpyAsync:
+      case HIP_API_ID_hipMemcpyAtoH:
+      case HIP_API_ID_hipMemcpyDtoD:
+      case HIP_API_ID_hipMemcpyDtoDAsync:
+      case HIP_API_ID_hipMemcpyDtoH:
+      case HIP_API_ID_hipMemcpyDtoHAsync:
+      case HIP_API_ID_hipMemcpyFromArray:
+      case HIP_API_ID_hipMemcpyFromSymbol:
+      case HIP_API_ID_hipMemcpyFromSymbolAsync:
+      case HIP_API_ID_hipMemcpyHtoA:
+      case HIP_API_ID_hipMemcpyHtoD:
+      case HIP_API_ID_hipMemcpyHtoDAsync:
+      case HIP_API_ID_hipMemcpyParam2D:
+      case HIP_API_ID_hipMemcpyParam2DAsync:
+      case HIP_API_ID_hipMemcpyPeer:
+      case HIP_API_ID_hipMemcpyPeerAsync:
+      case HIP_API_ID_hipMemcpyToArray:
+      case HIP_API_ID_hipMemcpyToSymbol:
+      case HIP_API_ID_hipMemcpyToSymbolAsync:
+      case HIP_API_ID_hipMemcpyWithStream:
+        capture_correlation_id(data->correlation_id);
         break;
       default:
         // not necessary.
@@ -267,16 +314,17 @@ void Tau_roctracer_api_callback(
     }
   } else {
     TAU_STOP(activity_name);
+    /*
     switch (cid) {
       case HIP_API_ID_hipMalloc:
-        TAU_VERBOSE("*ptr(0x%p)",
-          *(data->args.hipMalloc.ptr));
+        TAU_VERBOSE("*ptr(0x%p)", *(data->args.hipMalloc.ptr));
         break;
       default:
         break;
     }
+    */
   }
-  TAU_VERBOSE("\n"); fflush(stdout);
+  //TAU_VERBOSE("\n"); fflush(stdout);
 }
 
 // gpu based events
@@ -289,18 +337,26 @@ void Tau_roctracer_hcc_event(const roctracer_record_t *record,
     std::string name = Tau_roctracer_lookup_activity(record->correlation_id);
     char *demangled_name = Tau_demangle_name(name.c_str());
     std::stringstream ss;
-    ss << roctracer_op_string(record->domain, record->op, record->kind)
-       << " " << demangled_name;
+    ss << roctracer_op_string(record->domain, record->op, record->kind);
+    if (strlen(demangled_name) == 0) {
+        ss << " " << name;
+    } else {
+        ss << " " << demangled_name;
+    }
     free(demangled_name);
     joined_name = strdup(ss.str().c_str());
   } else {
     joined_name = strdup(roctracer_op_string(record->domain, record->op, record->kind));
   }
-  TAU_VERBOSE("*** Begin: %lu, End: %lu\n", begin_us, end_us);
+  // ignore marker events, not sure where they come from or what they are for
+  if (strncmp(joined_name, "Marker", 6) == 0) return;
+  // ignore FillBuffer events, not sure where they come from or what they are for
+  if (strncmp(joined_name, "FillBuffer", 6) == 0) return;
+  //TAU_VERBOSE("*** Begin: %lu, End: %lu\n", begin_us, end_us);
   //double ts = Tau_metric_set_synchronized_gpu_timestamp(task_id, ((double)(begin_us))); // convert to microseconds
   metric_set_gpu_timestamp(task_id, ((double)(begin_us)));
   TAU_START_TASK(joined_name, task_id);
-  TAU_VERBOSE("Started event %s on task %d timestamp = %lu \n", joined_name, task_id, begin_us);
+  //TAU_VERBOSE("Started event %s on task %d timestamp = %lu \n", joined_name, task_id, begin_us);
 
   // and the context ID
   if (TauEnv_get_thread_per_gpu_stream()) {
@@ -313,15 +369,31 @@ void Tau_roctracer_hcc_event(const roctracer_record_t *record,
   //Tau_metric_set_synchronized_gpu_timestamp(task_id, ((double)(end_us))); // convert to microseconds
   metric_set_gpu_timestamp(task_id, ((double)(end_us)));
   TAU_STOP_TASK(joined_name, task_id);
-  TAU_VERBOSE("Stopped event %s on task %d timestamp = %lu \n", joined_name, task_id, end_us);
+  //TAU_VERBOSE("Stopped event %s on task %d timestamp = %lu \n", joined_name, task_id, end_us);
   Tau_set_last_timestamp_ns(end_us);
   free (joined_name);
 
   return;
 }
 
+//In some cases, such as srun, the pool is unsorted
+uint64_t get_lowest_timestamp(const char* begin, const char* end)
+{
+  const roctracer_record_t* record = reinterpret_cast<const roctracer_record_t*>(begin);
+  const roctracer_record_t* end_record = reinterpret_cast<const roctracer_record_t*>(end);
+  uint64_t lowest = record->begin_ns;
+  while (record < end_record) {
+    if(record->begin_ns < lowest)
+      lowest = record->begin_ns;
+    ROCTRACER_CALL(roctracer_next_record(record, &record));
+  }
+  return lowest;
+}
+
+
 // Activity tracing callback
 void Tau_roctracer_activity_callback(const char* begin, const char* end, void* arg) {
+  TAU_VERBOSE("Tau_roctracer_activity_callback\n"); fflush(stdout);
   //bool dummy = run_once();  // actually, run it every time we process the buffer
   int dispatch_task_id=-1;
   int copy_task_id = -1;
@@ -329,14 +401,17 @@ void Tau_roctracer_activity_callback(const char* begin, const char* end, void* a
   const roctracer_record_t* record = reinterpret_cast<const roctracer_record_t*>(begin);
   const roctracer_record_t* end_record = reinterpret_cast<const roctracer_record_t*>(end);
   TAU_VERBOSE("\tActivity records :\n"); fflush(stdout);
+  uint64_t lowest_timestamp = (get_lowest_timestamp(begin, end) + deltaTimestamp_ns) * 1.0e-3;
+
+  //TAU_VERBOSE("\tActivity records :\n"); fflush(stdout);
   while (record < end_record) {
     const char * name = roctracer_op_string(record->domain, record->op, record->kind);
     // adjust the timestamp drift between CPU clock and GPU clock, and
     // TAU uses microseconds, so convert ns to us
     uint64_t begin_us = (record->begin_ns + deltaTimestamp_ns) * 1.0e-3;
     uint64_t end_us = (record->end_ns + deltaTimestamp_ns) * 1.0e-3;
-    TAU_VERBOSE("\t%s\tcorrelation_id(%lu) time_ns(%lu:%lu)\n",
-      name, record->correlation_id, begin_us, end_us);
+    /*TAU_VERBOSE("\t%s\tcorrelation_id(%lu) time_ns(%lu:%lu)\n",
+      name, record->correlation_id, begin_us, end_us);*/
     if (record->domain == ACTIVITY_DOMAIN_HIP_OPS) {
       dispatch_task_id = Tau_get_initialized_queues(record->queue_id);
       if (dispatch_task_id == -1) {
@@ -346,8 +421,7 @@ void Tau_roctracer_activity_callback(const char* begin, const char* end, void* a
         // So each one needs a unique virtual thread.
         // We lock the DB, so that we can create three consecutive thread
         // IDs with impunity.
-        RtsLayer::LockDB();
-        TAU_VERBOSE("ACTIVITY_DOMAIN_HIP_API: creating task\n");
+        //TAU_VERBOSE("ACTIVITY_DOMAIN_HIP_API: creating task\n");
         // one for dispatch
         TAU_CREATE_TASK(dispatch_task_id);
         Tau_set_initialized_queues(record->queue_id, dispatch_task_id);
@@ -361,22 +435,22 @@ void Tau_roctracer_activity_callback(const char* begin, const char* end, void* a
             copy_task_id = dispatch_task_id;
             barrier_task_id = dispatch_task_id;
         }
-        RtsLayer::UnLockDB();
         //Tau_metric_set_synchronized_gpu_timestamp(task_id, ((double)begin_us));
-        metric_set_gpu_timestamp(dispatch_task_id, ((double)(begin_us)));
+        metric_set_gpu_timestamp(dispatch_task_id, ((double)(lowest_timestamp)));
         Tau_create_top_level_timer_if_necessary_task(dispatch_task_id);
         Tau_add_metadata_for_task("TAU_TASK_ID", dispatch_task_id, dispatch_task_id);
         Tau_add_metadata_for_task("ROCM_GPU_ID", record->device_id, dispatch_task_id);
         Tau_add_metadata_for_task("ROCM_QUEUE_ID", record->queue_id, dispatch_task_id);
         Tau_metadata_task("ROCM_QUEUE_TYPE", "Kernel Dispatch", dispatch_task_id);
+        //Use the same timestamp to simplify things
         if (TauEnv_get_thread_per_gpu_stream()) {
-            metric_set_gpu_timestamp(copy_task_id, ((double)(begin_us)));
+            metric_set_gpu_timestamp(copy_task_id, ((double)(lowest_timestamp)));
             Tau_create_top_level_timer_if_necessary_task(copy_task_id);
             Tau_add_metadata_for_task("TAU_TASK_ID", copy_task_id, copy_task_id);
             Tau_add_metadata_for_task("ROCM_GPU_ID", record->device_id, copy_task_id);
             Tau_add_metadata_for_task("ROCM_QUEUE_ID", record->queue_id, copy_task_id);
             Tau_metadata_task("ROCM_QUEUE_TYPE", "Memory Transfers", copy_task_id);
-            metric_set_gpu_timestamp(barrier_task_id, ((double)(begin_us)));
+            metric_set_gpu_timestamp(barrier_task_id, ((double)(lowest_timestamp)));
             Tau_create_top_level_timer_if_necessary_task(barrier_task_id);
             Tau_add_metadata_for_task("TAU_TASK_ID", barrier_task_id, barrier_task_id);
             Tau_add_metadata_for_task("ROCM_GPU_ID", record->device_id, barrier_task_id);
@@ -394,10 +468,10 @@ void Tau_roctracer_activity_callback(const char* begin, const char* end, void* a
               barrier_task_id = dispatch_task_id;
           }
       }
-      TAU_VERBOSE(" device_id(%d) queue_id(%lu)\n",
+      /*TAU_VERBOSE(" device_id(%d) queue_id(%lu)\n",
         record->device_id,
         record->queue_id
-      );
+      );*/
       if (record->op == HIP_OP_ID_COPY) {
         Tau_roctracer_hcc_event(record, copy_task_id, begin_us, end_us);  // on the gpu
       } else if (record->op == HIP_OP_ID_BARRIER) {
@@ -405,23 +479,34 @@ void Tau_roctracer_activity_callback(const char* begin, const char* end, void* a
       } else if (record->op == HIP_OP_ID_DISPATCH) {
         Tau_roctracer_hcc_event(record, dispatch_task_id, begin_us, end_us);  // on the gpu
       } else {
-        TAU_VERBOSE("# Unhandled event! \n");
+        TAU_VERBOSE("# roctracer Unhandled event! \n");
       }
     } else {
       TAU_VERBOSE("Bad domain %d\n", record->domain);
       abort();
     }
-    if (record->op == HIP_OP_ID_COPY) TAU_VERBOSE(" bytes(0x%zx)", record->bytes);
-    TAU_VERBOSE("\n");
-    fflush(stdout);
+    //if (record->op == HIP_OP_ID_COPY) TAU_VERBOSE(" bytes(0x%zx)", record->bytes);
+    //TAU_VERBOSE("\n");
+    //fflush(stdout);
     ROCTRACER_CALL(roctracer_next_record(record, &record));
   }
+    // Why is this here?  Well, to make sure that this thread isn't
+    // holding the lock!  FOR SOME REASON, the TauGpu code will finish
+    // without releasing the lock...
+    int tmp = RtsLayer::getNumDBLocks();
+    while (tmp != 0) {
+        //printf ("\n*\n*\n*\nHolding %d locks!\n*\n*\n*\n",tmp);
+        //fflush(stdout);
+        RtsLayer::UnLockDB();
+        tmp = RtsLayer::getNumDBLocks();
+    }
 }
 
 
 // Init tracing routine
 int Tau_roctracer_init_tracing() {
-  TAU_VERBOSE("# START init_tracing: #############################\n");
+  //return 0;
+  TAU_VERBOSE("# START roctracer init_tracing: #############################\n");
 #if (!(defined (TAU_MPI) || (TAU_SHMEM)))
   if (Tau_get_node() == -1) {
       TAU_PROFILE_SET_NODE(0);
@@ -443,8 +528,9 @@ int Tau_roctracer_init_tracing() {
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // Start tracing routine
 extern void Tau_roctracer_start_tracing() {
+  //return;
   static int flag = Tau_roctracer_init_tracing();
-  TAU_VERBOSE("# START #############################\n");
+  TAU_VERBOSE("# START roctracer start_tracing #############################\n");
   // Enable HIP API callbacks
   ROCTRACER_CALL(roctracer_enable_domain_callback(ACTIVITY_DOMAIN_HIP_API, Tau_roctracer_api_callback, NULL));
   // Enable ROCTX Instrumentation API support
@@ -456,17 +542,19 @@ extern void Tau_roctracer_start_tracing() {
 
 // Stop tracing routine
 extern void Tau_roctracer_stop_tracing() {
+  //return;
   if (RtsLayer::myThread() != 0) return;
   ROCTRACER_CALL(roctracer_disable_domain_callback(ACTIVITY_DOMAIN_HIP_API));
   ROCTRACER_CALL(roctracer_disable_domain_activity(ACTIVITY_DOMAIN_HIP_OPS));
   ROCTRACER_CALL(roctracer_flush_activity());
   //Tau_stop_top_level_timer_if_necessary(); // check if this is the call for tasks.
-  TAU_VERBOSE("# STOP  #############################\n");
+  TAU_VERBOSE("# STOP roctracer stop_tracing  #############################\n");
 }
 
 // Flush tracing routine
 extern void Tau_roctracer_flush_tracing() {
-  TAU_VERBOSE("# FLUSHING ASYNC! ###################\n");
+  //return;
+  TAU_VERBOSE("# roctracer FLUSHING ASYNC! ###################\n");
   ROCTRACER_CALL(roctracer_flush_activity());
 }
 
