@@ -127,6 +127,10 @@ extern FunctionInfo * Tau_create_thread_state_if_necessary_string(const string &
 extern "C" void Tau_ompt_resolve_callsite(FunctionInfo &fi, char * resolved_address);
 extern "C" int Tau_get_usesMPI();
 
+#ifdef TAU_MPI
+extern "C" int PMPI_Initialized(int *inited);
+#endif
+
 #if defined(TAU_OPENMP) && !defined (TAU_USE_OMPT_TR6) && !defined (TAU_USE_OMPT_TR7) && !defined (TAU_USE_OMPT_5_0) && (defined(TAU_USE_OMPT) || defined (TAU_IBM_OMPT))
 extern "C" int Tau_get_thread_omp_state(int tid);
 #endif
@@ -368,16 +372,78 @@ struct tau_sampling_flags {
   x_uint64 previousTimestamp[TAU_MAX_COUNTERS];
   /* The trace for this node, mulithreaded execution currently not supported? */
   FILE *ebsTrace;
+
+  tau_sampling_flags() : samplingEnabled(0), suspendSampling(0), 
+    numSamples(0), samplesDroppedTau(0), samplesDroppedSuspended(0), 
+    previousTimestamp(), ebsTrace(NULL) {
+    
+  };
 };
 
 /* depending on the compiler support, use the fastest solution */
 
+
 #ifdef TAU_USE_TLS
-// thread local storage
-struct tau_sampling_flags *tau_sampling_flags(void) {
-    static thread_local struct tau_sampling_flags tau_sampling_tls_flags;
-    return &tau_sampling_tls_flags;
+// It's not possible to access a thread_local variable directly from another thread,
+// so in order to allow one thread to start sampling on another thread (for deferred init
+// with MPI), we need to maintain a map through which one thread can get another thread's flags.
+struct tau_sampling_flagsMap : map<int, tau_sampling_flags*> {
+    tau_sampling_flagsMap() {}
+    
+    virtual ~tau_sampling_flagsMap(){
+        Tau_destructor_trigger();
+    }
+};
+
+// Returns map from thread ID to flags
+static tau_sampling_flagsMap & tau_sampling_tls_flags_map(){
+	static tau_sampling_flagsMap theFlagsMap;
+	return theFlagsMap;
 }
+
+// Mutex to protect flags map
+static std::mutex & TheSamplingFlagsMapMutex() {
+    static std::mutex sampling_flags_map_mutex;
+    return sampling_flags_map_mutex;
+}
+// thread local storage 
+
+
+struct tau_sampling_flags * tau_sampling_flags_by_tid(int tid) {
+    const int myThread = RtsLayer::myThread();
+    static thread_local struct tau_sampling_flags * local_cache = NULL;
+
+    // If asking for my own thread's flags, and we've already cached it locally,
+    // return the cached value.
+    if(tid == myThread && local_cache != NULL) {
+        return local_cache;
+    }
+
+    // Otherwise, check the map
+    std::lock_guard<std::mutex> guard(TheSamplingFlagsMapMutex());
+    tau_sampling_flagsMap & flagsMap = tau_sampling_tls_flags_map();
+    auto it = flagsMap.find(tid);
+    struct tau_sampling_flags * result;
+    if(it == flagsMap.end()) {
+        // Not in map; create and store
+        result = new struct tau_sampling_flags();
+        flagsMap[tid] = result;
+    } else {
+        // In map
+        result = it->second;
+    }
+
+    if(tid == myThread){
+        local_cache = result;
+    }
+
+    return result;
+}
+
+struct tau_sampling_flags *tau_sampling_flags(void) {
+    return tau_sampling_flags_by_tid(RtsLayer::myThread());
+}
+
 #elif defined(TAU_USE_DTLS)
 // thread local storage
 __declspec(thread) struct tau_sampling_flags tau_sampling_tls_flags;
@@ -389,13 +455,68 @@ pthread_key_t tau_sampling_tls_key;
 static inline struct tau_sampling_flags *tau_sampling_flags(void)
 { return (struct tau_sampling_flags*)(pthread_getspecific(tau_sampling_tls_key)); }
 #else
-// worst case - array of flags, one for each thread.
-struct tau_sampling_flags tau_sampling_tls_flags[TAU_MAX_THREADS];
+// worst case - vector of flags, one for each thread.
+struct tau_sampling_flagsList : vector<tau_sampling_flags*> {
+    tau_sampling_flagsList(){
+         //printf("Creating tau_sampling_tls_flags at %p\n", this);
+      }
+     virtual ~tau_sampling_flagsList(){
+         //printf("Destroying tau_sampling_tls_flags at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+   };
+static tau_sampling_flagsList & tau_sampling_tls_flags(){
+	static tau_sampling_flagsList theFlagsList;
+	return theFlagsList;
+}
 static inline struct tau_sampling_flags *tau_sampling_flags(void)
-{ return &tau_sampling_tls_flags[Tau_get_local_tid()]; }
+{ 
+    int tid = Tau_get_local_tid();
+    while(tau_sampling_tls_flags().size()<=tid){
+        RtsLayer::LockDB();
+		tau_sampling_tls_flags().push_back(new struct tau_sampling_flags());
+        RtsLayer::UnLockDB();
+	}
+    return tau_sampling_tls_flags()[tid]; }
 #endif
 
-static bool samplingThrInitialized[TAU_MAX_THREADS] = { false };
+#ifndef TAU_USE_TLS
+#ifdef TAU_MPI
+#warning "Without TLS support, deferred thread sampling initialization will not be supported."
+#endif
+struct tau_sampling_flags * tau_sampling_flags_by_tid(int tid) {
+    return tau_sampling_flags();
+}
+#endif //!TAU_USE_TLS
+
+struct sampThrInit:vector<bool>{
+        sampThrInit(){
+         //printf("Creating tau_sampling_tls_flags at %p\n", this);
+      }
+     virtual ~sampThrInit(){
+         //printf("Destroying tau_sampling_tls_flags at %p, with size %ld\n", this, this->size());
+         Tau_destructor_trigger();
+     }
+};
+static sampThrInit & samplingThrInitialized(){
+    static sampThrInit theSamplingThrInitializedList;
+    return theSamplingThrInitializedList;
+}
+void checkSampThrInitVector(int tid){
+	while(samplingThrInitialized().size()<=tid){
+        RtsLayer::LockDB();
+		samplingThrInitialized().push_back(false);
+        RtsLayer::UnLockDB();
+	}
+}
+static inline bool getSamplingThrInitialized(int tid){
+    checkSampThrInitVector(tid);
+	return samplingThrInitialized()[tid];
+}
+static inline void setSamplingThrInitialized(int tid, bool value){
+    checkSampThrInitVector(tid);
+	samplingThrInitialized()[tid]=value;
+}
 
 /* The trace for this node, mulithreaded execution currently not supported */
 //FILE *ebsTrace[TAU_MAX_THREADS] = { NULL };
@@ -476,10 +597,10 @@ unsigned long get_pc(void *p)
   if (uct != NULL)
     pc = uct->uc_mcontext.jmp_context.iar;
   else
-    pc = 0; 
-  // printf("pc = %p\n", pc); 
-  //pc = (((os_ucontext*)(uct))->uc_mcontext.jmp_context.iar); 
-  return pc; 
+    pc = 0;
+  // printf("pc = %p\n", pc);
+  //pc = (((os_ucontext*)(uct))->uc_mcontext.jmp_context.iar);
+  return pc;
 
 /* EVERYTHING ELSE SUPPORT */
 
@@ -509,6 +630,8 @@ unsigned long get_pc(void *p)
   pc = (unsigned long)sc->arm_pc;
 # elif __aarch64__
   pc = (unsigned long)sc->pc;
+# elif __riscv
+  pc = ((ucontext_t *)p)->uc_mcontext.__gregs[REG_PC];
 # elif __NEC__
   pc = (unsigned long)sc->IC;
 #elif defined(TAU_FUJITSU)
@@ -527,13 +650,13 @@ extern "C" FILE* Tau_sampling_get_ebsTrace()
 }
 
 extern "C" void Tau_sampling_suspend(int tid)
-{
-  tau_sampling_flags()->suspendSampling = 1;
+{  
+    tau_sampling_flags_by_tid(tid)->suspendSampling = 1;
 }
 
 extern "C" void Tau_sampling_resume(int tid)
 {
-  tau_sampling_flags()->suspendSampling = 0;
+    tau_sampling_flags_by_tid(tid)->suspendSampling = 0;
 }
 
 extern "C" void Tau_sampling_timer_pause() {
@@ -583,17 +706,17 @@ extern "C" void Tau_sampling_dlopen()
 
 void Tau_sampling_outputTraceHeader(int tid)
 {
-  fprintf(tau_sampling_flags()->ebsTrace, "# Format version: 0.2\n");
-  fprintf(tau_sampling_flags()->ebsTrace,
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "# Format version: 0.2\n");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace,
       "# $ | <timestamp> | <delta-begin> | <delta-end> | <metric 1> ... <metric N> | <tau callpath> | <location> [ PC callstack ]\n");
-  fprintf(tau_sampling_flags()->ebsTrace,
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace,
       "# %% | <delta-begin metric 1> ... <delta-begin metric N> | <delta-end metric 1> ... <delta-end metric N> | <tau callpath>\n");
-  fprintf(tau_sampling_flags()->ebsTrace, "# Metrics:");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "# Metrics:");
   for (int i = 0; i < Tau_Global_numCounters; i++) {
     const char *name = TauMetrics_getMetricName(i);
-    fprintf(tau_sampling_flags()->ebsTrace, " %s", name);
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, " %s", name);
   }
-  fprintf(tau_sampling_flags()->ebsTrace, "\n");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "\n");
 }
 
 void Tau_sampling_outputTraceCallpath(int tid)
@@ -601,32 +724,32 @@ void Tau_sampling_outputTraceCallpath(int tid)
   Profiler *profiler = TauInternal_CurrentProfiler(tid);
   // *CWL* 2012/3/18 - EBS traces cannot handle callsites for now. Do not track.
   if ((profiler->CallPathFunction != NULL) && (TauEnv_get_callpath())) {
-    fprintf(tau_sampling_flags()->ebsTrace, "%lld", profiler->CallPathFunction->GetFunctionId());
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%lld", profiler->CallPathFunction->GetFunctionId());
   } else if (profiler->ThisFunction != NULL) {
-    fprintf(tau_sampling_flags()->ebsTrace, "%lld", profiler->ThisFunction->GetFunctionId());
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%lld", profiler->ThisFunction->GetFunctionId());
   }
 }
 
 void Tau_sampling_flushTraceRecord(int tid, TauSamplingRecord *record, void *pc, ucontext_t *context)
 {
-  fprintf(tau_sampling_flags()->ebsTrace, "$ | %lld | ", record->timestamp);
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "$ | %lld | ", record->timestamp);
 
 #ifdef TAU_EXP_DISABLE_DELTAS
-  fprintf(tau_sampling_flags()->ebsTrace, "0 | 0 | ");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "0 | 0 | ");
 #else
-  fprintf(tau_sampling_flags()->ebsTrace, "%lu | %lu | ", record->deltaStart, record->deltaStop);
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%lu | %lu | ", record->deltaStart, record->deltaStop);
 #endif
 
   for (int i = 0; i < Tau_Global_numCounters; i++) {
-    fprintf(tau_sampling_flags()->ebsTrace, "%.16G ", record->counters[i]);
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%.16G ", record->counters[i]);
   }
 
-  fprintf(tau_sampling_flags()->ebsTrace, "| ");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "| ");
 
   /* *CWL* - consider a check for TauEnv_get_callpath() here */
   Tau_sampling_outputTraceCallpath(tid);
 
-  fprintf(tau_sampling_flags()->ebsTrace, " | %p", (void*)(record->pc));
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, " | %p", (void*)(record->pc));
 
 #ifdef TAU_UNWIND
   if (TauEnv_get_ebs_unwind() == 1) {
@@ -640,23 +763,23 @@ void Tau_sampling_flushTraceRecord(int tid, TauSamplingRecord *record, void *pc,
 
 void Tau_sampling_outputTraceStop(int tid, Profiler *profiler, double *stopTime)
 {
-  fprintf(tau_sampling_flags()->ebsTrace, "%% | ");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%% | ");
 
   for (int i = 0; i < Tau_Global_numCounters; i++) {
     double startTime = profiler->StartTime[i];    // gtod must be counter 0
     x_uint64 start = (x_uint64)startTime;
-    fprintf(tau_sampling_flags()->ebsTrace, "%lld ", start);
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%lld ", start);
   }
-  fprintf(tau_sampling_flags()->ebsTrace, "| ");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "| ");
 
   for (int i = 0; i < Tau_Global_numCounters; i++) {
     x_uint64 stop = (x_uint64)stopTime[i];
-    fprintf(tau_sampling_flags()->ebsTrace, "%lld ", stop);
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "%lld ", stop);
   }
-  fprintf(tau_sampling_flags()->ebsTrace, "| ");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "| ");
 
   Tau_sampling_outputTraceCallpath(tid);
-  fprintf(tau_sampling_flags()->ebsTrace, "\n");
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "\n");
 }
 
 /*********************************************************************
@@ -669,7 +792,7 @@ int Tau_sampling_write_maps(int tid, int restart)
   int node = RtsLayer::myNode();
   node = 0;
   char filename[4096];
-  sprintf(filename, "%s/ebstrace.map.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
+  snprintf(filename, sizeof(filename),  "%s/ebstrace.map.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
 
   FILE *output = fopen(filename, "a");
 
@@ -706,7 +829,7 @@ void Tau_sampling_outputTraceDefinitions(int tid)
   char filename[4096];
   int node = RtsLayer::myNode();
   node = 0;
-  sprintf(filename, "%s/ebstrace.def.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
+  snprintf(filename, sizeof(filename),  "%s/ebstrace.def.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
 
   FILE *def = fopen(filename, "w");
 
@@ -731,14 +854,14 @@ void Tau_sampling_outputTraceDefinitions(int tid)
     fprintf(stderr, "TAU Sampling: Error, unable to read /proc/self/exe\n");
   } else {
     buffer[rc] = 0;
-    fprintf(tau_sampling_flags()->ebsTrace, "# exe: %s\n", buffer);
+    fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "# exe: %s\n", buffer);
   }
 
   /* write out the node number */
-  fprintf(tau_sampling_flags()->ebsTrace, "# node: %d\n", RtsLayer::myNode());
-  fprintf(tau_sampling_flags()->ebsTrace, "# thread: %d\n", tid);
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "# node: %d\n", RtsLayer::myNode());
+  fprintf(tau_sampling_flags_by_tid(tid)->ebsTrace, "# thread: %d\n", tid);
 
-  fclose(tau_sampling_flags()->ebsTrace);
+  fclose(tau_sampling_flags_by_tid(tid)->ebsTrace);
 
 #if (defined (TAU_BGP) || (TAU_BGQ))
   /* do nothing */
@@ -903,7 +1026,7 @@ CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag
 	    resolvedInfo.filename = "";
     }
     char lineno[32];
-    sprintf(lineno, "%d", resolvedInfo.lineno);
+    snprintf(lineno, sizeof(lineno),  "%d", resolvedInfo.lineno);
     // make sure we allocate enough space for the buffer!!!!
     if (childName) {
         if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FILE) {
@@ -918,6 +1041,13 @@ CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag
             sprintf(buff, "[%s] %s [@] %s [{%s} {0}]",
                 tag, childName, resolvedInfo.funcname,
                 resolvedInfo.filename);
+        } else if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FUNCTION_LINE) {
+            buff = (char*)malloc(strlen(tag) + strlen(childName) +
+                    strlen(resolvedInfo.funcname) + strlen(resolvedInfo.filename) +
+                    strlen(lineno) + 32);
+            sprintf(buff, "[%s] %s [@] %s [{%s} {%d}]",
+                tag, childName, resolvedInfo.funcname,
+                resolvedInfo.filename, Tau_get_lineno_for_function(TheBfdUnitHandle(), resolvedInfo.funcname));
         } else { // Line resolution
             buff = (char*)malloc(strlen(tag) + strlen(childName) +
                     strlen(resolvedInfo.funcname) +
@@ -944,6 +1074,15 @@ CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag
                 resolvedInfo.filename);
             *newShortName = (char*)malloc(strlen(resolvedInfo.funcname) + 2);
             sprintf(*newShortName, "%s", resolvedInfo.funcname);
+        } else if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FUNCTION_LINE) {
+            buff = (char*)malloc(strlen(tag) +
+                    strlen(resolvedInfo.funcname) +
+                    strlen(resolvedInfo.filename) + 32);
+            sprintf(buff, "[%s] %s [{%s} {%d}]",
+                tag, resolvedInfo.funcname,
+                resolvedInfo.filename, Tau_get_lineno_for_function(TheBfdUnitHandle(), resolvedInfo.funcname));
+            *newShortName = (char*)malloc(strlen(resolvedInfo.funcname) + 2);
+            sprintf(*newShortName, "%s", resolvedInfo.funcname);
         } else { // Line resolution
             buff = (char*)malloc(strlen(tag) +
                     strlen(resolvedInfo.funcname) +
@@ -1141,14 +1280,14 @@ void Tau_sampling_finalizeProfile(int tid)
           string temp_ss(resolved_address);
           parentTauContext->SetName(temp_ss);
     }
-
-    if ((parentTauContext->pathHistogram[tid] == NULL) || (parentTauContext->pathHistogram[tid]->size() == 0)) {
+    TauPathHashTable<TauPathAccumulator>* pathHistogram=parentTauContext->GetPathHistogram(tid);//pathHistogram[tid]; //TODO:DYNAPROF
+    if ((pathHistogram == NULL) || (pathHistogram->size() == 0)) {
       // No samples encountered in this TAU context. Continue to next TAU context.
 //      DEBUGMSG("Tau Context %s has no samples", parentTauContext->GetName());
       continue;
     }
-    parentTauContext->pathHistogram[tid]->resetIter();
-    pair<unsigned long *, TauPathAccumulator> * item = parentTauContext->pathHistogram[tid]->nextIter();
+    pathHistogram->resetIter();
+    pair<unsigned long *, TauPathAccumulator> * item = pathHistogram->nextIter();
     while (item) {
       // This is a placeholder for more generic pcStack extraction routines.
       CallSiteCandidate * candidate = new CallSiteCandidate(item->first, item->second.count, parentTauContext);
@@ -1158,7 +1297,7 @@ void Tau_sampling_finalizeProfile(int tid)
       }
       candidates.push_back(candidate);
       delete item;
-      item = parentTauContext->pathHistogram[tid]->nextIter();
+      item = pathHistogram->nextIter();
     }
   }
   RtsLayer::UnLockDB();
@@ -1364,16 +1503,16 @@ void Tau_sampling_finalizeProfile(int tid)
   //
   char tmpstr[512];
   char tmpname[512];
-  sprintf(tmpname, "TAU_EBS_SAMPLES_TAKEN_%d", tid);
-  sprintf(tmpstr, "%lld", tau_sampling_flags()->numSamples);
+  snprintf(tmpname, sizeof(tmpname),  "TAU_EBS_SAMPLES_TAKEN_%d", tid);
+  snprintf(tmpstr, sizeof(tmpstr),  "%lld", tau_sampling_flags_by_tid(tid)->numSamples);
   TAU_METADATA(tmpname, tmpstr);
 
-  sprintf(tmpname, "TAU_EBS_SAMPLES_DROPPED_TAU_%d", tid);
-  sprintf(tmpstr, "%lld", tau_sampling_flags()->samplesDroppedTau);
+  snprintf(tmpname, sizeof(tmpname),  "TAU_EBS_SAMPLES_DROPPED_TAU_%d", tid);
+  snprintf(tmpstr, sizeof(tmpstr),  "%lld", tau_sampling_flags_by_tid(tid)->samplesDroppedTau);
   TAU_METADATA(tmpname, tmpstr);
 
-  sprintf(tmpname, "TAU_EBS_SAMPLES_DROPPED_SUSPENDED_%d", tid);
-  sprintf(tmpstr, "%lld", tau_sampling_flags()->samplesDroppedSuspended);
+  snprintf(tmpname, sizeof(tmpname),  "TAU_EBS_SAMPLES_DROPPED_SUSPENDED_%d", tid);
+  snprintf(tmpstr, sizeof(tmpstr),  "%lld", tau_sampling_flags_by_tid(tid)->samplesDroppedSuspended);
   TAU_METADATA(tmpname, tmpstr);
 
   while (!candidates.empty()) {
@@ -1393,9 +1532,13 @@ void Tau_sampling_handle_sampleProfile(void *pc, ucontext_t *context, int tid) {
       if (TauEnv_get_ebs_enabled_tau()) {
 	    // if we are sampling to measure TAU, the profile might not be done yet
 	    return;
+      } else if (collectingSamples == 0) {
+	    // Are we wrapping up?
+	    return;
 	  } else {
 	    printf("STILL no top level timer on thread %d!\n", tid);
 	    fflush(stdout);
+        abort();
 	    exit(999);
 	  }
 	}
@@ -1453,16 +1596,16 @@ void Tau_sampling_handle_sampleProfile(void *pc, ucontext_t *context, int tid) {
       // Hypothesis: Triggering PAPI overflows resets the values to 0.
       //             (or close to 0).
       deltaValues[i] = ebsPeriod;
-      tau_sampling_flags()->previousTimestamp[i] += ebsPeriod;
+      tau_sampling_flags_by_tid(tid)->previousTimestamp[i] += ebsPeriod;
     } else {
-      deltaValues[i] = values[i] - tau_sampling_flags()->previousTimestamp[i];
-      /*
-       printf("[%s] tid=%d ctr=%d, Delta computed as %f minus %lld = %f\n",
+      deltaValues[i] = values[i] - tau_sampling_flags_by_tid(tid)->previousTimestamp[i];
+       /*
+       fprintf(stderr, "[%s] tid=%d ctr=%d, Delta computed as %f minus %lld = %f\n",
        samplingContext->GetName(),
        tid, i,
        values[i], tau_sampling_flags()->previousTimestamp[i], deltaValues[i]);
        */
-      tau_sampling_flags()->previousTimestamp[i] = values[i];
+      tau_sampling_flags_by_tid(tid)->previousTimestamp[i] = values[i];
     }
     //printf("tid = %d, sampling previousTimestamp = %llu, period = %d\n", tid, tau_sampling_flags()->previousTimestamp[i], ebsPeriod); fflush(stdout);
   }
@@ -1510,12 +1653,14 @@ void Tau_sampling_event_start(int tid, void **addresses)
   // Protect TAU from itself
   TauInternalFunctionGuard protects_this_function;
 
-  // This is undefined when no unwind capability has been linked into TAU
+  if(getSamplingThrInitialized(tid)) {
+    // This is undefined when no unwind capability has been linked into TAU
 #ifdef TAU_UNWIND
-  if (TauEnv_get_ebs_unwind() == 1) {
-    Tau_sampling_unwindTauContext(tid, addresses);
-  }
+    if (TauEnv_get_ebs_unwind() == 1) {
+        Tau_sampling_unwindTauContext(tid, addresses);
+    }
 #endif /* TAU_UNWIND */
+  }
 
 /* Kevin here. This code is a bad idea. I have disabled it for now.
  * Sampling and instrumentation can play together just fine, if a
@@ -1568,23 +1713,25 @@ int Tau_sampling_event_stop(int tid, double *stopTime)
   // Protect TAU from itself
   TauInternalFunctionGuard protects_this_function;
 
-  tau_sampling_flags()->samplingEnabled = 0;
+  if(getSamplingThrInitialized(tid)) {
+    tau_sampling_flags_by_tid(tid)->samplingEnabled = 0;
 
-  Profiler *profiler = TauInternal_CurrentProfiler(tid);
+    Profiler *profiler = TauInternal_CurrentProfiler(tid);
 
-  if (TauEnv_get_tracing()) {
-    if (!profiler->needToRecordStop) {
-      tau_sampling_flags()->samplingEnabled = 1;
-      return 0;
+    if (TauEnv_get_tracing()) {
+        if (!profiler->needToRecordStop) {
+        tau_sampling_flags_by_tid(tid)->samplingEnabled = 1;
+        return 0;
+        }
+        Tau_sampling_outputTraceStop(tid, profiler, stopTime);
     }
-    Tau_sampling_outputTraceStop(tid, profiler, stopTime);
-  }
 
-  if (TauEnv_get_profiling()) {
-    Tau_sampling_eventStopProfile(tid, profiler, stopTime);
-  }
+    if (TauEnv_get_profiling()) {
+        Tau_sampling_eventStopProfile(tid, profiler, stopTime);
+    }
 
-  tau_sampling_flags()->samplingEnabled = 1;
+    tau_sampling_flags_by_tid(tid)->samplingEnabled = 1;
+  }
 #endif
   return 0;
 }
@@ -1714,6 +1861,13 @@ int Tau_sampling_init(int tid, pid_t pid)
   tau_sampling_tls_flags->samplesDroppedSuspended = 0;
   tau_sampling_tls_flags->ebsTrace = NULL;
   pthread_setspecific(tau_sampling_tls_key, tau_sampling_tls_flags);
+#elif defined(TAU_USE_TLS)
+  tau_sampling_flags_by_tid(tid)->samplingEnabled = 0;
+  tau_sampling_flags_by_tid(tid)->suspendSampling = 0;
+  tau_sampling_flags_by_tid(tid)->numSamples = 0;
+  tau_sampling_flags_by_tid(tid)->samplesDroppedTau = 0;
+  tau_sampling_flags_by_tid(tid)->samplesDroppedSuspended = 0;
+  tau_sampling_flags_by_tid(tid)->ebsTrace = NULL;
 #else
   tau_sampling_flags()->samplingEnabled = 0;
   tau_sampling_flags()->suspendSampling = 0;
@@ -1729,10 +1883,10 @@ int Tau_sampling_init(int tid, pid_t pid)
   char filename[4096];
 
   if (TauEnv_get_tracing()) {
-    sprintf(filename, "%s/ebstrace.raw.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
+    snprintf(filename, sizeof(filename),  "%s/ebstrace.raw.%d.%d.%d.%d", profiledir, RtsLayer::getPid(), node, RtsLayer::myContext(), tid);
 
-    tau_sampling_flags()->ebsTrace = fopen(filename, "w");
-    if (tau_sampling_flags()->ebsTrace == NULL) {
+    tau_sampling_flags_by_tid(tid)->ebsTrace = fopen(filename, "w");
+    if (tau_sampling_flags_by_tid(tid)->ebsTrace == NULL) {
       fprintf(stderr, "Tau Sampling Error: Unable to open %s for writing\n", filename);
       exit(-1);
     }
@@ -1753,8 +1907,13 @@ int Tau_sampling_init(int tid, pid_t pid)
    See Tau_sampling_handle_sample().
    */
 #ifndef TAU_BGQ
-  if (strcmp(TauEnv_get_ebs_source(), "itimer") == 0 ||
-      strcmp(TauEnv_get_ebs_source(), "TIME") == 0)
+  // TauEnv_get_ebs_source_orig() still returns the original value even
+  // after it's overridden later with TauEnv_override_ebs_source.
+  // This avoids a race condition where one thread changes
+  // TAU_EBS_SOURCE, causing this if to evaluate false on
+  // later-executing threads.
+  if (strcmp(TauEnv_get_ebs_source_orig(), "itimer") == 0 ||
+      strcmp(TauEnv_get_ebs_source_orig(), "TIME") == 0)
   {
 #endif // TAU_BGQ
 
@@ -1887,10 +2046,12 @@ int Tau_sampling_init(int tid, pid_t pid)
     }
     // Since we've now initialized sigaction, we can start the timer on any deferred threads
     for(DeferredInitVector::iterator it = TheDeferredInitVector().begin(); it != TheDeferredInitVector().end(); ++it) {
-        if(!samplingThrInitialized[it->tid]) {
+        if(!getSamplingThrInitialized(it->tid)) {
             TAU_VERBOSE("Will create sampling timer for deferred thread %d\n", it->tid);
-            samplingThrInitialized[it->tid] = true;
+            setSamplingThrInitialized(it->tid, true);
             Tau_sampling_init(it->tid, it->pid);
+        } else {
+            TAU_VERBOSE("Skipping deferred thread %d because sampling has already been initialized.\n", it->tid);
         }
     }
   } else { //!sigaction_initialized
@@ -1922,15 +2083,24 @@ int Tau_sampling_init(int tid, pid_t pid)
    TAU_VERBOSE(" *** (S%d) send alarm to %d\n", gettid(), sev.sigev_notify_thread_id);
 #endif
    ret = timer_create(CLOCK_REALTIME, &sev, &timerid);
+
    {
      std::lock_guard<std::mutex> guard(TheThreadTimerMapMutex());
      TheThreadTimerMap()[pid == 0 ? RtsLayer::getTid() : pid] = timerid;
    }
-   TAU_VERBOSE("Created sampling timer for TAU tid = %d, kernel TID = %jd\n", tid, (intmax_t)sev.sigev_notify_thread_id);
-  if (ret != 0) {
-    fprintf(stderr, "TAU: (%d, %d) Sampling error 6: %s\n", RtsLayer::myNode(), RtsLayer::myThread(), strerror(ret));
-    return -1;
-  }
+
+   // If the thread no longer exists, we get EINVAL back from timer_create
+   if(ret == EINVAL && pid != 0) {
+     TAU_VERBOSE("Invalid argument error while initializing sampling on deferred thread %d (pid=%jd). The thread may have exited already.\n", tid, (intmax_t)pid);
+     return -1;
+   } 
+   
+   if (ret != 0) {
+     fprintf(stderr, "TAU: (node=%d, myThread=%d, tid=%d) Sampling error 6: %d: %s\n", RtsLayer::myNode(), RtsLayer::myThread(), tid, ret, strerror(errno));
+     return -1;
+   }
+
+   TAU_VERBOSE("Created sampling timer for TAU tid = %d, kernel TID = %jd, timer id = %jd\n", tid, (intmax_t)sev.sigev_notify_thread_id, (intmax_t)timerid);
    struct itimerspec it;
 
   /* this timer is in nanoseconds, but our parameters are in microseconds. */
@@ -1983,11 +2153,11 @@ int Tau_sampling_init(int tid, pid_t pid)
   TauMetrics_internal_alwaysSafeToGetMetrics(tid, values);
   //for (int x = 0; x < TAU_MAX_THREADS; x++) {
     for (int y = 0; y < Tau_Global_numCounters; y++) {
-      tau_sampling_flags()->previousTimestamp[y] = values[y];
+      tau_sampling_flags_by_tid(tid)->previousTimestamp[y] = values[y];
     }
     //printf("tid = %d, init previousTimestamp = %llu\n", tid, tau_sampling_flags()->previousTimestamp[y]); fflush(stdout);
   //}
-  tau_sampling_flags()->samplingEnabled = 1;
+  tau_sampling_flags_by_tid(tid)->samplingEnabled = 1;
   collectingSamples = 1;
   return 0;
 }
@@ -1997,14 +2167,14 @@ int Tau_sampling_init(int tid, pid_t pid)
  ********************************************************************/
 int Tau_sampling_finalize(int tid)
 {
-  if (TauEnv_get_tracing() && !tau_sampling_flags()->ebsTrace) return 0;
+  if (TauEnv_get_tracing() && !tau_sampling_flags_by_tid(tid)->ebsTrace) return 0;
   TAU_VERBOSE("TAU: <Node=%d.Thread=%d> finalizing sampling for %d...\n", RtsLayer::myNode(), Tau_get_local_tid(), tid); fflush(stdout);
 
   // Protect TAU from itself
   TauInternalFunctionGuard protects_this_function;
 
   /* Disable sampling first */
-  tau_sampling_flags()->samplingEnabled = 0;
+  tau_sampling_flags_by_tid(tid)->samplingEnabled = 0;
   if(tid == 0) {
     collectingSamples = 0;
   }
@@ -2104,11 +2274,17 @@ extern "C" void Tau_sampling_init_if_necessary(void)
       Tau_sampling_defer_init();
       return;
   }
+  int mpi_initialized = 0;
+  PMPI_Initialized(&mpi_initialized);
+  if(!mpi_initialized) {
+    Tau_sampling_defer_init();
+    return;
+  }
 #endif
 
   int tid = RtsLayer::localThreadId();
   // have we initialized already?
-  if (samplingThrInitialized[tid]) return;
+  if (getSamplingThrInitialized(tid)) return;
 
   /* Greetings, intrepid thread developer. We had a problem with OpenMP applications
    * which did not call instrumented functions or regions from an OpenMP region. In
@@ -2155,9 +2331,9 @@ extern "C" void Tau_sampling_init_if_necessary(void)
         {
           // Getting the thread ID registers the OpenMP thread.
           int myTid = Tau_get_thread ();
-          if (!samplingThrInitialized[myTid]) {
+          if (!getSamplingThrInitialized(myTid)) {
             Tau_sampling_init(myTid);
-            samplingThrInitialized[myTid] = true;
+            setSamplingThrInitialized(myTid,true);
             TAU_VERBOSE("Thread %d, %d initialized sampling\n", tid, myTid);
           }
         }    // critical
@@ -2175,26 +2351,44 @@ extern "C" void Tau_sampling_init_if_necessary(void)
 
 #else
 // handle all other cases!
-  if (!samplingThrInitialized[tid]) {
-    samplingThrInitialized[tid] = true;
+  if (!getSamplingThrInitialized(tid)) {
+    setSamplingThrInitialized(tid, true);
     Tau_sampling_init(tid, 0);
     TAU_VERBOSE("Thread %d initialized sampling\n", tid);
   }
 #endif
 }
 
+struct thrFinalizedVector:vector<bool>{
+    thrFinalizedVector() {
+        // nothing
+    }
+
+    virtual ~thrFinalizedVector(){
+        Tau_destructor_trigger();
+    }
+};
+
+inline void checkBVector (thrFinalizedVector * v, int tid){
+    RtsLayer::LockDB();
+    while(v->size()<=tid){
+		v->push_back(false);
+	}
+    RtsLayer::UnLockDB();
+}
+
 extern "C"
 void Tau_sampling_finalize_if_necessary(int tid)
 {
-  static bool finalized = false;
-  static bool thrFinalized[TAU_MAX_THREADS] = {false};
-  //int tid = Tau_get_local_tid();
+    static bool finalized = false;
 
-  TAU_VERBOSE("TAU: Finalize(if necessary) <Node=%d.Thread=%d> finalizing sampling...\n", RtsLayer::myNode(), tid); fflush(stderr);
+    TAU_VERBOSE("TAU: Finalize(if necessary) <Node=%d.Thread=%d> finalizing sampling...\n", RtsLayer::myNode(), tid); fflush(stderr);
 
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
 
+    static thrFinalizedVector thrFinalized;
+
     // before wrapping things up, stop listening to signals.
     sigset_t x;
     sigemptyset(&x);
@@ -2205,6 +2399,8 @@ void Tau_sampling_finalize_if_necessary(int tid)
     sigprocmask(SIG_BLOCK, &x, NULL);
 #endif
 
+    checkBVector(&thrFinalized,tid);
+
     if (!finalized) {
       TAU_VERBOSE("TAU: <Node=%d.Thread=%d> finalizing sampling...\n", RtsLayer::myNode(), tid); fflush(stdout);
       RtsLayer::LockEnv();
@@ -2218,29 +2414,30 @@ void Tau_sampling_finalize_if_necessary(int tid)
       RtsLayer::UnLockEnv();
     }
 
+    RtsLayer::LockEnv();
     if (!thrFinalized[tid]) {
-      RtsLayer::LockEnv();
-      if (!thrFinalized[tid]) {
-        tau_sampling_flags()->samplingEnabled = 0;
+        tau_sampling_flags_by_tid(tid)->samplingEnabled = 0;
         thrFinalized[tid] = true;
         Tau_sampling_finalize(tid);
-      }
-      RtsLayer::UnLockEnv();
     }
+    RtsLayer::UnLockEnv();
 
     // Kevin: should we finalize all threads on this process? I think so.
-  if (tid == 0) {
-    for (int i = 0; i < RtsLayer::getTotalThreads(); i++) {
-      if (!thrFinalized[i]) {
-        RtsLayer::LockEnv();
-        if (!thrFinalized[i]) {
-          thrFinalized[i] = true;
-          Tau_sampling_finalize(i);
+    if (tid == 0) {
+        checkBVector(&thrFinalized, RtsLayer::getTotalThreads());
+        for (int i = 0; i < RtsLayer::getTotalThreads(); i++) {
+            RtsLayer::LockEnv();
+            if (!thrFinalized[i]) {
+                thrFinalized[i] = true;
+                Tau_sampling_finalize(i);
+            }
+            RtsLayer::UnLockEnv();
         }
-        RtsLayer::UnLockEnv();
-      }
     }
-  }
+}
+
+void Tau_sampling_stop_sampling() {
+    collectingSamples = 0;
 }
 
 #endif //TAU_WINDOWS && TAU_ANDROID
