@@ -1,12 +1,19 @@
 package edu.uoregon.tau.common;
 
-import java.awt.*;
+import java.awt.Color;
+import java.awt.Component;
+import java.awt.Container;
+import java.awt.Font;
+import java.awt.FontMetrics;
+import java.awt.GridBagConstraints;
 import java.net.URL;
-import java.util.StringTokenizer;
 import java.util.NoSuchElementException;
+import java.util.StringTokenizer;
 
 import javax.swing.ImageIcon;
 import javax.swing.JFrame;
+import javax.swing.JTable;
+import javax.swing.table.DefaultTableCellRenderer;
 
 import org.jfree.chart.JFreeChart;
 import org.jfree.chart.axis.Axis;
@@ -17,13 +24,19 @@ import org.jfree.chart.title.LegendTitle;
 
 public class Utility {
 
+	private static final String SPACE=" ";
+	private static final String MSPACE_REGX=" {2,}";
     public static String removeRuns(String str) {
-        int loc = str.indexOf("  ");
-        while (loc > 0) {
-            str = str.substring(0, loc) + str.substring(loc + 1);
-            loc = str.indexOf("  ");
-        }
-        return str;
+    	
+    	String[] cut = str.split(MSPACE_REGX);
+    	StringBuilder strb = new StringBuilder(str.length());
+    	strb.append(cut[0]);
+    	for(int i =1;i<cut.length;i++){
+    		strb.append(SPACE);
+    		strb.append(cut[i]);
+    	}
+
+        return strb.toString();
     }
 
     public static ImageIcon getImageIconResource(String name) {
@@ -127,6 +140,9 @@ public class Utility {
      * @return the shortened function name
      */
     public static String shortenFunctionName(String longName) {
+		if (longName == null) {
+			return "";
+		}
         StringTokenizer st = new StringTokenizer(longName, "(");
         String shorter = null;
         try {
@@ -163,5 +179,16 @@ public class Utility {
         gbc.gridheight = h;
         frame.getContentPane().add(c, gbc);
     }
+    
+    
+    /**
+     *  Sets the given table's row height based on the size of the selected font.
+     */
+    public static void setTableFontHeight(JTable table, Font f){
+    	DefaultTableCellRenderer tcr = new DefaultTableCellRenderer();
+    	Component c = tcr.getTableCellRendererComponent(table, "null", true, true, 0, 0);
+    	FontMetrics fm = c.getFontMetrics(f);
+    	table.setRowHeight(fm.getHeight());
+    }
 
 }
