@@ -264,12 +264,12 @@ private ParaProfManagerWindow paraProfManager = null;
          return;
       }
 
-      ArrayList<Object> collectTrials = collectTrials(sel); 
+      ArrayList<ParaProfTrial> collectTrials = collectTrials(sel); 
 
       ArrayList<Object> errors = new ArrayList<Object>();
       for (int i=0;i<collectTrials.size();i++){
          try {
-            applyToTrial((ParaProfTrial) collectTrials.get(i), arg1Field.getText());
+            applyToTrial(collectTrials.get(i), arg1Field.getText());
          } catch (MetricNotFoundException e) {
             errors.add(collectTrials.get(i));
          }
@@ -282,44 +282,36 @@ private ParaProfManagerWindow paraProfManager = null;
       }
 
    }
-   @SuppressWarnings("unchecked")
-private ArrayList<Object> collectTrials(DefaultMutableTreeNode sel) {
+   
+   private ArrayList<ParaProfTrial> collectTrials(DefaultMutableTreeNode sel) {
+	   ArrayList<ParaProfTrial> collectTrials = new ArrayList<ParaProfTrial>();
+	   collectTrials(sel, collectTrials);
+	   return collectTrials;
+   }
 
-      Object selected = paraProfManager.getSelectedObject().getUserObject();
-      ArrayList<Object> collectTrials = new ArrayList<Object>();
+private void collectTrials(DefaultMutableTreeNode sel, ArrayList<ParaProfTrial> collectTrials ) {
+
+      Object selected = sel.getUserObject();
+      
       if(selected instanceof ParaProfMetric){
          ParaProfMetric met = (ParaProfMetric) selected;
          collectTrials .add(met.getParaProfTrial());
       }else if(selected instanceof ParaProfTrial){
-         collectTrials.add(sel.getUserObject());
-      }else if(selected instanceof ParaProfExperiment){
-         paraProfManager.expand(sel);
-         Enumeration<DefaultMutableTreeNode> trials = sel.children();
-         while(trials.hasMoreElements()){
-            DefaultMutableTreeNode next = (DefaultMutableTreeNode)trials.nextElement();
-            paraProfManager.expand(next);
-            collectTrials.add(next.getUserObject());
-         }
-      }else if(selected instanceof ParaProfApplication){
-         paraProfManager.expand( sel);
-         Enumeration<DefaultMutableTreeNode> exps = sel.children();
-         while(exps.hasMoreElements()){
-            DefaultMutableTreeNode next = (DefaultMutableTreeNode)exps.nextElement();
-            paraProfManager.expand(next);
-            Enumeration<DefaultMutableTreeNode> trls = next.children();
-            while(trls.hasMoreElements()){
-               DefaultMutableTreeNode node = (DefaultMutableTreeNode) trls.nextElement();
-               paraProfManager.expand(node);
-               collectTrials.add(node.getUserObject());
-            }
-         }
-
+         collectTrials.add((ParaProfTrial)(selected));
+      }else if(selected instanceof ParaProfExperiment|| selected instanceof ParaProfApplication|| selected instanceof ParaProfView){
+    	  paraProfManager.expand( sel);
+          Enumeration<DefaultMutableTreeNode> exps = sel.children();
+          while(exps.hasMoreElements()){
+        	  DefaultMutableTreeNode node = (DefaultMutableTreeNode) exps.nextElement();
+              paraProfManager.expand(node);
+              collectTrials(node, collectTrials);
+          }
       }else{
          JOptionPane.showMessageDialog(paraProfManager, 
                "Please select a trial, experiment or application.",
                "Warning", JOptionPane.WARNING_MESSAGE);
          //Please select a trial, experiment or application
-      }      return collectTrials;
+      }      
    }
 
    private static void sleep(int msec) {
@@ -347,7 +339,7 @@ private ArrayList<Object> collectTrials(DefaultMutableTreeNode sel) {
          }else if(arg.equals("/")){
             insertString("/");
          }else if(arg.equals("(")){
-            insertString(")");
+            insertString("(");
          }else if(arg.equals(")")){
             insertString(")");
          }else if(arg.equals("=")){
@@ -367,7 +359,14 @@ private ArrayList<Object> collectTrials(DefaultMutableTreeNode sel) {
    }
 
    public void insertMetric(ParaProfMetric metric) {
-      arg1Field.replaceSelection("\""+metric.getName()+"\"");//puts string at point of cursor
+	   String quoteMetric=metric.getName();
+	   quoteMetric=quoteMetric.replace(' ', '\"');
+	   if(!quoteMetric.startsWith("(")||!quoteMetric.endsWith(")")){
+		   arg1Field.replaceSelection("\""+quoteMetric+"\"");//puts string at point of cursor
+	   }
+	   else{
+		   arg1Field.replaceSelection(quoteMetric);//puts string at point of cursor
+	   }
       arg1Field.requestFocusInWindow();
       //de-select the text
       int pos=arg1Field.getSelectionStart();
@@ -382,11 +381,25 @@ private ArrayList<Object> collectTrials(DefaultMutableTreeNode sel) {
       //de-select the text
       int pos=arg1Field.getSelectionStart();
       String text  =  arg1Field.getText();
-      String first = text.substring(0,pos-(metric.getName().length() +2));
-      String second = text.substring(pos+1);
+      String name = metric.getName();
+      int len = name.length();
+      int firstDex=pos-(len +2);
+      String first = "";
+      if(firstDex>0){
+    	  text.substring(0,firstDex);
+      }
+      int secondDex=pos+1;
+      String second = "";
+      if(secondDex>0)
+      {
+    	  second = text.substring(secondDex);
+      }
       arg1Field.setText(first+second);
       pos = pos-(metric.getName().length() +2);
-      arg1Field.setCaretPosition(pos);
+      if(pos>=0&&pos<arg1Field.getText().length())
+      {
+    	  arg1Field.setCaretPosition(pos);
+      }
       if(arg1Field.getText().length()==pos){
          arg1Field.setText(arg1Field.getText()+" ");
          arg1Field.setCaretPosition(pos);
@@ -432,11 +445,11 @@ private ArrayList<Object> collectTrials(DefaultMutableTreeNode sel) {
                      "Expression Error", JOptionPane.ERROR_MESSAGE);
             }else{
 
-               ArrayList<Object> collectTrials = collectTrials(sel); 
+               ArrayList<ParaProfTrial> collectTrials = collectTrials(sel); 
 
                for (int i=0;i<collectTrials.size();i++){
                   try {
-                     applyToTrial((ParaProfTrial) collectTrials.get(i), expression);
+                     applyToTrial( collectTrials.get(i), expression);
                   } catch (MetricNotFoundException e) {
                      errors.add(collectTrials.get(i));
                   }
