@@ -8,17 +8,24 @@
 
 package edu.uoregon.tau.paraprof;
 
-import java.util.*;
-import javax.swing.tree.*;
-import edu.uoregon.tau.perfdmf.*;
-import edu.uoregon.tau.perfdmf.database.*;
+import java.util.ListIterator;
+import java.util.Vector;
+
+import javax.swing.tree.DefaultMutableTreeNode;
+import javax.swing.tree.TreePath;
+
+import edu.uoregon.tau.perfdmf.Application;
 
 public class ParaProfApplication extends Application implements ParaProfTreeNodeUserObject {
 
-    private DefaultMutableTreeNode defaultMutableTreeNode = null;
+    /**
+	 * 
+	 */
+	private static final long serialVersionUID = 1662692425763662408L;
+	private DefaultMutableTreeNode defaultMutableTreeNode = null;
     private TreePath treePath = null;
     private boolean dBApplication = false;
-    private Vector experiments = new Vector();
+    private Vector<ParaProfExperiment> experiments = new Vector<ParaProfExperiment>();
 
     public ParaProfApplication() {
         super();
@@ -55,14 +62,14 @@ public class ParaProfApplication extends Application implements ParaProfTreeNode
     }
 
     public ParaProfExperiment getExperiment(int experimentID) {
-        return (ParaProfExperiment) experiments.elementAt(experimentID);
+        return experiments.elementAt(experimentID);
     }
 
-    public Vector getExperiments() {
+    public Vector<ParaProfExperiment> getExperiments() {
         return experiments;
     }
 
-    public ListIterator getExperimentList() {
+    public ListIterator<ParaProfExperiment> getExperimentList() {
         return experiments.listIterator();
     }
 
@@ -79,16 +86,6 @@ public class ParaProfApplication extends Application implements ParaProfTreeNode
         experiments.remove(experiment);
     }
 
-    public boolean isExperimentPresent(String name) {
-        for (Enumeration e = experiments.elements(); e.hasMoreElements();) {
-            ParaProfExperiment exp = (ParaProfExperiment) e.nextElement();
-            if (name.equals(exp.getName()))
-                return true;
-        }
-        //If we make it here, the experiment run name is not present.  Return false.
-        return false;
-    }
-
     public String getIDString() {
         return Integer.toString(this.getID());
     }
