@@ -1,9 +1,10 @@
 package edu.uoregon.tau.paraprof.interfaces;
 
-import java.awt.Rectangle;
+import javax.swing.JFrame;
+
 
 public interface ParaProfWindow {
+    public JFrame getFrame();
     public void help(boolean display);
     public void closeThisWindow();
-
 }
