@@ -1,17 +1,22 @@
 package edu.uoregon.tau.perfdmf;
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStreamReader;
 import java.util.StringTokenizer;
 import java.util.Vector;
 
 public class GprofDataSource extends DataSource {
 
-//    private int indexStart = 0;
-//    private int percentStart = 0;
-//    private int selfStart = 0;
-//    private int descendantsStart = 0;
-//    private int calledStart = 0;
-//    private int nameStart = 0;
+    //    private int indexStart = 0;
+    //    private int percentStart = 0;
+    //    private int selfStart = 0;
+    //    private int descendantsStart = 0;
+    //    private int calledStart = 0;
+    //    private int nameStart = 0;
     private boolean fixNames = false;
     private int linenumber = 0;
     private boolean fixLengths = true;
@@ -76,9 +81,9 @@ public class GprofDataSource extends DataSource {
 
             boolean callPathSection = false;
             boolean parent = true;
-            Vector parents = new Vector();
+            Vector<LineData> parents = new Vector<LineData>();
             LineData self = null;
-            Vector children = new Vector();
+            Vector<LineData> children = new Vector<LineData>();
 
             fixLengths = true;
             linenumber = 1;
@@ -107,9 +112,6 @@ public class GprofDataSource extends DataSource {
                         if ((inputString.indexOf("index") == 0) && (inputString.indexOf("time") >= 0)
                                 && (inputString.indexOf("self") >= 0) && (inputString.indexOf("called") >= 0)
                                 && (inputString.indexOf("name") >= 0)) {
-                            // this line has the lengths of the fields.
-                            // we need this.
-//                            getFieldLengths(inputString);
                         } else if (inputString.charAt(0) == '[') {
                             self = getSelfLineData(inputString);
                             parent = false;
@@ -123,22 +125,22 @@ public class GprofDataSource extends DataSource {
 
                             functionProfile.setInclusive(0, self.d1 + self.d2);
                             functionProfile.setExclusive(0, self.d1);
-                            functionProfile.setNumCalls(self.i0);
+                            functionProfile.setNumCalls(self.d3);
 
-                            int numSubr = 0;
+                            double numSubr = 0;
                             for (int i = 0; i < children.size(); i++) {
-                                LineData lineDataChild = (LineData) children.get(i);
-                                numSubr += lineDataChild.i0;
+                                LineData lineDataChild = children.get(i);
+                                numSubr += lineDataChild.d3;
                             }
 
                             functionProfile.setNumSubr(numSubr);
-                            //functionProfile.setInclusivePerCall(0, (self.d1 + self.d2) / self.i0);
+                            //functionProfile.setInclusivePerCall(0, (self.d1 + self.d2) / self.d3);
 
                             for (int i = 0; i < parents.size(); i++) {
-                                LineData lineDataParent = (LineData) parents.elementAt(i);
+                                LineData lineDataParent = parents.elementAt(i);
                                 function = this.addFunction(lineDataParent.s0, 1);
                                 function.addGroup(addGroup("TAU_DEFAULT"));
-                                String s = lineDataParent.s0 + " => " + self.s0 + "  ";
+                                //String s = lineDataParent.s0 + " => " + self.s0 + "  ";
                                 callPathFunction = this.addFunction(lineDataParent.s0 + " => " + self.s0 + "  ", 1);
                                 callPathFunction.addGroup(addGroup("TAU_DEFAULT"));
                                 callPathFunction.addGroup(addGroup("TAU_CALLPATH"));
@@ -147,19 +149,19 @@ public class GprofDataSource extends DataSource {
                                 thread.addFunctionProfile(functionProfile);
                                 functionProfile.setInclusive(0, lineDataParent.d0 + lineDataParent.d1);
                                 functionProfile.setExclusive(0, lineDataParent.d0);
-                                functionProfile.setNumCalls(lineDataParent.i0);
+                                functionProfile.setNumCalls(lineDataParent.d3);
 
                                 //  functionProfile.setInclusivePerCall(0,
-                                //         (lineDataParent.d0 + lineDataParent.d1) / lineDataParent.i0);
+                                //         (lineDataParent.d0 + lineDataParent.d1) / lineDataParent.d3);
 
                             }
                             parents.clear();
 
                             for (int i = 0; i < children.size(); i++) {
-                                LineData lineDataChild = (LineData) children.elementAt(i);
+                                LineData lineDataChild = children.elementAt(i);
                                 function = this.addFunction(lineDataChild.s0, 1);
                                 function.addGroup(addGroup("TAU_DEFAULT"));
-                                String s = self.s0 + " => " + lineDataChild.s0 + "  ";
+                                //String s = self.s0 + " => " + lineDataChild.s0 + "  ";
                                 callPathFunction = this.addFunction(self.s0 + " => " + lineDataChild.s0 + "  ", 1);
                                 callPathFunction.addGroup(addGroup("TAU_DEFAULT"));
                                 callPathFunction.addGroup(addGroup("TAU_CALLPATH"));
@@ -168,26 +170,24 @@ public class GprofDataSource extends DataSource {
                                 thread.addFunctionProfile(functionProfile);
                                 functionProfile.setInclusive(0, lineDataChild.d0 + lineDataChild.d1);
                                 functionProfile.setExclusive(0, lineDataChild.d0);
-                                functionProfile.setNumCalls(lineDataChild.i0);
-                                //functionProfile.setInclusivePerCall(0,
-                                //        (lineDataChild.d0 + lineDataChild.d1) / lineDataChild.i0);
+                                functionProfile.setNumCalls(lineDataChild.d3);
                             }
                             children.clear();
                             parent = true;
                         } else if (inputString.charAt(length - 1) == ']') {
                             // check for cycle line
                             if (inputString.indexOf("<cycle") >= 0) {
-                                if (parent)
+                                if (parent) {
                                     parents.add(getParentChildLineData(inputString));
-                                else
+                                } else {
                                     children.add(getParentChildLineData(inputString));
+                                }
                             } else {
-                                if (parent)
+                                if (parent) {
                                     parents.add(getParentChildLineData(inputString));
-                                //parents.add(getParentLineData(inputString));
-                                else
+                                } else {
                                     children.add(getParentChildLineData(inputString));
-                                //children.add(getChildLineData(inputString));
+                                }
                             }
                         }
                     } else if (inputString.charAt(length - 1) == ']') {
@@ -213,38 +213,38 @@ public class GprofDataSource extends DataSource {
     //Gprof.dat string processing methods.
     //######
 
-//    private void getFieldLengths(String string) {
-//
-//        System.out.println("FIELD LENGTHS@!!!!");
-//
-//        /*
-//         * parse a line that looks like: index %time self childen
-//         * called+self name index ...or... index % time self children called
-//         * name index [xxxx] 100.0 xxxx.xx xxxxxxxx.xx xxxxxxx+xxxxxxx ssssss...
-//         */
-//        StringTokenizer st = new StringTokenizer(string, " \t\n\r");
-//        String index = st.nextToken();
-//        String percent = st.nextToken();
-//        if (percent.compareTo("%") == 0)
-//            percent += " " + st.nextToken();
-//        String self = st.nextToken();
-//        String descendants = st.nextToken();
-//        String called = st.nextToken();
-//        String name = st.nextToken();
-//        // this should be 0, left justified
-//        indexStart = string.indexOf(index);
-//        // this should be about 7, right justified
-//        percentStart = string.indexOf(percent);
-//        // this should be about 13, right justified
-//        selfStart = string.indexOf(percent) + percent.length() + 1;
-//        // this should be about 21, right justified
-//        descendantsStart = string.indexOf(self) + self.length() + 1;
-//        // this should be about 33, left justified
-//        calledStart = string.indexOf(descendants) + descendants.length() + 1;
-//        // this should be about 49, left justified
-//        nameStart = string.indexOf(name);
-//        return;
-//    }
+    //    private void getFieldLengths(String string) {
+    //
+    //        System.out.println("FIELD LENGTHS@!!!!");
+    //
+    //        /*
+    //         * parse a line that looks like: index %time self childen
+    //         * called+self name index ...or... index % time self children called
+    //         * name index [xxxx] 100.0 xxxx.xx xxxxxxxx.xx xxxxxxx+xxxxxxx ssssss...
+    //         */
+    //        StringTokenizer st = new StringTokenizer(string, " \t\n\r");
+    //        String index = st.nextToken();
+    //        String percent = st.nextToken();
+    //        if (percent.compareTo("%") == 0)
+    //            percent += " " + st.nextToken();
+    //        String self = st.nextToken();
+    //        String descendants = st.nextToken();
+    //        String called = st.nextToken();
+    //        String name = st.nextToken();
+    //        // this should be 0, left justified
+    //        indexStart = string.indexOf(index);
+    //        // this should be about 7, right justified
+    //        percentStart = string.indexOf(percent);
+    //        // this should be about 13, right justified
+    //        selfStart = string.indexOf(percent) + percent.length() + 1;
+    //        // this should be about 21, right justified
+    //        descendantsStart = string.indexOf(self) + self.length() + 1;
+    //        // this should be about 33, left justified
+    //        calledStart = string.indexOf(descendants) + descendants.length() + 1;
+    //        // this should be about 49, left justified
+    //        nameStart = string.indexOf(name);
+    //        return;
+    //    }
 
     private LineData getSelfLineData(String string) {
         LineData lineData = new LineData();
@@ -254,7 +254,7 @@ public class GprofDataSource extends DataSource {
         //the number of calls for the top level function (usually main).
         //Check the number of tokens to see if we are in this case. If so,
         //by default, we assume a number of calls value of 1.
-        int numberOfTokens = st.countTokens();
+        //int numberOfTokens = st.countTokens();
 
         // Skip the first token.
         // Entries are numbered with consecutive integers.
@@ -302,7 +302,7 @@ public class GprofDataSource extends DataSource {
             ALSO BAD:       0.27    0.00 1135396/4373228     hypre_BoxGetStrideSize [56]
 
             */
-//            calledStart = string.indexOf(fixer) + fixer.length() + 1;
+            //            calledStart = string.indexOf(fixer) + fixer.length() + 1;
             fixLengths = false;
         }
 
@@ -314,8 +314,12 @@ public class GprofDataSource extends DataSource {
 
         boolean hasCalls;
         String str = st.nextToken();
+//        if(str.contains("CSRMatrix2")){
+//        	System.out.println("Hey!");
+//        }
         try {
-            int foo = Integer.parseInt(str);
+            //int foo = 
+            	Integer.parseInt(str);
             hasCalls = true;
         } catch (NumberFormatException nfe) {
             hasCalls = false;
@@ -323,14 +327,14 @@ public class GprofDataSource extends DataSource {
 
         if (!hasCalls) {
             // if the number of calls is absent, assume 1.
-            lineData.i0 = 1;
+            lineData.d3 = 1;
             lineData.s0 = str;
         } else {
             if (str.indexOf("+") < 0) {
                 StringTokenizer st2 = new StringTokenizer(str, "+");
-                lineData.i0 = Integer.parseInt(st2.nextToken());
+                lineData.d3 = Double.parseDouble(st2.nextToken());
                 // do this?
-                // lineData.i0 += Integer.parseInt(st2.nextToken());
+                // lineData.d3 += Integer.parseInt(st2.nextToken());
             }
             lineData.s0 = st.nextToken(); //Name
         }
@@ -345,13 +349,13 @@ public class GprofDataSource extends DataSource {
     }
 
     private LineData getParentChildLineData(String string) {
-        //System.out.println("string = " + string);
-        
+
         // first, check to see if we are missing the index, %time, and self values.
         String tmp = string.trim();
         boolean missing = false;
-        if (tmp.length() < string.length() - 20)
-        	missing = true;
+        if (tmp.length() < string.length() - 20) {
+            missing = true;
+        }
 
         LineData lineData = new LineData();
         StringTokenizer st = new StringTokenizer(string, " \t\n\r");
@@ -396,138 +400,123 @@ public class GprofDataSource extends DataSource {
 
         if (missing) {
             lineData.d0 = 0.0;
-            lineData.d1 = 0.0;        	
+            lineData.d1 = 0.0;
         } else {
-	        tmpStr = st.nextToken().trim();// string.substring(selfStart, descendantsStart).trim();
-	        //String tmpStr = st.nextToken();
-	        if (tmpStr.length() > 0) {
-	            lineData.d0 = 1000000.0 * Double.parseDouble(tmpStr);
-	        } else {
-	            lineData.d0 = 0.0;
-	        }
-	        
-	//        System.err.println("Error parsing file: " + currentFile + ", line: " + linenumber);
-	//        System.err.println("selfStart: " + selfStart);
-	//        System.err.println("descendantsStart: " + descendantsStart);
-	//        System.err.println("calledStart: " + calledStart);
-	
-	        try {
-	            tmpStr = st.nextToken().trim();//string.substring(descendantsStart, calledStart).trim();
-	            //tmpStr = st.nextToken();
-	            if (tmpStr.length() > 0) {
-	                lineData.d1 = 1000000.0 * Double.parseDouble(tmpStr);
-	            } else {
-	                lineData.d1 = 0.0;
-	            }
-	        } catch (Exception e) {
-	            System.err.println("Error parsing file: " + currentFile + ", line: " + linenumber);
-	//            System.err.println("selfStart: " + selfStart);
-	//            System.err.println("descendantsStart: " + descendantsStart);
-	//            System.err.println("calledStart: " + calledStart);
-	            System.err.println(e.getMessage());
-	            throw new DataSourceException(e, currentFile);
-	        }
+            tmpStr = st.nextToken().trim();// string.substring(selfStart, descendantsStart).trim();
+            //String tmpStr = st.nextToken();
+            if (tmpStr.length() > 0) {
+                lineData.d0 = 1000000.0 * Double.parseDouble(tmpStr);
+            } else {
+                lineData.d0 = 0.0;
+            }
+
+            //        System.err.println("Error parsing file: " + currentFile + ", line: " + linenumber);
+            //        System.err.println("selfStart: " + selfStart);
+            //        System.err.println("descendantsStart: " + descendantsStart);
+            //        System.err.println("calledStart: " + calledStart);
+
+            try {
+                tmpStr = st.nextToken().trim();//string.substring(descendantsStart, calledStart).trim();
+                //tmpStr = st.nextToken();
+                if (tmpStr.length() > 0) {
+                    lineData.d1 = 1000000.0 * Double.parseDouble(tmpStr);
+                } else {
+                    lineData.d1 = 0.0;
+                }
+            } catch (Exception e) {
+                System.err.println("Error parsing file: " + currentFile + ", line: " + linenumber);
+                //            System.err.println("selfStart: " + selfStart);
+                //            System.err.println("descendantsStart: " + descendantsStart);
+                //            System.err.println("calledStart: " + calledStart);
+                System.err.println(e.getMessage());
+                throw new DataSourceException(e, currentFile);
+            }
         }
 
         // check if the counts 'spill' into the name field.
-//        String spillTest = string.substring(calledStart, string.length()).trim();
-//        int space = spillTest.indexOf(" ");
-//        if (space > (nameStart - calledStart)) {
-//            tmpStr = spillTest.substring(0, space).trim();
-//        } else {
-//            tmpStr = string.substring(calledStart, nameStart).trim();
-//        }
-        
-        tmpStr=st.nextToken().trim();
-        
+        //        String spillTest = string.substring(calledStart, string.length()).trim();
+        //        int space = spillTest.indexOf(" ");
+        //        if (space > (nameStart - calledStart)) {
+        //            tmpStr = spillTest.substring(0, space).trim();
+        //        } else {
+        //            tmpStr = string.substring(calledStart, nameStart).trim();
+        //        }
+
+        tmpStr = st.nextToken().trim();
+
         // check for a ratio
         if (tmpStr.indexOf("/") >= 0) {
             StringTokenizer st2 = new StringTokenizer(tmpStr, "/");
+            
+            
+            try{
+            
             // the number of times self was called from parent
-            lineData.i0 = Integer.parseInt(st2.nextToken());
+            lineData.d3 = Double.parseDouble(st2.nextToken());
             // the total number of nonrecursive calls to self from all
-            // its parents
-            lineData.i1 = Integer.parseInt(st2.nextToken());
+            // its parents (This is currently not used, and may be bigger than an int!)
+            //lineData.i1 = Integer.parseInt();
+            st2.nextToken();
+            }catch( NumberFormatException e){
+            	e.printStackTrace();
+            }
         } else {
-            lineData.i0 = Integer.parseInt(tmpStr);
-            lineData.i1 = lineData.i0;
+        	if(tmpStr.indexOf('+')>=0){
+        		tmpStr=tmpStr.split("\\+")[0];
+        	}
+            lineData.d3 = Double.parseDouble(tmpStr);
+            lineData.d4 = lineData.d3;
         }
 
         // the rest is the name
-//        if (space > (nameStart - calledStart)) {
-//            int end = spillTest.lastIndexOf("[") - 1;
-//            lineData.s0 = spillTest.substring(space, end).trim();
-//        } else {
-//            int end = string.lastIndexOf("[") - 1;
-//            lineData.s0 = string.substring(nameStart, end).trim();
-//        }
-        
-        lineData.s0 = fix(st.nextToken());
-        return lineData;
-    }
+        //        if (space > (nameStart - calledStart)) {
+        //            int end = spillTest.lastIndexOf("[") - 1;
+        //            lineData.s0 = spillTest.substring(space, end).trim();
+        //        } else {
+        //            int end = string.lastIndexOf("[") - 1;
+        //            lineData.s0 = string.substring(nameStart, end).trim();
+        //        }
 
-    /*
-     * private LineData getParentChildLineData(String string){ LineData lineData =
-     * new LineData(); try{ StringTokenizer st1 = new StringTokenizer(string, "
-     * \t\n\r");
-     *  // get the estimate of the amount of time spent directly // in child
-     * when it was called from self // ...or... // get the estimate of the
-     * amount of time spent in self when // it was called from parent
-     * lineData.d0 = Double.parseDouble(st1.nextToken()); // get the estimate of
-     * the amount of time spent in // subroutines of child when child was called
-     * from self. // The sum of the self and children fields is an estimate //
-     * of the total time spent in calls to child from self // ...or... // get
-     * the estimate of the amount of time spent in subroutines // of self when
-     * self was called from parent. The sum of the // self and children fields
-     * is an estimate of the amount of // time spent within calls to self from
-     * parent. lineData.d1 = 1000.0 * Double.parseDouble(st1.nextToken());
-     *  // for cycles, there is no ratio. To check for cycles, check // to see
-     * if there is a ratio. String tmpStr = st1.nextToken(); if
-     * (tmpStr.indexOf("/") >= 0) { StringTokenizer st2 = new
-     * StringTokenizer(tmpStr, "/"); // This ratio is used to determine how much
-     * of self and // children time gets credited to parent. // ...or... // the
-     * number of times self was called from parent lineData.i0 =
-     * Integer.parseInt(st2.nextToken()); // get the number of calls to child
-     * from self // ...or... // get the total number of nonrecursive calls to
-     * report. lineData.i1 = Integer.parseInt(st2.nextToken()); } else {
-     * lineData.i0 = Integer.parseInt(tmpStr); lineData.i1 =
-     * Integer.parseInt(tmpStr); }
-     * 
-     * lineData.s0 = st1.nextToken(); //Name while (st1.hasMoreTokens()) {
-     * String tmp = st1.nextToken(); if ((tmp.indexOf("[") != 0) &&
-     * (!tmp.endsWith("]"))) lineData.s0 += " " + tmp; //Name } }
-     * catch(Exception e){ System.out.println("***\n" + string + "\n***");
-     * e.printStackTrace(); UtilFncs.systemError(e, null, "GOS03"); } return
-     * lineData; }
-     */
-
-    private LineData getSummaryLineData(String string) {
-        LineData lineData = new LineData();
-        StringTokenizer st = new StringTokenizer(string, " \t\n\r");
-
-        lineData.d0 = Double.parseDouble(st.nextToken());
-        lineData.d1 = 1000.0 * Double.parseDouble(st.nextToken());
-        lineData.d2 = 1000.0 * Double.parseDouble(st.nextToken());
-        if (st.countTokens() > 5) {
-            lineData.i0 = Integer.parseInt(st.nextToken());
-            lineData.d3 = Double.parseDouble(st.nextToken());
-            lineData.d4 = Double.parseDouble(st.nextToken());
-        } else {
-            lineData.i0 = 1;
-            lineData.d3 = lineData.d2;
-            lineData.d4 = lineData.d2;
-        }
-
-        lineData.s0 = st.nextToken(); //Name
+        String eventName = "";
         while (st.hasMoreTokens()) {
-            String tmp = st.nextToken();
-            if ((tmp.indexOf("[") != 0) && (!tmp.endsWith("]")))
-                lineData.s0 += " " + tmp; //Name
+            // skip last token
+            String token = st.nextToken();
+            if (st.hasMoreTokens()) {
+                eventName = eventName + " " + token;
+            }
         }
-        lineData.s0 = fix(lineData.s0);
+        eventName = eventName.trim();
+        lineData.s0 = eventName;
         return lineData;
     }
 
+//    private LineData getSummaryLineData(String string) {
+//        LineData lineData = new LineData();
+//        StringTokenizer st = new StringTokenizer(string, " \t\n\r");
+//
+//        lineData.d0 = Double.parseDouble(st.nextToken());
+//        lineData.d1 = 1000.0 * Double.parseDouble(st.nextToken());
+//        lineData.d2 = 1000.0 * Double.parseDouble(st.nextToken());
+//        if (st.countTokens() > 5) {
+//            lineData.d3 = Integer.parseInt(st.nextToken());
+//            lineData.d3 = Double.parseDouble(st.nextToken());
+//            lineData.d4 = Double.parseDouble(st.nextToken());
+//        } else {
+//            lineData.d3 = 1;
+//            lineData.d3 = lineData.d2;
+//            lineData.d4 = lineData.d2;
+//        }
+//
+//        lineData.s0 = st.nextToken(); //Name
+//        while (st.hasMoreTokens()) {
+//            String tmp = st.nextToken();
+//            if ((tmp.indexOf("[") != 0) && (!tmp.endsWith("]")))
+//                lineData.s0 += " " + tmp; //Name
+//        }
+//        lineData.s0 = fix(lineData.s0);
+//        return lineData;
+//    }
+
     /*
      * when C and Fortran code are mixed, the C routines have to be mapped to
      * either .function or function_ . Strip the leading period or trailing
