@@ -1,6 +1,6 @@
-from client import ScriptFacade
-from common import TransformationType
-from common import AnalysisType
+from edu.uoregon.tau.perfexplorer.client import ScriptFacade
+from edu.uoregon.tau.perfexplorer.common import TransformationType
+from edu.uoregon.tau.perfexplorer.common import AnalysisType
 
 print "--------------- JPython test script start ------------"
 
