@@ -1,10 +1,21 @@
 package edu.uoregon.tau.perfexplorer.client;
 
-import javax.swing.*;
-import java.awt.event.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.KeyEvent;
+
+import javax.swing.JMenu;
+import javax.swing.JMenuBar;
+import javax.swing.JMenuItem;
+import javax.swing.KeyStroke;
 
 public class PerfExplorerChartJMenuBar extends JMenuBar {
 
+	/**
+	 * 
+	 */
+	private static final long serialVersionUID = -6359699047764668001L;
+
 	public PerfExplorerChartJMenuBar(ActionListener listener) {
 		super();
 		createFileMenu(listener);
