@@ -1,8 +1,12 @@
 package edu.uoregon.tau.perfexplorer.client;
 
-import edu.uoregon.tau.perfexplorer.common.*;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
 
 public class PerfExplorerModel extends RMIPerfExplorerModel {
+	/**
+	 * 
+	 */
+	private static final long serialVersionUID = -6644066993143334180L;
 	private static PerfExplorerModel theModel = null;
 
 	public static PerfExplorerModel getModel() {
@@ -16,4 +20,8 @@ public class PerfExplorerModel extends RMIPerfExplorerModel {
 		return new RMIPerfExplorerModel(theModel);
 	}
 
+
+
+
+
 }
