@@ -2,7 +2,7 @@
  * Created on Mar 16, 2005
  *
  */
-package clustering;
+package edu.uoregon.tau.perfexplorer.clustering;
 
 
  /**
@@ -11,7 +11,7 @@ package clustering;
   * is a diagonal matrix, and only the lower-left quadrants below the
   * diagonal are used.
   *
-  * <P>CVS $Id: DistanceMatrix.java,v 1.3 2007/01/23 22:57:01 khuck Exp $</P>
+  * <P>CVS $Id: DistanceMatrix.java,v 1.7 2009/11/18 10:17:32 khuck Exp $</P>
   * @author khuck
   * @version 0.1
   * @since   0.1
@@ -21,6 +21,7 @@ public class DistanceMatrix {
 
     protected int dimension = 0;
     protected double[][] distances = null;
+    protected double[] weights = null;
     
     /**
     * Default constructor - requires a dimension parameter.
@@ -30,10 +31,12 @@ public class DistanceMatrix {
     public DistanceMatrix(int dimension) {
         this.dimension = dimension;
         this.distances = new double[dimension][dimension];
+        this.weights = new double[dimension];
         for (int i = 0; i < dimension ; i++) {
             for (int j = i+1; j < dimension ; j++) {
-                this.distances[i][j] = 1.0;
+                this.distances[i][j] = 0.0;
             }
+            weights[i] = 1.0;
         }
     }
     
@@ -45,10 +48,12 @@ public class DistanceMatrix {
     public DistanceMatrix(DistanceMatrix distances) {
         this.dimension = distances.dimension;
         this.distances = new double[dimension][dimension];
+        this.weights = new double[dimension];
         for (int i = 0; i < dimension ; i++) {
             for (int j = 0; j < dimension ; j++) {
                 this.distances[i][j] = distances.distances[i][j];
             }
+            weights[i] = distances.weights[i];
         }
     }
     
@@ -79,9 +84,10 @@ public class DistanceMatrix {
     * @param data
     */
     public void solveManhattanDistances(RawDataInterface data) {
+		System.out.println("Using Manhattan distances...");
         for (int i = 0; i < dimension ; i++) {
             for (int j = 0; j < i ; j++) {
-                distances[i][j] = data.getManhattanDistance(i,j);
+                distances[i][j] = distances[j][i] = data.getManhattanDistance(i,j);
             }
         }
     }
@@ -93,9 +99,10 @@ public class DistanceMatrix {
     * @param data
     */
     public void solveCartesianDistances(RawDataInterface data) {
+		System.out.println("Using Cartesian distances...");
         for (int i = 0; i < dimension ; i++) {
             for (int j = 0; j < i ; j++) {
-                distances[i][j] = data.getCartesianDistance(i,j);
+                distances[i][j] = distances[j][i] = data.getCartesianDistance(i,j);
             }
         }
     }
@@ -106,7 +113,7 @@ public class DistanceMatrix {
      * @return
      */
     public String toString() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
         for (int i = 0; i < dimension ; i++) {
             for (int j = 0; j < dimension ; j++) {
                 buf.append(distances[i][j] + " ");
@@ -126,19 +133,14 @@ public class DistanceMatrix {
     * @param y
     */
     public void mergeDistances (int x, int y) {
-        for (int i = 0 ; i < x ; i++) {
-            int z = dimension - (i+1);
-            distances[x][i] = ((distances[x][i] * distances[x][z]) + 
-                               (distances[y][i] * distances[y][z])) / 
-                              (distances[x][z] + distances[y][z]);
-            distances[x][z] = distances[x][z] + distances[y][z];
-        }
-        for (int i = y+1 ; i < dimension ; i++) {
-            int z = dimension - (i+1);
-            distances[i][x] = ((distances[i][x] * distances[z][x]) + 
-                               (distances[i][y] * distances[z][y])) / 
-                              (distances[z][x] + distances[z][y]);
-            distances[z][x] = distances[z][x] + distances[z][y];
+    	// merge the horizontal distances
+        for (int i = 0 ; i < dimension ; i++) {
+            double firstWeight = weights[x];
+            double secondWeight = weights[y];
+            double totalWeight = firstWeight + secondWeight;
+            distances[x][i] = ((distances[x][i] * firstWeight) + 
+                    (distances[y][i] * firstWeight)) / totalWeight;
+            weights[x] = totalWeight;
         }
     }
 }
