@@ -210,7 +210,7 @@ void InitSnapshot(){
 		s_out+="/";
 	}
 	char filename [32];
-	sprintf(filename,"profile.xml");//%d.0.%d",((*it).second).nodeToken,((*it).second).threadToken);
+	snprintf(filename, sizeof(filename), "profile.xml");//%d.0.%d",((*it).second).nodeToken,((*it).second).threadToken);
 	s_prefix=s_out+filename;
 	snapshot.open(s_prefix.c_str());//, ofstream::app|ofstream::out
 	snapshot.precision(16);
@@ -550,7 +550,7 @@ void PrintSnapshot(double time, Thread &finalizer, bool printProfile){//map<int,
 	}
 	//for (map< int,Thread >:: iterator it = mainmap.begin(); it != mainmap.end(); it++){
 		char filename [32];
-		sprintf(filename,"profile.%d.0.%d",finalizer.nodeToken,finalizer.threadToken);//((*it).second)
+		snprintf(filename, sizeof(filename), "profile.%d.0.%d",finalizer.nodeToken,finalizer.threadToken);//((*it).second)
 		s_prefix=s_out+filename;
 		profile.open(s_prefix.c_str());
 		profile.precision(16);
@@ -643,7 +643,7 @@ void PrintSnapshot(double time, Thread &finalizer, bool printProfile){//map<int,
 			}
 			//for (map< int,Thread >:: iterator it = mainmap.begin(); it != mainmap.end(); it++)
 				memset(filename, 0, 32);
-				sprintf(filename,"profile.%d.0.%d",finalizer.nodeToken,finalizer.threadToken);
+				snprintf(filename, sizeof(filename), "profile.%d.0.%d",finalizer.nodeToken,finalizer.threadToken);
 				s_prefix=s_out+filename;
 				profile.open(s_prefix.c_str());
 				profile.precision(16);
